function bintab = binexp2bintab3(binexp,variable,tab)
% bintab = binexp2bintab3(binexp,variable)
% Berechnung der Funktionstabelle bintab fr einen als String binexp
% gegebenen schaltalgebraischen Ausdruck mit den Variablen variable.
% Der Ausdruck darf in MATLAB-Syntax alle binren Operatoren enthalten
% und zustzlich die GF(2)-Verknpfungen * und +
%
% binexp   :  String fr den schaltalgebraischen Ausdruck.
%             Es kann auch ein CellArray von Strings angegeben werden,
%             dann erfolgt die Berechnung fr alle Ausdrcke.
%             Beispiel:  '(a1|b) & (~a2|c) & b|c'
%
% variable :  Cell array mit den Variablen als Strings
%             Folgende Variablennamen sind nicht erlaubt:
%             iiiii,jjjjj,kkkkk,lllll,nnnnn,binexp,binexpp,bintab,...
%             eval,sprintf,variable sowie alle Schlsselwrter von MATLAB 
%             wie for, end, ...
%             Beispiel:  {'a1'; 'a2'; 'b'; 'c'}
%
% tab      :  wenn tab mit (beliebigem Wert) angegeben, dann
%             Ausgabe der Funktionstabelle bintab als String
%             andernfalls als Binrvektor.
% tab = 2  :  Die Funktionstabelle wird in einem Fenster ausgegeben
%             sofern die Grsse es zulsst.
%
% Beispiel:  v = {'a1'; 'b'; 'c'}; binexp2bintab3(rmbasis(v),v)
%
% Siehe auch binexp2bintab

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<2  variable = {'a';'b';'c';'d'}; end
if nargin<1  binexp = '(a&c) | (~b&d&~c)'; end

% Konstante
% =========
nnnnn = length(variable);                 % Anzahl der Variablen

% Berechnung der Funktionstabellen
% ================================
if ~iscell(binexp)
   binexp = ['mod(' binexp ',2)'];
   for jjjjj=0:(2^nnnnn-1)                % Schleife ber alle ...
      kkkkk = num2binvec(jjjjj,nnnnn);    % Input-Bitmuster             
      for iiiii=1:nnnnn                   % alle Variablen setzen
         eval(sprintf('%s=kkkkk(%d);',variable{iiiii},iiiii));
      end
      try
         eval([binexp ';'])               % Ausdruck evaluieren
         bintab(jjjjj+1) = ans;           % Resultat in Tabelle
      catch
         bintab(jjjjj+1) = NaN;           % Ausdruck liefert NaN
      end
   end
else
   for lllll=1:length(binexp)
      binexpp = ['mod(' binexp{lllll} ',2)'];
      for jjjjj=0:(2^nnnnn-1)             % Schleife ber alle ...
         kkkkk = num2binvec(jjjjj,nnnnn); % Input-Bitmuster             
         for iiiii=1:nnnnn                % alle Variablen setzen
            eval(sprintf('%s=kkkkk(%d);',variable{iiiii},iiiii));
         end
         try
            eval([binexpp ';'])           % Ausdruck evaluieren
            bintab(jjjjj+1,lllll) = ans;  % Resultat in Tabelle
         catch
            bintab(jjjjj+1,lllll) = NaN;  % Ausdruck liefert NaN
         end
      end
   end
end

if nargin<3 & nargin<3 return; else tab = 2; end

% Ausgabe als Stringarray
% =======================
if iscell(binexp)  nf = length(binexp); else nf = 1; end
l  = 2*(nnnnn + nf + 2);                  % Zeilenlnge bei Textausgabe
b  = '01';                                % Bits als Strings
for v=1:nnnnn
   z = '';
   for w=1:v-1  z = [z '| ']; end
   vv = variable(v);
   z = [z  cat(1,vv{:})];
   for w=length(z)+1:l  z = [z  ' ']; end
   t(v,:) = z;
end
for k=0:(2^nnnnn-1)
   bin = binmatout(num2binvec(k,nnnnn));
   z = reshape([bin(1:end-1); blanks(nnnnn)],1,2*nnnnn);
   if nf==1
      z = [z  '|  '  b(bintab(k+1)+1)];
   else
      z = [z  '|  '];
      for j=1:nf
         z = [z  ' '  b(bintab(k+1,j)+1)];
      end
   end
   for w=length(z)+1:l  z = [z  ' ']; end
   t(k+nnnnn+1,:) = z;
end

bintab = t;

% if tab==2 & nnnnn<6
%    n = nnnnn;
%    m = 2^n;
%    h = n + m;
%    b = 2*(n + nf + 2);
%    if n<5  fsiz = 21; else fsiz = 13; end
%    fh = fullscreen('Funktionstabelle','w',0,min([0 b/1500 0 h/550]*(fsiz+5),1));
%    axes('Position',[0 0 1 1]); 
%    text(0.5,0,t,'FontName','FixedWidth','FontSize',fsiz); 
%    axis([0 2*n+5  [-1 +1]*2^(n-1)]); axis off
% end

if iscell(binexp) & length(binexp)>1
   Funktionstabellen = t;
   result2fig(Funktionstabellen)
else
   Funktionstabelle = t;
   result2fig(Funktionstabelle)
end
