function knf = binexp2knfexp(expr,variable,opsymb)
% knf = binexp2knfexp(expr,variable,opsymb)
% Wandlung eines schaltalgebraischen Ausdrucks mit den 
% Variablen variable in die Konjunktive Normalform.
%
% expr     :  String mit dem schaltalgebraischen Ausdruck in
%             MATLAB-Syntax
% variable :  Cell array mit den Variablen als Strings
%             Default = {'a';'b';'c';'d'}
% opsymb   :  String mit drei Symbolen fr folgende Operationen:
%             Infixsymbol fr die Und-Verknpfung
%             Infixsymbol fr die Oder-Verknpfung
%             Prfixsymbol fr die Negation
%             Default = '&|~' fr MATLAB-Syntax
%             Diese Symbole gelten fr den Eingabe-Ausdruck expr 
%             und fr den Ausgabe-Ausdruck knf.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3  opsymb = '&|~'; end
if nargin<2  variable = {'a';'b';'c';'d'}; end
if nargin<1  expr = bintab2dnfexp(randb(16)); end

% Berechnung der Funktionstabelle
% ===============================
xx = strrep(strrep(strrep(expr,opsymb(1),'&'),opsymb(2),'|'),opsymb(3),'~');
ftab = binexp2bintab3(xx,variable);

% Konstanten
% ==========
nn = length(ftab);                    % Lnge der Funktionstabelle
n  = round(log2(nn));                 % Anzahl der Variablen

% Transformation
% ==============
x = '(';
for k=1:nn
   if ~ftab(k)
      bv = num2binvec(k-1,n);
      for j=1:n
         if bv(j)  x = [x  opsymb(3)]; end
         x = [x  variable{j}  opsymb(2)];
      end
      x = [x(1:end-1)  ') '  opsymb(1)  ' ('];
   end
end
x = x(1:end-4);
knf = x;

% Ausgabe
% =======
if ~nargout
   KonjunktiveNormalForm = knf;
   result2fig(KonjunktiveNormalForm)
end

