function rmf = binexp2rmexp(expr,variable,opsymb)
% rmf = binexp2rmexp(expr,variable,opsymb)
% Wandlung eines schaltalgebraischen Ausdrucks mit den 
% Variablen variable in die Disjunktive Normalform.
%
% expr     :  String mit dem schaltalgebraischen Ausdruck in
%             MATLAB-Syntax
% variable :  Cell array mit den Variablen als Strings
%             Default = {'a';'b';'c';'d'}
% opsymb   :  String mit fnf Symbolen fr folgende Operationen:
%             Infixsymbol fr die Und-Verknpfung in expr
%             Infixsymbol fr die Oder-Verknpfung in expr
%             Prfixsymbol fr die Negation in expr
%             Infixsymbol fr die Multiplikation in rmf
%             Infixsymbol fr die Addition in rmf
%             Default = '&|~*+' fr MATLAB-Syntax
%             Der Eingabe-Ausdruck darf neben den hier angegebenen
%             Operationen Und und Oder weitere definierte Operationen
%             enthalten.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3  opsymb = '&|~*+'; end
if nargin<2  variable = {'a';'b';'c';'d'}; end
if nargin<1  expr = bintab2dnfexp(randb(16)); end

% Berechnung der Funktionstabelle
% ===============================
xx = strrep(strrep(strrep(expr,opsymb(1),'&'),opsymb(2),'|'),opsymb(3),'~');
ftab = binexp2bintab3(xx,reverse(variable));

% Reed-Muller-Form
% ================
rmf = bintab2rmexp(ftab,variable,opsymb(4:5));

% Ausgabe
% =======
if ~nargout
   ReedMullerForm = rmf;
   result2fig(ReedMullerForm)
end