function x = bintab2dnfexp(ftab,variable,opsymb)
% x = bintab2dnfexp(ftab,variable,opsymb)
% Wandlung der Funktionstabelle einer Schaltfunktion mit den Variablen 
% variable in die Disjunktive Normalform.
%
% ftab     :  Funktionstabelle von 2^length(variable) binren Werten
% variable :  Cell array mit den Variablen als Strings
%             Default = {'a';'b';'c';'d'}
% opsymb   :  String mit drei Symbolen fr folgende Operationen:
%             Infixsymbol fr die Intra-Term-Verknpfung
%             Infixsymbol fr die Verknpfung von Termen
%             Prfixsymbol fr Negation
%             Default = '&|~' fr MATLAB-Syntax
% Die Terme werden geklammert.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3  opsymb = '&|~'; end
if nargin<2  variable = {'a';'b';'c';'d'}; end
if nargin<1  ftab = randb(16); end

% Konstanten
% ==========
nn = length(ftab);                    % Lnge der Funktionstabelle
n  = round(log2(nn));                 % Anzahl der Variablen

% Sonderflle Eins- und Nullpunktfunktion
% =======================================
if all(ftab)   x = '1'; return; end
if all(~ftab)  x = '0'; return; end

% Transformation
% ==============
x = '(';
for k=1:nn
   if ftab(k)
      bv = num2binvec(k-1,n);
      for j=1:n
         if ~bv(j)  x = [x  opsymb(3)]; end
         x = [x  variable{j}  opsymb(1)];
      end
      x = [x(1:end-1)  ') '  opsymb(2)  ' ('];
   end
end
x = x(1:end-4);

% Ausgabe
% =======
if ~nargout
   DisjunktiveNormalForm = x;
   result2fig(DisjunktiveNormalForm)
end

