function x = bintab2rmexp(ftab,variable,opsymb)
% x = bintab2rmexp(ftab,variable,opsymb)
% Wandlung der Funktionstabelle einer Schaltfunktion mit den Variablen 
% variable in die Reed-Muller-Form.
%
% ftab     :  Funktionstabelle von 2^length(variable) binren Werten
% variable :  Cell array mit den Variablen als Strings
%             Default = {'a';'b';'c';'d'}
% opsymb   :  String mit zwei Symbolen fr folgende Operationen:
%             Infixsymbol fr die Intra-Term-Verknpfung
%             Infixsymbol fr die Verknpfung von Termen
%             Default = '*+'

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3  opsymb = '*+'; end
if nargin<2  variable = {'a';'b';'c';'d'}; end
if nargin<1  ftab = randb(16); end

% Konstanten
% ==========
nn = length(ftab);                 % Lnge der Funktionstabelle
n  = round(log2(nn));              % Anzahl der Variablen

% Transformation in die Reed-Muller-Form
% ======================================
r = bintab2rm(ftab);

% Transformation
% ==============
if r(1)  
   x = '1 + ';
else
   x = '';
end
for k=2:length(r)                  % alle Terme der Reed-Muller-Form
   if r(k)                         % wenn Term vorhanden
      bvk = num2binvec(k-1,n);     % bvk zeigt welche Variablem im Term vorkommen
      for j=1:n
         if bvk(n-j+1)             % wenn Variable vorkommt dann einfgen
            x = [x sprintf('%s*',variable{j})];
         end
      end
      x = [x(1:end-1) ' + '];      % Verknpfung mit nchstem Term
   end
end
if length(x)>0 
   x = x(1:end-3);                 % letzte Verknpfung streichen
else
   x = '0';
end
