function bintreegraph(x,variable)
% bintreegraph(x,variable)
% Darstellung des in x reprsentierten Binrbaums.
% Optional knnen die Variablennamen eingetragen werden.
% variable ist ein CellArray mit den Strings der Variablennamen.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1   x = reducebintree(bintab2bintree(binexp2bintab)); end

% Konstanten
% ==========
r = 0.5;                               % Radius fr Knoten
col = 0.8*[1 1 1];                     % Fllfarbe fr Knoten
lc  = 'rg';                            % Farben der Kanten fr 0 bzw. 1
lw  = 1.5;                             % Dicke der Kanten
n   = treedepth(x);                    % Tiefe des Baumes = Anzahl der Variablen

% Graphik
% =======
%dy = (2^n + 3)/(n+2)/1.333;            % vertikaler Abstand der Ebenen 
dy = (2^(n+0.7)+2)/(n+2)/1.333;        % vertikaler Abstand der Ebenen 
fsiz = round(400*2^-n);                % FontSize
fh = fullscreen('BinTree');            % Fenster
axes('Position',[0 0 1 1])             % Achsen
drawtree(x,[0 0; 2^(n-2) dy],r,col,lw) % Graphen zeichnen
set(gca,'XLim',[-1.05 +1]*2^(n-0.9))   % Achsenskalierung
if n<3  axis(2*axis+[0 0 1 1]); end    % Achsenskalierung
axis square
axis off
if nargin==2
   for k=0:length(variable)-1
      text(-2^(n-0.9),-k*dy,variable{k+1},'FontSize',10+60/n,'FontUnits','normalized')
   end
   ss = get(0,'screensize');
   pp = min(ss(3),80*2^length(variable));
   if length(variable)<4  set(fh,'Position',[3 30 pp pp]); end
end

%-------------------------------------------------------------------------------

% Lokale Funktion zum rekursiven Zeichnen eines Binrbaumes
% =========================================================
function  drawtree(x,pos,r,col,lw)
a  = atan(pos(2,1)/pos(2,2));
dx = r*sin(a);
dy = r*cos(a);
if length(x)>1
   x2 = x{2};
   if iscell(x2) | x2==0 | x2==1
      drawccir(pos(1,:),r,col);
      line(pos(1,1)-[dx pos(2,1)],pos(1,2)-[dy pos(2,2)],'Color','r','LineWidth',lw)
      line(pos(1,1)+[dx pos(2,1)],pos(1,2)-[dy pos(2,2)],'Color','g','LineWidth',lw)
      drawtree(x{1},[pos(1,1)-pos(2,1),pos(1,2)-pos(2,2);pos(2,1)/2,pos(2,2)],r,col,lw)
      drawtree(x{2},[pos(1,1)+pos(2,1),pos(1,2)-pos(2,2);pos(2,1)/2,pos(2,2)],r,col,lw)
   else
      line(pos(1,1)-[0 0],pos(1,2)-[0 pos(2,2)],'Color','k','LineWidth',lw)
      drawtree(x{1},[pos(1,1),pos(1,2)-pos(2,2);pos(2,1)/2,pos(2,2)],r,col,lw)
   end      
else
   drawsq(pos(1,:)-r,2*r,col,num2str(x),10+80*2^(pos(1,2)/4));
end
