function [k_b,k_b1] = complements(z,b,n,m)
% [k_b,k_b1] = complements(z,b,n,m)
% berechnet die Komplemente der Zahl z
% z    :  darzustellende Zahl
% b    :  Basis der Zahlendarstellung (natrliche Zahl>1)
% n    :  Anzahl der Vorkommastellen
% m    :  Anzahl der Nachkommastellen
% k_b  :  b-Komplement
% k_b1 :  (b-1)-Komplement

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  z = 728; end      % irgendeine Zahl < b^n
if nargin<2  b = 10;  end      % Basis des Stellensystems
if nargin<3  n = 5 ;  end      % Anzahl der Vorkommastellen
if nargin<4  m = 0;   end      % Anzahl der Nachkommastellen

% Komplemente
% ===========
bk   = b^n - z                 % das b-Komplement
b1k  = b^n - b^-m - z          % das (b-1)-Komplement
k_b  = num2basstr2(bk,b,n,m)   % das b-Komplement in Darstellung zur Basis b
k_b1 = num2basstr2(b1k,b,n,m)  % das (b-1)-Komplement in Darstellung zur Basis b

% Darstellung
% ===========
sz = num2basstr2(z,b,n,m);  nz = length(sz);
sb = num2str(b);    nb = length(sb);
s1 = num2str(b-1);  n1 = length(s1);
sn = num2str(n);    nn = length(sn);
sm = num2str(m);    nm = length(sm);
mx = max([nb nn nm]);

Komplemente = {
[blanks(nb)  'Dezimalzahl:  ' num2str(z)];
[blanks(nb)  '      Basis:  '  blanks(mx-nb)  sb];
[blanks(nb)  '  #Vorkomma:  '  blanks(mx-nn)  sn];
[blanks(nb)  ' #Nachkomma:  '  blanks(mx-nm)  sm];
[blanks(nb)  '       Zahl:  ' sz];
[blanks(nb-n1)  num2str(b-1)  '-Komplement:  ' k_b1];
[sb  '-Komplement:  ' k_b];
              }
if ~nargout
   result2fig(Komplemente);
end