function  demobitrep(bits,lev,lim,r)
% demobitrep(bits,lev,lim,r)
% Darstellung der Reprsentation des Bitvektors bits 
% als Zeitfunktion einer physikalischen Grsse.
% bits:  darzustellender Bitvektor.  Ist bits eine Zahl, 
%        so werden bits zufllige Bits generiert (wenigstens 3).
% lev :  Vektor der beiden Soll-Niveaus fr die 
%        Reprsentation von 0 und 1
% lim :  2*2-Matrix mit den Grenzen der Reprsentations-
%        bereiche, Zeile 1 fr die 0, Zeile 2 fr die 1
% r   :  Standardabweichung eines additiven Rauschens
%        Wird ein Vektor mit 2 Komponenten angegeben, 
%        so gilt die erste (zweite) fr den Bereich 0 (1).

% Defaults
% ========
if nargin<1  bits = randb(64); end
if nargin<2   lev = [0 3.3];   end
if nargin<3   lim = [-0.5 0.7; 2.0 5.5]; end
if nargin<4     r = [0.1 0.4]; end
if length(bits)==1  bits = randb(round(max(3,bits))); end
if length(r)<2  r = r*[1 1];   end

% Konstanten
% ==========
fsiz = 13;                       % FontSize
n  = round(4096/length(bits));   % Schrittweite

% Signal
% ======
y  = raisedcosine(lev(bits+1) + r(bits+1).*randn(size(bits)),n);
ly = length(y);

% Graphik
% =======
fh = fullscreen('BitRep','w',3,[2 1; 1 1]);
axes('Position',[0 0 1 1])
patch([1 ly ly 1],[-5 -5 lim(1,1) lim(1,1)],0.5*[1 1 1])
patch([1 ly ly 1],[10 10 lim(2,2) lim(2,2)],0.5*[1 1 1])
patch([1 ly ly 1],[lim(2,1) lim(2,1) lim(1,2) lim(1,2)],0.8*[1 1 1])
text(n,lim(2,2)+1,'Bitreprsentation','FontSize',fsiz+4)
text(n,mean(lim(1,:)),'Reprsentation der 0','FontSize',fsiz)
text(n,mean(lim(2,:)),'Reprsentation der 1','FontSize',fsiz)
line(1:ly,y)
line(1:n:ly,y(1:n:ly),'Marker','.','MarkerSize',20,'Color','r','LineStyle','none')
axis([1  max(2,ly)  -4  9]); axis off