function b = democrc(f)
% b = democrc(k,f)
% Ein zuflliger binrer Informationsvektor der Lnge 512 
% wird generiert und mit einem CRC codiert.
% Das Codewort x wird an den im Indexvektor f 
% gegebenen Stellen invertiert und dieses Wort 
% wird decodiert.
%
% f :  Binrvektor mit den Indices der Fehlerstellen des Codewortes
%      Default Indexvektor mit Fehlerwahrscheinlichkeit 0.1 

% Defaults und Prfung des Parameters
% ===================================
k = 512;
n = k + 16;
if nargin<1  f = find(rand(1,n)<0.1); end
if any([f>n f<1 round(f)~=f])  
   error('demohammingcode:  der Fehlervektor enthlt ungltige Indices'); 
end
f = union(f,f);              % Duplikate vermeiden

% Information
% ===========
u = randb(512);              % zufllige Information von 512 Bits

% Codierung
% =========
p = [16 12 5 0];             % Polynom  x^16 + x^12 + x^5 + 1
x = [u crc(u,p)];            % systematische Kodierung

% Kanalsimulation
% ===============
q = zeros(1,n);
q(f) = 1;
y = mod( x+q ,2);

% Decodierung
% ===========
c = crc(y,p);                % CRC des empfangenen Wortes y
b = any(c)                   % igendein Fehler ?

% Ausgabe
% =======
if ~nargout
   CRC{1} = ['(528,512)-CRC-Code'];
   CRC{2} = ['#Fehlerstellen:  ' num2str(length(f))];
   CRC{3} = ['Fehler erkannt:  ' num2str(b)];
   result2fig(CRC)
end


