function nso = demoeinschritt(nsi)
% nso = demoeinschritt(nsi)
% Graphische Demonstration einschrittiger Codes.
% nsi :  Vektor, der die Ganzzahlinterpretation der 
%        Binrcodewrter enthlt in der Reihenfolge 
%        der Reprsentation fr 0, 1, 2, 3, ...
% Beispiele:
% demoeinschritt(0:59);            % nicht-einschrittiger Code
% demoeinschritt(einschritt(60));  % einschrittiger Code

if nargin<1  nsi = [0  8 10 14 12  4  6  7 15 11  3  2]; end

n = ceil(log2(max(nsi)+1));     % Anzahl der Bits

bitmat = num2binmat(nsi',n)';   % zeilenweise Binrdarstellung von nsi

for k=1:n
   bsig(k,:) = binsignal(bitmat(k,:),10,5,0); % Signalgenerierung
end

itp = ones(1,2^n)*2^n;          % Voreinstellung: undefiniert
itp(nsi+1) =  0:length(nsi)-1;  % Interpretationstabelle

% Graphik
% =======
fsiz = 13;                      % FontSize
[n,m] = size(bsig);
fh1 = fullscreen('Signale','w',0,[2 1; 1 1]);
set(fh1,'renderer','OpenGL')
axes('Position',[0 0 1 1])
line([0;m]*ones(1,n),[1;1]*(1:n),'Color',0.7*[1 1 1])
line((1:m)'*ones(1,n),0.5*bsig'+ones(1,m)'*(1:n),'Color','b')
axis([1 m 0.3 n+1.3]); axis off
text(10,n+1,'Bitsignale','FontSize',fsiz)
%setptr(fh1,'fullcrosshair')
set(fh1,'Pointer','circle')
%setptr(fh1,'crosshair')

fh2 = fullscreen('Interpretation','w',0,[2 1; 2 1]);
axes('Position',[0 0 1 1])
plot(1:m,itp(1+binmat2num(bsig'>0.5)))
axis([1 m 0 2^n]); axis off
text(10,0.8*2^n,'Interpretation als Zahl','FontSize',fsiz)
