function demofixpointadd(b,n,x,y)
% demofixpointadd(b,n,x,y)
% Interaktive Graphik zur Darstellung der Addition von ganzen 
% Zahlen unter Benutzung des b-Kompements.
% b :  Basis
% n :  Anzahl der Stellen
% x :  Operand 1
% y :  Operand 2

%  Autor:  Klaus von der Heide,  v.d.heide@on-line.de

%#function demofixpointaddcb

% Defaults
% ========
if nargin<1  b = 10;  end
if nargin<2  n = 8;   end
if nargin<3  x = 761; end
if nargin<4  y = -27; end

x = floor(x);
y = floor(y);

% Konstanten
% ==========
[fnam,fwgt,fsz,pres] = getfont;               % Font
if pres
   scrs = get(0,'ScreenSize');
   fsiz = round(22/1024*scrs(3));             % FontSize
   figp = [0 1 0 1];
else
   fsiz = fsz;
   figp = [0 0.5 0.5 1];
end
yp   = 0.83;                                  % y-Position der GUIs
dy   = 0.06;                                  % Hhe der GUIs
dt   = 0.1;                                   % Vertikalabstand Text - GUI
tx0  = 0.04;                                  % linker Textrand
tx1  = 0.5;                                   % linker Textrand fr Additionsfeld
tu   = 0.1;                                   % unterer Textrand
za   = 0.044;                                 % Zeilenabstand
         
% Parameter der Texte und GUIs
% ============================
helptxt = {'Hilfe zu demofixpointadd';        % Hilfetext
           'Alle Inhalte der Fenster knnen';
           'nach Klick editiert und mit der';
           'Eingabetaste abgeschlossen werden.'};
        
upos = [0.04  yp  0.1   dy;                   % Position der UIs
        0.24  yp  0.1   dy;
        0.49  yp  0.15  dy;
        0.78  yp  0.15  dy];
     
ucol = [1 1 1;                                % Farben der UIs
        1 1 1;
        1 1 1;
        1 1 1];
     
ustr = {num2str(b);                           % UI-Strings
        num2str(n);
        num2str(x);
        num2str(y)};

txt  = {'Basis';                              % Texte
        '#Stellen';
        'x';
        'y';
        'Reprsentation von abs(x) bzw. abs(y)';
        '(b-1)-Komplement';
        'b-Komplement';
        'Reprsentation von x und y';
        'Addition';
        'i';
        'x_i';
        'y_i';
        'c_i';
        'z_i';
        '(b-1)-Komplement';
        'b-Komplement';
        'Interpretation dezimal'};
     
tpos = [upos(1,1)+upos(1,3)/2  yp+dt;         % Position der Texte
        upos(2,1)+upos(2,3)/2  yp+dt;
        upos(3,1)+upos(3,3)/2  yp+dt;
        upos(4,1)+upos(4,3)/2  yp+dt;
        tx0                    tu+za*15;
        tx0                    tu+za*14;
        tx0                    tu+za*13;
        tx0                    tu+za*11;
        tx0                    tu+za*9;
        tx1                    tu+za*9;
        tx1                    tu+za*8;
        tx1                    tu+za*7;
        tx1                    tu+za*6;
        tx1                    tu+za*5;
        tx0                    tu+za*3;
        tx0                    tu+za*2;
        tx0                    tu];

tsiz = fsiz*[1 1 1 1 0.8*[1 1 1 1 1 1 1 1 1 1 1 1 1]];  % Grsse der Texte

if x<0
   vt1 = num2basstr(floor(abs(x)),b,n);
   vt3 = komplemb1(b,vt1);
   vt5 = komplemb(b,vt1);
   rx  = vt5;
else 
   vt1 = '';
   vt3 = '';
   vt5 = '';
   rx  = num2basstr(x,b,n);
end

if y<0
   vt2 = num2basstr(floor(abs(y)),b,n);
   vt4 = komplemb1(b,vt2);
   vt6 = komplemb(b,vt2);
   ry  = vt6;
else 
   vt2 = '';
   vt4 = '';
   vt6 = '';
   ry  = num2basstr(floor(y),b,n);
end

vt9  = '';
for k=0:n-1  vt9 = [sprintf('%d',mod(k,10)) vt9]; end

xbv  = basstr2basvec(rx,b);
ybv  = basstr2basvec(ry,b);
cbv(1) = 0;
for k=1:n
   [zbv(k) cbv(k+1)] = fulladd(xbv(k),ybv(k),cbv(k),b); % Addition
end

vt12 = basvec2str(cbv,n);
vt13 = basvec2str(zbv,n);
if basstr2num(vt13(1),b)>floor((b-1)/2)
   vt14 = komplemb1(b,vt13);
   vt15 = komplemb(b,vt13);
   vt16 = num2str(-basstr2num(vt15,b));
else
   vt14 = '';
   vt15 = '';
   vt16 = num2str(basstr2num(vt13,b));
end

vtx  = {vt1; vt2; vt3; vt4; vt5; vt6; rx; ry; vt9;
        rx; ry; vt12; vt13; vt14; vt15; vt16};
        
vx   = (upos(3,1)+upos(4,1)+upos(3,3))/2;     
vpos = [upos(3,1)+upos(3,3)/2  tu+za*15;
        upos(4,1)+upos(4,3)/2  tu+za*15;
        upos(3,1)+upos(3,3)/2  tu+za*14;
        upos(4,1)+upos(4,3)/2  tu+za*14;
        upos(3,1)+upos(3,3)/2  tu+za*13;
        upos(4,1)+upos(4,3)/2  tu+za*13;
        upos(3,1)+upos(3,3)/2  tu+za*11;
        upos(4,1)+upos(4,3)/2  tu+za*11;
        vx                     tu+za*9;
        vx                     tu+za*8;
        vx                     tu+za*7;
        vx                     tu+za*6;
        vx                     tu+za*5;
        vx                     tu+za*3;
        vx                     tu+za*2;
        vx                     tu];

vsiz = (fsiz+4)*ones(1,length(vtx));          % Grsse der variablen Texte
     
vcol = zeros(length(vtx),3);                  % Farben der variablen Texte
vcol(9,:) = [1 1 1];

% Bild und Achsen
% ===============
fh = fullscreen('ADD','w',0,figp);            % Schirmbild
axes('Position',[0 0 1 1])
axis([0 1 0 1 ]); axis off                    % Einheiten

% Grundflche
% ===========
patch([0 1 1 0],[0 0 1 1],0.8*[1 1 1]);

% Texte
% =====
for k=1:length(txt)
   txh(k) = text(tpos(k,1),tpos(k,2),txt{k},'VerticalAlignment','middle',...
                 'FontSize',tsiz(k),'Color','b');
end
for k=1:4  set(txh(k),'HorizontalAlignment','center','Color','r'); end           
set(txh(17),'Color','r')
set(txh,'FontUnits','normalized')

% variable Texte
% ==============
for k=1:length(vtx)
   vth(k) = text(vpos(k,1),vpos(k,2),vtx{k},'VerticalAlignment','middle',...
                 'FontName','FixedWidth',...
                 'HorizontalAlignment','center','FontSize',vsiz(k),'Color',vcol(k,:));
end
set(vth(16),'FontSize',fsiz+8,'Color','r','FontWeight','bold')
set(vth,'FontUnits','normalized')

% Interaktivitt
% ==============
for k=1:4
   uih(k) = uicontrol(fh,'Units','normalized','Position',upos(k,:),'String',ustr{k},'Style','edit',...
                      'FontSize',fsiz,'FontName','Arial','FontWeight','bold',...
                      'CallBack',sprintf('demofixpointaddcb(%d)',k),'BackgroundColor',ucol(k,:),...
                      'ForegroundColor','r');
end
set(uih,'FontUnits','normalized')
               
% UserData des Fensters fr die Interaktivitt
% ============================================
fud.helptext = helptxt;
fud.uih      = uih;
fud.vth      = vth;
fud.ustr     = ustr;
fud.data     = [b n x y];
set(fh,'UserData',fud,'KeyPressFcn','helpfcn');
