function demofsm(stab,sname,z0,typ,xname,xcod)
% demofsm(stab,sname,z0,typ,xname,xcod)
% Zustandsgraph und bergangstabelle fr ein Schaltwerk.
%
% stab  :  m*n-Matrix, die in der k-ten Zeile die Indices der
%          Folgezustnde fr den Zustand mit dem Index k enthlt
%          und zwar in der Spalte j den zur Eingangsbelegung mit 
%          dem Index j gehrigen Folgezustand.
%          Ist der Index <1, so wird keine Kante gezeichnet.  
% sname :  StructureArray mit den Strings der Zustandsnamen.
%          Die Namen sollten hchstens 3 Buchstaben haben,
%          sie mssen alle verschieden sein.  Nur der 
%          Zustandsname '*' fr Don't care darf mehrfach 
%          vorkommen.
% z0    :  Index des Anfangszustands (Default 1)
% typ=0 :  getaktetes Schaltwerk (Default)
% typ=1 :  nicht-getaktetes Schaltwerk
%          stab-Werte <1 fhren evtl. zu Fehlermeldungen.
% xname :  StructureArray mit den Strings der Inputnamen.
%          Hier kann man z.B. die Bitmuster angeben.
% xcod  :  String einer kurzen Erluterung zu xname, z.B. im 
%          Falle von Bitmustern 'D,C' um die Reihenfolge festzulegen.
%
% Beispiel:  demofsm([1 2; 3 4; 2 3; 1 4],{'RR';'Go';'Z1';'Z2'},1,1,{'0';'1'})

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  stab = [1 2; 3 4; 2 3; 1 4]; end
if nargin<2  
   sname = {'A';'B';'C';'D';'E';'F';'G';'H';'I';'J';'K';'L';'M';...
            'N';'O';'P';'Q';'R';'S';'T';'U';'V';'W';'X';'Y';'Z'};
   k = 1;   
   while length(sname)<length(stab)
      sname = cat(1,sname,strcat(sname{k},sname));
      k = k + 1;
   end
end
if nargin<3   z0 = 1;  end
if nargin<4  typ = 0;  end
if nargin<6 xcod = ''; end

% Prfung der Parameter
% =====================
[m,n] = size(stab);
if any(stab>m)  error('demofsm: bergangstabelle berprfen'); end
ok = 1;
for k=1:length(sname)-1
   for kk=k+1:length(sname)
      if ~strcmp(sname{k},'*') & strcmp(sname{kk},sname{k})  
         error('demofsm: Liste der Zustandsnamen berprfen'); 
      end
   end
end
if z0<1 | z0>m  error('demofsm:  Anfangszustand berprfen'); end
if ~(typ==0 | typ==1)  error('demofsm:  Schaltwerktyp berprfen'); end
if nargin>5
   for k=1:length(xname)-1
      for kk=k+1:length(xname)
         if strcmp(xname{kk},xname(k))  error('demofsm: Liste der Inputnamen berprfen'); end
      end
   end
end

% bergangstabelle bzw. Flusstafel
% ================================
if nargin<5 
   fsmtab(stab,sname,z0,typ); 
else 
   fsmtab(stab,sname,z0,typ,xname,xcod); 
end
fh1 = gcf;

% Zustandsdiagramm
% ================
drawfsm(stab,sname,z0)
fh2 = gcf;

% Interaktion beider Bilder
% =========================
fud1 = get(fh1,'UserData');
fud1.fig = fh2;
set(fh1,'UserData',fud1)
fud2 = get(fh2,'UserData');
fud2.fig = fh1;
set(fh2,'UserData',fud2)

% Nicht-getaktetes Schaltwerk
% ===========================
go = typ;
while go
   try
      pause(0.3)
      fud2 = get(fh2,'UserData');
      fud2.currs = fud2.stab(fud2.currs,fud2.currx);
      set(fh2,'UserData',fud2)
      fud1 = get(fh1,'UserData');
      fud1.currs = fud2.currs;
      fud1.currx = fud2.currx;
      set(fh1,'UserData',fud1);      % Update UserData
      fsmtabupdate(fh1)              % Update der Graphik
      drawfsmupdate(fh2)             % Update der Graphik
   catch
      go = 0;
   end
end