function demofsm2(stab,sname,z0,typ,xname,xcod)
% demofsm2(stab,sname,z0,typ,xname,xcod)
% Zustandsgraph und bergangstabelle fr ein Schaltwerk.
%
% stab  :  m*n-Matrix, die in der k-ten Zeile die Indices der
%          Folgezustnde fr den Zustand mit dem Index k enthlt
%          und zwar in der Spalte j den zur Eingangsbelegung mit 
%          dem Index j gehrigen Folgezustand.
%          Ist der Index <1, so wird keine Kante gezeichnet.  
% sname :  StructureArray mit den Strings der Zustandsnamen.
%          Die Namen sollten hchstens 3 Buchstaben haben.
% z0    :  Index des Anfangszustands (Default 1)
% typ=0 :  getaktetes Schaltwerk (Default)
% typ=1 :  nicht-getaktetes Schaltwerk
%          stab-Werte <1 fhren evtl. zu Fehlermeldungen.
% xname :  StructureArray mit den Strings der Inputnamen.
%          Hier kann man z.B. die Bitmuster angeben.
% xcod  :  String einer kurzen Erluterung zu xname, z.B. im 
%          Falle von Bitmustern 'D,C' um die Reihenfolge festzulegen.
%
% Beispiel:  demofsm([1 2; 3 4; 2 3; 1 4],{'RR';'Go';'Z1';'Z2'},1,1,{'0';'1'})

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  stab = '[1 2; 3 4; 2 3; 1 4]'; end
if nargin<2  
   sname = '{''A'';''B'';''C'';''D'';''E'';''F'';''G'';''H'';''I'';''J'';''K'';''L'';''M''}';
%            'N';'O';'P';'Q';'R';'S';'T';'U';'V';'W';'X';'Y';'Z'}';
%   k = 1;   
%   while length(sname)<length(stab)
%      sname = cat(1,sname,strcat(sname{k},sname));
%      k = k + 1;
%   end
end
if nargin<3   z0 = 1;  end
if nargin<4  typ = 0;  end
if nargin<6 xcod = ''; end

stab  = eval(stab);
sname = eval(sname);

% bergangstabelle bzw. Flusstafel
% ================================
if nargin<5  fsmtab(stab,sname,z0,typ); else fsmtab(stab,sname,z0,typ,xname,xcod); end
fh1 = gcf;

% Zustandsdiagramm
% ================
drawfsm(stab,sname,z0)
fh2 = gcf;

% Interaktion beider Bilder
% =========================
fud1 = get(fh1,'UserData');
fud1.fig = fh2;
set(fh1,'UserData',fud1)
fud2 = get(fh2,'UserData');
fud2.fig = fh1;
set(fh2,'UserData',fud2)

% Nicht-getaktetes Schaltwerk
% ===========================
go = typ;
while go
   try
      pause(0.3)
      fud2 = get(fh2,'UserData');
      fud2.currs = fud2.stab(fud2.currs,fud2.currx);
      set(fh2,'UserData',fud2)
      fud1 = get(fh1,'UserData');
      fud1.currs = fud2.currs;
      fud1.currx = fud2.currx;
      set(fh1,'UserData',fud1);      % Update UserData
      fsmtabupdate(fh1)              % Update der Graphik
      drawfsmupdate(fh2)             % Update der Graphik
   catch
      go = 0;
   end
end

%---------------------------------------------------------------------

function drawfsm(stab,sname,z0)
% drawfsm(stab,sname,z0)
% Zustandsgraph fr ein Schaltwerk mit der 
% bergangstabelle stab.
%
% stab  :  m*n-Matrix, die in der k-ten Zeile die Indices der
%          Folgezustnde fr den Zustand mit dem Index k enthlt
%          und zwar in der Spalte j den zur Eingangsbelegung mit 
%          dem Index j gehrigen Folgezustand.
%          Ist der Index <1, so wird keine Kante gezeichnet.  
% sname :  StructureArray mit den Strings der Zustandsnamen.
%          Die Namen sollten hchstens 3 Buchstaben haben.
% z0    :  Index des Anfangszustands (Default 1)
%
% Beispiel:  drawfsm([1 2; 3 4; 2 3; 1 4],{'RR';'Go';'Z1';'Z2'})

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  stab = [1 2; 3 4; 2 3; 1 4]; end
if nargin<2  
   sname = {'A';'B';'C';'D';'E';'F';'G';'H';'I';'J';'K';'L';'M';...
            'N';'O';'P';'Q';'R';'S';'T';'U';'V';'W';'X';'Y';'Z'};
   k = 1;   
   while length(sname)<length(stab)
      sname = cat(1,sname,strcat(sname{k},sname));
      k = k + 1;
   end
end
if nargin<3  z0 = 1; end

% Konstanten
% ==========
lw = 1.5;                    % LineWidth
nc = 0.7*[1 1 1];            % Knoten-Farbe
sc = 'g';                    % Farbe selektierter Knoten
nl = 'k';                    % Kanten-Farbe
sl = 'r';                    % Farbe selektierter Kanten
[ns,nx] = size(stab);        % Anzahl von Knoten und mglichen Inputs
r  = 1/ns;                   % Knotenradius
sf = 0.25/ns;                % Buchstabenabstand in Kantenbeschriftung
fsz= min(20,160*r);          % Fontsize fr Zustandsbeschriftung

% Hilfetext
% =========
helptext = {'Hilfe zum Zustandsdiagramm';
            '=====================';
            '';
            'Whlen des Eingabeindex:';
            '   Tasten 1, 2, ... 9';
            '';
            'Fortschalten in Folgezustand:';
            '   Taste c (clock) oder t (Takt)';
            '   oder n (next), auch C,T,N'};

% Schlingenprototyp
% =================
u  = pi*(2:18)/10;           % Polygonwinkel 
sx = r*sin(u);               % x-Koordinaten der Polygonsttzstellen
sy = 1 + 2*r - 1.5*r*cos(u); % y-Koordinaten der Polygonsttzstellen
tx = 0;                      % x-Koordinate des Textes
ty = 1 + 2*r;                % y-Koordinate des Textes

% Fenster
% =======
fh = fullscreen('Zustandsdiagramm','w',0,[0 0.75 0 1]);
axes('Position',[0 0 1 1])

% Zustandsknoten
% ==============
a  = (0:2*pi/ns:1.99*pi)';   % Winkelpositionen der Knoten
cc = [sin(a) cos(a)];        % Positionen der Zustandsknoten
for k=1:ns
   sh(k) = drawccir(cc(k,:),r,nc,sname{k},fsz);
end

% bergnge
% =========
for k=1:ns
   for j=1:nx
      fs = stab(k,j);        % Folgezustand
      dx = sf*sum(stab(k,1:j)==fs);
      if fs==k               % Schlinge
         A  = [cc(k,[2 1]); -cc(k,1) cc(k,2)];
         xy = A*[sx;sy];
         ddx= sf*sum(stab(k,1:nx)==fs)/2;
         txy= A*[tx;ty];
         px = [xy(1,1) xy(1,2)+0.3*(xy(2,2)-xy(2,1))*[-1 +1]];
         py = [xy(2,1) xy(2,2)-0.3*(xy(1,2)-xy(1,1))*[-1 +1]];
         lh{k,j} = [line(xy(1,:),xy(2,:),'Color',nl,'LineWidth',lw),...
                    patch(px,py,nl),...
                    text(txy(1)+dx-ddx,txy(2),num2str(j),'FontName','FixedWidth',...
                         'VerticalAlignment','middle','HorizontalAlignment','center')];
      elseif fs>0            % Kante
         dd = cc(fs,:) - cc(k,:);
         ll = sqrt(dd*dd');
         if prod(dd)>0
            xy = cc(k,:) + 2*r*dd/ll;
            thnd = text(xy(1)+dx-0.02,xy(2),num2str(j),'FontName','FixedWidth',...
                        'VerticalAlignment','top','HorizontalAlignment','left');
         else
            xy = cc(k,:) + 2*r*dd/ll;
            thnd = text(xy(1)-dx+0.03,xy(2),num2str(j),'FontName','FixedWidth',...
                        'VerticalAlignment','top','HorizontalAlignment','right');
         end
         lh{k,j} = [drawarr(cc(k,:),cc(fs,:),nl,lw,r/3,r,r),thnd];
      end
   end
end

axis((1+6/ns)*[-1 +1 -1 +1]); axis off

% Interaktion
% ===========
fud.helptext = helptext;
fud.nodes    = sh;
fud.edges    = lh;
fud.stab     = stab;
fud.currs    = z0;
fud.currx    = 1;
fud.fig      = [];
fud.figfcn   = [];
set(fh,'UserData',fud,'KeyPressFcn',@drawfsmkpf);
set(fud.nodes(fud.currs),'FaceColor',sc)
set(findobj(fud.edges{fud.currs,fud.currx},'Type','patch'),'FaceColor',sl)
set(findobj(fud.edges{fud.currs,fud.currx},'Type','line'),'Color',sl,'LineWidth',3)

%---------------------------------------------------------------------

function fsmtab(stab,sname,z0,typ,xname,xcod,yname)
% fsmtab(stab,sname,z0,typ,xname,xcod,yname)
% Graphik der Flusstafel fr die bergangstabelle stab eines Schaltwerks.
%
% stab  :  m*n-Matrix, die in der k-ten Zeile die Indices der
%          Folgezustnde fr den Zustand mit dem Index k enthlt
%          und zwar in der Spalte j den zur Eingangsbelegung mit 
%          dem Index j gehrigen Folgezustand.
%          Ist der Index 0, so bleibt der Folgezustand leer, 
%          Ist der Index -1, so tritt anstelle des Folgezustands ein * .
%          stab kann auch ein StructureArray sein von zwei Tabellen:
%          die erste wie oben beschrieben fr die bergangsfunktion,
%          die zweite fr die Ausgabefunktion. Letztere ist im Falle 
%          eines Moore-Schaltwerks ein m-Spaltenvektor und im Falle 
%          eines Mealy-Schaltwerks eine m*n-Matrix von Indices in yname.
% sname :  StructureArray mit den Strings der Zustandsnamen.
%          Die Namen sollten hchstens 3 Buchstaben haben.
% z0    :  Index des Anfangszustands (Default 1)
% typ=0 :  getaktetes Scahltwerk
% typ=1 :  nicht-getaktetes Schaltwerk, stabile Zustnde markiert.
%          stab-Werte <1 fhren evtl. zu Fehlermeldungen.
% xname :  StructureArray mit den Strings der Inputnamen.
%          Hier kann man z.B. die Bitmuster angeben.
% xcod  :  String einer kurzen Erluterung zu xname, z.B. im 
%          Falle von Bitmustern 'D,C' um die Reihenfolge festzulegen.
% yname :  StructureArray mit den Strings der Ausgabecodierung oder -bezeichnung.
%
% Beispiel:  fsmtab([1 2; 3 4; 2 3; 1 4],{'RR';'Go';'Z1';'Z2'},3,1,{'0';'1'})

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  stab = [1 2; 3 4; 2 3; 1 4]; end
if nargin<2  
   sname = {'A';'B';'C';'D';'E';'F';'G';'H';'I';'J';'K';'L';'M';...
            'N';'O';'P';'Q';'R';'S';'T';'U';'V';'W';'X';'Y';'Z'};
   k = 1;   
   while length(sname)<length(stab)
      sname = cat(1,sname,strcat(sname{k},sname));
      k = k + 1;
   end
end
if nargin<3   z0 = 1;  end
if nargin<4  typ = 0;  end
if nargin<6 xcod = ''; end
if nargin<7 
   yname = {'a';'b';'c';'d';'e';'f';'g';'h';'i';'j';'k';'l';'m';...
            'n';'o';'p';'q';'r';'s';'t';'u';'v';'w';'x';'y';'z'};
end

% Konstanten
% ==========
if iscell(stab)
   yt   = 1;
   ytab = stab{2};           % Ausgangstabelle
   stab = stab{1};           % bergangstabelle
   [dummy,ny] = size(ytab)
else
   yt = 0;
   ny = 0;
end
[ns,nx] = size(stab);        % Anzahl von Knoten und mglichen Inputs
fsiz = min(15,500/ns);       % FontSize der Tabelle
fsz  = min(15,fsiz);         % FontSize der Beschriftung
ls = 0;
for k=1:length(sname) ls = max([ls,length(sname{k})]); end
fszc = min(fsiz-2,sqrt(2/ls)*fsz); % FontSize der Zustandsbezeichnung
lw = 1.5;                    % LineWidth
lc = 'k';                    % Linien-Farbe
nc = 0.7*[1 1 1];            % Knoten-Farbe
sc = 'g';                    % Farbe selektierter Knoten
nl = 'k';                    % Kanten-Farbe
sl = 'r';                    % Farbe selektierter Kanten
r  = 1/ns;
[msn,nsn] = size(sname);
if msn==1
   snm = cat(2,{'*',' '},sname);
else
   snm = cat(1,{'*';' '},sname);
end

% Hilfetext
% =========
helptext = {' ';
            ' ';
            'Hilfe zur Flusstafel';
            '=============';
            '';
            'nderung des Eingabeindex:';
            '   Mausklick auf den Eingabeindex';
            '   oder zugleich mit Takt:';
            '';
            'Fortschalten in Folgezustand:';
            '   Mausklick auf einen Folgezustand';
            '   des momentanen Zustandes'};

% Fenster
% =======
fh = fullscreen('Flusstafel','w',0,[max(0,0.81-0.03*nx) 1 0 1]);
axes('Position',[0 0 1 1])

% Achsenskalierung
% ================
axis([-2.5 nx+ny+2.5 -ns-1 +1]); axis off

% Flusstafel
% ==========
for k=1:ns
   for j=1:nx
      th(k,j) = text(j,-k,snm{2+stab(k,j)},'FontSize',fszc,'FontName','Helvetica',...
                     'ButtonDownFcn',@fsmtabbdf,...
                     'HorizontalAlignment','center','VerticalAlignment','middle');
      if typ & stab(k,j)==k  drawcircle(j,-k,0.06*ns^0.7,'LineWidth',2,'Color','b'); end
   end
   if yt
      for j=1:ny
         text(nx+j+1,-k,yname{ytab(k,j)},'FontSize',fszc,'FontName','Helvetica',...
              'HorizontalAlignment','center','VerticalAlignment','middle');
      end
   end
end

% Beschriftung der Flusstafel
% ===========================
line([-1.8 nx+ny+1.5],[0 0]-0.5,'LineWidth',lw,'Color',lc)
line([0 0],[-ns-0.4 0.6],'LineWidth',lw,'Color',lc)
text(-1,0,'Zustand','FontSize',fsz,'FontWeight','bold','HorizontalAlignment','center')
text(0.5+nx/2,0.5,'Index der Eingabe','FontSize',fsz,'FontWeight','bold','HorizontalAlignment','center')
%if yt
%   if ny==1  ystr = '/y'; else ystr = '/Ausgabe'; end
%   text(1.5+nx+ny/2,0.5,ystr,'FontSize',fsz,'FontWeight','bold','HorizontalAlignment','center')
%end
if nargin>4
   text(0.5+nx/2,0.25,['(darunter Codierung ' xcod ')'],'FontSize',fszc,'HorizontalAlignment','center')
end
for k=1:ns
   ths(k) = text(-1,-k,snm{k+2},'FontSize',fsz,'FontName','Helvetica',...
                 'HorizontalAlignment','center','VerticalAlignment','middle');
end
for j=1:nx
   thx(j) = text(j,0,sprintf('%d',j),'FontSize',fsz,'FontName','Helvetica',...
                 'ButtonDownFcn',@fsmtabbdf,...
                 'HorizontalAlignment','center','VerticalAlignment','middle');
   if nargin>4  text(j,-0.25,xname{j},'FontSize',fszc,'FontName','Helvetica',...
                     'ButtonDownFcn',@fsmtabbdf,...
                     'HorizontalAlignment','center','VerticalAlignment','middle');
                  
   end
end

% Interaktion
% ===========
fud.helptext = helptext;
fud.stab     = stab;
fud.currs    = z0;
fud.currx    = 1;
fud.ths      = ths;
fud.thx      = thx;
fud.th       = th;
fud.fig      = [];
fud.figfcn   = [];
set(fud.thx(fud.currx),'Color','r','FontWeight','bold')
set(fud.ths(fud.currs),'Color','g','FontWeight','bold')
set(fud.th(fud.currs,fud.currx),'Color','r','FontWeight','bold')
set(fh,'UserData',fud,'KeyPressFcn',@helpfcn);

%---------------------------------------------------------------------

function  drawfsmupdate(fh)
% drawfsmupdate(fh)
% UpdateFcn fr drawfsm mit figurehandle fh.

% Konstanten
% ==========
lw = 1.5;                   % LineWidth
nc = 0.7*[1 1 1];           % Knoten-Farbe
sc = 'g';                   % Farbe selektierter Knoten
nl = 'k';                   % Kanten-Farbe
sl = 'r';                   % Farbe selektierter Kanten
fud = get(fh,'UserData');   % Figure-User-Data
[ns,nx] = size(fud.stab);   % Anzahl der Zustnde und Eingangskombinationen

% nderungen in der Graphik
% =========================
for k=1:ns
   for j=1:nx
      set(findobj(flach(fud.edges{k,j}),'Type','patch'),'FaceColor',nl)
      set(findobj(flach(fud.edges{k,j}),'Type','line'),'Color',nl,'LineWidth',lw)
   end
end
set(fud.nodes,'FaceColor',nc);
set(findobj(fud.edges{fud.currs,fud.currx},'Type','patch'),'FaceColor',sl)
set(findobj(fud.edges{fud.currs,fud.currx},'Type','line'),'Color',sl,'LineWidth',3)
set(fud.nodes(fud.currs),'FaceColor',sc);

%---------------------------------------------------------------------

function  fsmtabupdate(fh)
% fsmtabupdate(fh)
% UpdateFcn fr fsmtab mit figurehandle fh.

fud = get(fh,'UserData');
set(fud.ths,'Color','k','FontWeight','normal')
set(fud.thx,'Color','k','FontWeight','normal')
set(fud.th, 'Color','k','FontWeight','normal')
set(fud.thx(fud.currx),'Color','r','FontWeight','bold')
set(fud.ths(fud.currs),'Color','g','FontWeight','bold')
set(fud.th(fud.currs,fud.currx),'Color','r','FontWeight','bold')

%---------------------------------------------------------------------

function drawfsmkpf(obj,eventdata)
% drawfsmkpf
% keypressfcn fr drawfsm

% Konstanten
% ==========
lw = 1.5;                   % LineWidth
nc = 0.7*[1 1 1];           % Knoten-Farbe
sc = 'g';                   % Farbe selektierter Knoten
nl = 'k';                   % Kanten-Farbe
sl = 'r';                   % Farbe selektierter Kanten
cfg = gcf;                  % Figure Handle
fud = get(cfg,'UserData');  % UserData
cc  = get(cfg,'CurrentCharacter'); % zuletzt eingegebener Buchstabe

% Interaktion
% ===========
switch cc
case {'T','t','C','c','N','n'}
   nexts = fud.stab(fud.currs,fud.currx);
   if nexts>0
      fud.currs = nexts;
   else
      sound(sin(2*pi*0.125*(0:1000)),8000) % Ton bei Falscheingabe
      return
   end
case {'h','H'}
   fsiz = 13;
   fud = get(cfg,'UserData');
   nz = length(fud.helptext);
   fh = fullscreen('Hilfe','w',0,[0 0.5 0 1]);
   set(fh,'ButtonDownFcn','delete(gcf)')
   axes('Position',[0.1 0.1 0.8 0.8])
   axis([0 1 -nz 2])
   axis off
   text(0,-nz/2,fud.helptext,'FontSize',fsiz)
case {'0','1','2','3','4','5','6','7','8','9'}
   ccn = str2num(cc);
   [ns,nx] = size(fud.stab);
   if ccn==0 | ccn>nx | fud.stab(fud.currs,ccn)<1  
      sound(sin(2*pi*0.125*(0:1000)),8000) % Ton bei Falscheingabe
      return
   else
      fud.currx = ccn;                     % neuer Inputindex 
   end
end

% Graphik ndern
% ==============
set(cfg,'UserData',fud);                   % Update UserData
drawfsmupdate(cfg)                         % Update der Graphik

% Assoziierte Graphik ndern
% ==========================
if ~isempty(fud.fig)
   try
      fudfig = get(fud.fig,'UserData');
      fudfig.currs = fud.currs;
      fudfig.currx = fud.currx;
      set(fud.fig,'UserData',fudfig);      % Update UserData
      fsmtabupdate(fud.fig)                % Update der Graphik
   end
end

if double(cc)==27  msvdelfig;   end        % delete all
if double(cc)==127 delete(gcf); end        % delete

%---------------------------------------------------------------------

function  fsmtabbdf(obj,eventdata)
% fsmtabbdf
% ButtonDownFcn fr fsmtab zur interaktiven 
% Eingabe des Schaltwerk-Inputs.

% Konstanten
% ==========
cfg = gcf;
cax = gca;
obj = gcbo;
fud = get(cfg,'UserData');

% Neuen Input setzen
% ==================
if ismember(obj,fud.thx) fud.currx = find(obj==fud.thx); end
if ismember(obj,fud.th)  
   [s,j] = find(obj==fud.th); 
   if s==fud.currs
      if fud.stab(s,j)>0
         fud.currs = fud.stab(s,j);
         fud.currx = j;
      else
         sound(cos(2*pi*(0:1000)/8),8000)
         return
      end
   else
      return
   end 
end
set(cfg,'UserData',fud)

% Graphik ndern
% ==============
fsmtabupdate(cfg)

% Assoziierte Graphik ndern
% ==========================
if ~isempty(fud.fig)
   try
      fudfig = get(fud.fig,'UserData');
      fudfig.currs = fud.currs;
      fudfig.currx = fud.currx;
      set(fud.fig,'UserData',fudfig);
      drawfsmupdate(fud.fig)
   end
end

%---------------------------------------------------------------------

function helpfcn(obj,eventdata)
% helpfcn
% KeypressFcn fr figure:  liefert mit der Taste h (H) 
% ein Fenster mit dem Text, der in 'UserData' des Bildes 
% als Struktur unter dem Strukturnamen helptext abgelegt ist.

fsiz = 13;
cfg = gcf;
cc = get(cfg,'CurrentCharacter');

if upper(cc)=='H'                     % Hilfe-Fenster
   fud = get(cfg,'UserData');
   nz = length(fud.helptext);
   fh = fullscreen('Hilfe','w',0,[0 0.5 max(1-nz/20,0.6) 1]);
   set(fh,'ButtonDownFcn','delete(gcf)')
   axes('Position',[0.1 0.1 0.8 0.8])
   axis([0 1 0 1])
   axis off
   text(0,0.75,fud.helptext,'FontSize',fsiz)
end

if double(cc)==27  msvdelfig;   end   % delete all
if double(cc)==127 delete(gcf); end   % delete

%---------------------------------------------------------------------


