function b = demohammingcode(u,f)
% demohammingcode(u,f)
% Der binre Informationsvektor u wird codiert.
% Das Codewort x wird an den im Indexvektor f 
% gegebenen Stellen invertiert und dieses Wort 
% wird decodiert.
%
% u :  Binrvektor der Lnge 1, 4, 11, 26
% f :  Binrvektor mit den Indices der Fehlerstellen des Codewortes
% Defaults:  u : Zufallsmuster fr (7,4)-Hammingcode
%            f : Indexvektor mit Fehlerwahrscheinlichkeit 0.1
% b :  Anzahl der Bitfehler der decodierten Information

% Defaults und Prfung der Parameter
% ==================================
if nargin<1  u = randb(4);        end
if ~any(length(u)==[1 4 11 26])  error('demohammingcode: Lnge des Informationsvektors ist nicht aus [1 4 11 26]'); end
k = length(u);
switch k
case 1,  n = 3;  r = 2;
case 4,  n = 7;  r = 3;
case 11, n = 15; r = 4;
case 26, n = 31; r = 5;
end
if ~all(u==0 | u==1)  error('demohammingcode: Der Informationsvektor ist nicht binr'); end
if nargin<2  f = find(rand(1,n)<0.1); end
if any([f>n f<1])  error('demohammingcode:  der Fehlervektor enthlt ungltige Indices'); end

% Code
% ====
G = hamminggenerator(r,0);
H = hamminggenerator(r,1);

% Codierung
% =========
x = mod( u*G ,2);

% Kanalsimulation
% ===============
q = zeros(1,n);
q(f) = 1;
y = mod( x+q ,2);

% Decodierung
% ===========
s = mod( y*H ,2);                    % Syndrom
b = 1;                               % Schleifenanfangswert
z = y;                              
if any(s) 
   while any(s~=H(b,:)) b = b+1; end % Suche nach Identitt der Zeile
   z(b) = mod( y(b)+1 ,2);           % Korrektur
end       
z(1:4)                               % decodierte Information
b = sum(z(1:k)~=u);                  % Anzahl der Bitfehler der decodierten Information

% Ausgabe
% =======
if ~nargout
   Hammningcode{1} = ['Information:   ' num2str(u)];
   Hammningcode{2} = ['Codewort:      ' num2str(x)];
   Hammningcode{3} = ['Fehlervektor:  ' num2str(q)];
   Hammningcode{4} = ['zu decodieren: ' num2str(y)];
   Hammningcode{5} = ['decodiert:     ' num2str(z(1:k))];
   Hammningcode{6} = ['#Bitfehler:    ' num2str(b)];
   result2fig(Hammningcode)
end

