function demoieee754(x,type)
% demoieee754(x,type)
% Interaktive Graphik zur Darstellung der IEEE-754-Formate fr Gleitkomma.
% x        :  darzustellende Zahl (bzw. entsprechender Ausdruck)
% type = 1 :  short real
% type = 2 :  long real
% type = 3 :  temporary real (deaktiviert)
%!!! Vorsicht: Diese Funktion soll nur der Demonstration dienen.
%!!! Wegen der Benutzung der Rechner-internen Arithmetik kann 
%!!! das Resultat in Grenzfllen bei long real falsch sein,  

%  Autor:  Klaus von der Heide,  v.d.heide@on-line.de

%#function demoieee754fcn

% Defaults
% ========
if nargin<2  type = 1; end
if nargin<1  x = 178.125; end

% Konstanten
% ==========
switch type
   case 1
      n = 32;                                 % Wortlnge
      m  = 24;                                % Index des LSB des Exponenten;
   case 2
      n = 64;                                 % Wortlnge
      m  = 53;                                % Index des LSB des Exponenten;
%   case 3
%      n = 80;                                 % Wortlnge
%      m  = 65;                                % Index des LSB des Exponenten;
end

[fname,fwgt,fsiz,pres] = getfont;             % Font
if pres  fsiz = 16; end

xm   = 0.23;
ym   = 0.3;
dx   = 0.0135;
dy   = 0.08;
yt   = 0.45;

txt  = {'IEEE-754';
        '\pm';                                % Texte
        'Exponent';
        'Mantisse';
        'implizit';
        'Zahl'};
     
fsz = fsiz + [22 0 0 0 0 0];                  % FontSize
     
ustr = {'178.125';                            % UI-Strings
        '0';
        '10000110';
        '1.';
        '01100100010000000000000';
        {'short real';
        'long real'}      %        'temporary real'}
       };
     
helptxt = {'Hilfe zu demoieee754:';           % Hilfetext
           'Alle Inhalte der Fenster knnen';
           'nach Klick editiert und mit der';
           'Eingabetaste abgeschlossen werden.'};

if pres
   fpos = [ 3  458  1018  288];               % Figure Position
else
   fpos = [ 3  493   783  253];               % Figure Position
end
tpos = [0.8 0.83;                             % Position der Texte
        xm-dx*(n-m+0.5) yt;
        xm-dx*(n-m)/2   yt;
        xm+dx*(m+1.5)/2 yt;
        xm+dx*1.25      ym+dy+0.01;
        0.27            0.8];
     
tcol = zeros(length(txt),3);                  % Farben der Texte
tcol(5,:) = [1 1 1];     
        
upos = [0.103           0.65  0.34      dy;   % Position der UIs
        xm-dx*(n-m+1.5) ym    0.02      dy; 
        xm-dx*(n-m)     ym    dx*(n-m)  dy;
        xm              ym    dx*2.5    dy;
        xm+dx*2.5       ym    dx*(m-1)  dy; 
        0.7             0.55  0.2       0.2];
     
ucol = [1 1 1;                                % Farben der UIs
        0.7 1 0.7;
        1 0.8 0.8;
        0 0 0;
        0.7 0.9 1;
        1 1 1];
           
% Bild und Achsen
% ===============
fh = fullscreen('IEEE754');                   % Schirmbild
set(fh,'Position',fpos)
axes('Position',[0 0 1 1])
axis([0 1 0 1 ]); axis off                    % Einheiten

% Grundflche
% ===========
patch([0 1 1 0],[0 0 1 1],0.8*[1 1 1]);

% Texte
% =====
for k=1:length(txt)
   txh(k) = text(tpos(k,1),tpos(k,2),txt{k},'HorizontalAlignment','center','VerticalAlignment','middle',...
                 'FontSize',fsz(k),'Color',tcol(k,:));
end
set(txh(1),'FontName','Times','Color','blue')           
set(txh,'FontUnits','normalized')
set(txh(5),'HorizontalAlignment','left','Rotation',90)

% Interaktivitt
% ==============
for k=1:5
   uih(k) = uicontrol(fh,'Units','normalized','Position',upos(k,:),'String',ustr{k},'Style','edit',...
                      'FontSize',fsiz,'FontName','FixedWidth','FontWeight','bold',...
                      'CallBack','demoieee754fcn(k)','BackgroundColor',ucol(k,:));
end
uih(6) = uicontrol(fh,'Units','normalized','Position',upos(6,:),'String',ustr{6},'Style','listbox',...
                   'FontSize',fsiz,'FontName','FixedWidth',...
                   'CallBack','demoieee754fcn(6)','BackgroundColor',ucol(6,:));
for k=1:6
   fcn = ['demoieee754fcn(' num2str(k) ')'];                   
   set(uih(k),'CallBack',fcn)                   
end
set(uih(4),'Enable','off','ForegroundColor','w')  % ForegroundColor: MATLAB-Fehler
set([uih,txh],'FontUnits','normalized')

% Font anpassen
% =============
ext5 = get(uih(5),'extent');
k = fsiz;
while ext5(3)>upos(5,3)
   k = k - 1;
   set([uih(1:5),txh(3:6)],'FontSize',k)
   ext5 = get(uih(5),'extent');
end
               
% UserData des Fensters fr die Interaktivitt
% ============================================
fud.helptext = helptxt;
fud.handles  = uih;
fud.txh      = txh;
fud.uistr    = ustr;
fud.uipos    = upos;
set(fh,'UserData',fud,'KeyPressFcn','helpfcn');
