function demokvdiagram(ftab)
% demokvdiagram(ftab)
% Animierte Graphik zur Erluterung des KV-Diagramms

% Autor:  Klaus von der Heide,  e-mail  v.d.heide@on-line.de

% Default
% =======
if nargin<1  ftab = [1 1 0 1 0 0 1 0]; end

% Konstanten
% ==========
nn  = length(ftab);
kl1 = 0.1;
kl2 = 0.5;
kl  = kl1;
farbe = 'g';
fsiz = 30;
view = [-15 20];
vw2  = [0 0];

% Positionen
% ==========
pos = [ 0 0 0;  0 0 1;  0 1 0;  0 1 1;  1 0 0;  1 0 1;  1 1 0;  1 1 1];
ps2 = [ 0 0 0;  0 0 1; -1 0 0; -1 0 1;  1 0 0;  1 0 1;  2 0 0;  2 0 1];
% Line-Positionen
i   = 0;
for k=1:length(pos)
   for j=k:length(pos)
      if sum(abs(pos(k,:)-pos(j,:)))==1
         i = i + 1;
         lp(i,:,:) = [pos(k,:); pos(j,:)]';
      end
   end
end
% Patch- und Text-Positionen
for k=1:length(ftab)
   pp(k,1,:,1) = pos(k,1)+kl*[-1 -1 -1 -1];
   pp(k,2,:,1) = pos(k,1)+kl*[-1  1  1 -1];
   pp(k,3,:,1) = pos(k,1)+kl*[-1  1  1 -1];
   pp(k,1,:,2) = pos(k,2)+kl*[-1 -1  1  1];
   pp(k,2,:,2) = pos(k,2)+kl*[-1 -1 -1 -1];
   pp(k,3,:,2) = pos(k,2)+kl*[-1 -1  1  1];
   pp(k,1,:,3) = pos(k,3)+kl*[-1  1  1 -1];
   pp(k,2,:,3) = pos(k,3)+kl*[-1 -1  1  1];
   pp(k,3,:,3) = pos(k,3)+kl*[ 1  1  1  1];
   tp(k,:)     = pos(k,:)-kl*[0 1.1 0];
end

% Fenster
% =======
fh = fullscreen('KV-Diagramm','w',3);
axes('Position',[0 0 1 1])

% Wrfelkanten
% ============
lh = [];
for k=1:length(lp)
   lh = [lh  line(squeeze(lp(k,1,:)),squeeze(lp(k,2,:)),squeeze(lp(k,3,:)),'Color','r','LineWidth',1.5)];
end

% Eins- und Null-Punkte
% =====================
ph = [];
th = [];
for k=1:nn
   for j=1:3
      ph = [ph  patch(squeeze(pp(k,j,:,1))',squeeze(pp(k,j,:,2))',squeeze(pp(k,j,:,3))',farbe)];
   end
   th = [th  text(tp(k,1),tp(k,2),tp(k,3),num2str(ftab(k)),'HorizontalAlignment','center',...
                  'verticalAlignment','middle','Color','k','FontSize',fsiz)];
end

axis(0.8*[-1 +2 -1 +2 -1 +2]);  
axis square;  axis off
set(gca,'View',view)

% Animation
% =========
pause(2)
for t = [(0:0.2:3).^2, 10:100]
   
   % Wrfelkantenlnge
   % =================
   kl = kl1 + t^2/10000*(kl2-kl1);
   
   % Positionen
   % ==========
   ps = pos + t/100*(ps2-pos);

   i   = 0;
   for k=1:length(pos)
      for j=k:length(pos)
         if sum(abs(pos(k,:)-pos(j,:)))==1
            i = i + 1;
            lp(i,:,:) = [ps(k,:); ps(j,:)]';
         end
      end
   end
   for k=1:length(ftab)
      pp(k,1,:,1) = ps(k,1)+kl*[-1 -1 -1 -1];
      pp(k,2,:,1) = ps(k,1)+kl*[-1  1  1 -1];
      pp(k,3,:,1) = ps(k,1)+kl*[-1  1  1 -1];
      pp(k,1,:,2) = ps(k,2)+kl*[-1 -1  1  1];
      pp(k,2,:,2) = ps(k,2)+kl*[-1 -1 -1 -1];
      pp(k,3,:,2) = ps(k,2)+kl*[-1 -1  1  1];
      pp(k,1,:,3) = ps(k,3)+kl*[-1  1  1 -1];
      pp(k,2,:,3) = ps(k,3)+kl*[-1 -1  1  1];
      pp(k,3,:,3) = ps(k,3)+kl*[ 1  1  1  1];
      tp(k,:)     = ps(k,:)-kl*[0 1.1 0];
   end
      
   % Wrfelkanten
   % ============
   for k=1:length(lp)
      set(lh(k),'XData',squeeze(lp(k,1,:)),'YData',squeeze(lp(k,2,:)),'ZData',squeeze(lp(k,3,:)))
   end

   % Eins- und Null-Punkte
   % =====================
   i = 0;
   for k=1:nn
      for j=1:3
         i  = i + 1;
         set(ph(i),'XData',squeeze(pp(k,j,:,1))','YData',squeeze(pp(k,j,:,2))','ZData',squeeze(pp(k,j,:,3))');
      end
      set(th(k),'Position',[tp(k,1) tp(k,2) tp(k,3)])
   end
   drawnow
   set(gca,'View',view+t/100*(vw2-view))
   axis((0.8+0.7*t/100)*[-1 +2 -1 +2 -1 +2]);  
end
