function demotransformation(p,j,w)
% demotransformation(p,j,w)
% Das durch die Eckpunkte der Matrix p und die Reihenfolge 
% der Eckpunkte j definierte graphische Objekt wird ungedreht 
% und um den Winkel w gedreht dargestellt.
% p :  2*n-Matrix oder 3*n-Matrix von Eckpunktkoordinaten
% j :  Vektor von Indices in Spalten von p
% w :  Winkel in Grad
% Falls p eine 3*n-Matrix ist, wird eine 3-D-Graphik 
% geliefert, die mit der Maus drehbar ist.  w ist dann 
% die Drehung um die z-Achse.

% Defaults
% ========
if nargin<3  w  = 70; end                              % Drehwinkel 70
if nargin<1  
   p  = [ -0.5  +0.5  -0.5  +0.5  0;  0  0  1  1  1.5 ]; % Eckpunkte der Graphik
   j  = [1 2 4 5 3 4 1 3 2];                             % Reihenfolge der Punkte
elseif nargin<2
   [m,n] = size(p);
   j = [randpermut(n) randpermut(n)];
end
[m,n] = size(p);

% Transformationsmatrix
% =====================
w  = w*pi/180;
if m==2
   A  = [cos(w)  -sin(w);  sin(w)  cos(w)];            % Transformationsmatrix
else
   A  = [cos(w)  -sin(w)  0;  sin(w)  cos(w)  0;  0  0  1];  % Transformationsmatrix
end

% Transformation
% ==============
q  = A*p;                                              % gedrehte Punkte       

% Graphik
% =======
mnn = min(min([p q]));
mxx = max(max([p q]));
fh = fullscreen('Drehung');
if m==2
   patch(p(1,j),p(2,j),'g')                             % 2D-Objekt ungedreht, grn
   patch(q(1,j),q(2,j),'r')                             % 2D-Objekt gedreht, rot
   axis([mnn mxx mnn mxx]); 
else
   patch(p(1,j),p(2,j),p(3,j),'g')                      % 3D-Objekt ungedreht, grn
   patch(q(1,j),q(2,j),q(3,j),'r')                      % 3D-Objekt gedreht, rot
   axis([mnn mxx mnn mxx mnn mxx]); 
   rotate3d
end
axis square
axis off
