function knf = dnfexp2knfexp(dnf,variable,opsymb)
% knf = dnfexp2knfexp(dnf,variable,opsymb)
% Wandlung der Disjunktiven Normalform einer Schaltfunktion mit den 
% Variablen variable in die Konjunktive Normalform.
%
% dnf      :  String mit der DNF
% variable :  Cell array mit den Variablen als Strings
%             Default = {'a';'b';'c';'d'}
% opsymb   :  String mit drei Symbolen fr folgende Operationen:
%             Infixsymbol fr die Und-Verknpfung
%             Infixsymbol fr die Oder-Verknpfung
%             Prfixsymbol fr die Negation
%             Default = '&|~' fr MATLAB-Syntax
% Die Terme der DNF mssen geklammert sein.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3  opsymb = '&|~'; end
if nargin<2  variable = {'a';'b';'c';'d'}; end
if nargin<1  ftab = randb(16); end

% Konstanten
% ==========
nn = length(ftab);                    % Lnge der Funktionstabelle
n  = round(log2(nn));                 % Anzahl der Variablen

% Transformation
% ==============
x = '(';
for k=1:nn
   if ftab(k)
      bv = num2binvec(k-1,n);
      for j=1:n
         if ~bv(j)  x = [x  opsymb(3)]; end
         x = [x  variable{j}  opsymb(1)];
      end
      x = [x(1:end-1)  ') '  opsymb(2)  ' ('];
   end
end
x = x(1:end-4);


