function drawbus(n)
% drawbus(n)
% Schema eines Busses mit n Zugngen (0 < n < 17)
% Durch MausKlick auf die Teilnehmer knnen die 
% Schalter bewegt werden.

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

% Default
% =======
if nargin<1  n = 4; end
if n~=round(n) | n<1 | n>16  error('drawbus:  Anzahl der Teilnehmer berprfen!'); end

% Konstanten
% ==========
clm  = colormap;
fsiz = 16;
lw   = 2;
pcol = clm(ceil((length(colormap)-1)*rand(1,n)),:);
lcol = 0.5*[1 1 1];
inc  = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdef';
while length(inc)<n  inc = [inc inc]; end

% Bild und Achsen
% ===============
fh  = fullscreen('BUS','w',0,[3 max(1,8/n); 1 1]);
axes('Position',[0 0 1 1])
axis([0 10*n+2 -1 13]);  axis off

% Graphik
% =======
bus = line([0 10*n+2],[0 0],'LineWidth',3,'Color',lcol);
for k=1:n
   h  = patch(10*k-7.5+[0 6 6 0],6+[0 0 6 6],pcol(k,:));
   text(10*k-4.5,9,inc(k),'FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','middle','FontUnits','normalized')
   bus = [bus line(10*k-6+[0 3],[0 0],'LineStyle','none','Marker','.','MarkerSize',30,'Color',lcol),...   
              drawarr([10*k-3 0],[10*k-3 6],lcol,2,0.5),...
              line(10*k-6+[0 0 0.3],[0 2 2],'LineWidth',lw,'Color',lcol)];
   h  = [h line(10*k-6,4,'LineStyle','none','Marker','o','MarkerSize',8,'Color',pcol(k,:))];
   h  = [h drawarr([10*k-6 6],[10*k-6 4.2],pcol(k,:),2,0.5)];
   h  = [h line(10*k-6+[0.1 1],[3.8 2],'LineWidth',lw,'Color',pcol(k,:))];
   h  = [h line(10*k-6+[0 0.2],[3.8 2],'LineWidth',lw,'Color',pcol(k,:),'Visible','off')];
   hd{k} = h;
end

% Interaktion
% ===========
for k=1:n
   ud.h   = hd{k};
   ud.on  = hd{k}(end);
   ud.off = hd{k}(end-1);
   ud.k   = k;
   set(hd{k},'UserData',ud,'ButtonDownFcn',@drawbusbdf)
end
fud.bus = bus;
fud.con = zeros(1,n);
fud.col = pcol;
set(fh,'UserData',fud,'KeyPressFcn',@drawbuskpf);

%---------------------------------------------------------------------------------------

function drawbusbdf(obj,eventdata)
oc = 0.8*[1 1 1];
fh = gcf;
fud = get(fh,'UserData');
co = gco;
tp = get(co,'type');
ud  = get(co,'UserData');
k   = ud.k;
if strcmp(get(ud.on,'Visible'),'off')
   an  = 'on';
   fud.con(k) = 1;
   if sum(fud.con)>1
      nc = 'k';
   else
      nc = fud.col(k,:);
   end
else 
   an  = 'off';
   fud.con(k) = 0;
   if sum(fud.con)>1
      nc = 'k';
   elseif sum(fud.con)==1
      nc = fud.col(find(fud.con),:); 
   else
      nc = oc;
   end
end
set(fh,'UserData',fud)
set(ud.on,'Visible',an); 
if strcmp(an,'on')
   set(ud.off,'Visible','off');
else 
   set(ud.off,'Visible','on');
end
for k=1:length(fud.bus)
   switch get(fud.bus(k),'type')
   case 'line', set(fud.bus(k),'Color',nc)
   case 'patch', set(fud.bus(k),'FaceColor',nc)
   end
end

%---------------------------------------------------------------------------------------

function drawbuskpf(obj,eventdata)
c  = get(gcf,'CurrentCharacter');
if double(c)==27  msvdelfig;   end
if double(c)==127 delete(gcf); end
