function fh = drawdelay(binexpr,delays,insigs)
% fh = drawdelay(binexpr,delays,insigs)
% Darstellung der Input- und Output-Signale fr die 
% Schaltfunktion mit dem schaltalgebraischen Ausdruck binexpr. 
%
% binexpr   : String des vollstndig geklammerten schaltalgebraischen 
%             Ausdrucks in MATLAB-Syntax (berzhlige Klammern sind verboten)
%             Folgende Variablennamen sind nicht erlaubt:
%             kkkkk,binexpr,eval,sprintf,variables sowie alle 
%             Schlsselwrter von MATLAB wie for, end, ...
%
% delays    : Vektor der Verzgerungen der logischen Glieder in der 
%             Reihenfolge ihres Auftretens beim Traversieren des zu binexpr 
%             gehrigen Baumes (ganzzahlig)
%
% insigs    : Matrix der abgetasteten Inputsignale mit je einer Zeile 
%             pro Variable

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

% Konstanten und Defaults
% =======================
[fname,fwgt,sz,pres] = getfont;
variables = symvar(binexpr);
nv = length(variables);
if nargin<3  insigs = [num2binmat((nfach(0:2^nv-1,10))',nv)',zeros(nv,10)]; end
if nargin<2  delays = ones(1,16384); end
dly = round(delays);
if any(dly~=delays)  
   warning('drawdelay: Delays werden gerundet!'); 
   delays = dly;   
end

% Ausdruck in Prfixnotation bringen
% ==================================
s = binexpr2prefix(binexpr);

% Delay-Funktion einfgen
% =======================
s = strrep(s,') ','); ');
s = strrep(s,'any','|');
s = strrep(s,'all','&');
s = strrep(s,'gfa','+');
for k=1:nv  lv(k) = length(variables{k}); end
[dummy,iv] = sort(lv);
for k=length(iv):-1:1
   s = strrep(s,variables{iv(k)},['!(' variables{iv(k)} ',---);']);
end
s = strrep(s,'|','any');
s = strrep(s,'&','all');
s = strrep(s,'+','gfa');
s = strrep(s,'!','delay');

% Verzgerungen einsetzen
% =======================
s  = [s '   '];
dl = [];
c  = 0;
mc = 0;
j  = 0;
for k=1:length(s)-2
   if s(k)=='['  
      j  = j + 1;
      dl = [dl delays(j)];
      c  = sum(dl);
      mc = max(mc,c);
   elseif s(k)==']'
      dl = dl(1:end-1);
      c  = sum(dl);
   elseif s(k:k+2)=='---'
      s(k:k+2) = sprintf('%3d',c);
   end
   dt(k) = c;
end
s = s(1:end-3);

% Zuweisungen der Inputsignale
% ============================
for kkkkk=1:nv
   eval([variables{kkkkk} ' = insigs(kkkkk,:);']);
end

% Berechnung des Ausgabesignals
% =============================
y = eval(s);

% Graphik
% =======
if pres  fpos = [1 1; 1 1]; else fpos = [2 2; 1 1]; end
m  = nv + 4;
fh = fullscreen('Delay','w',0,fpos);
[dum,ns] = size(insigs);
axes('Position',[0 0.9 1 0.1])
text(0.5,0.5,sprintf('Zeitverhalten der Schaltfunktion  %s',binexpr),...
     'FontSize',sz+4,'HorizontalAlignment','center','VerticalAlignment','middle','FontUnits','normalized')
axis off
for k=1:nv
   axes('Position',[0 0.9-k/m 1 0.5/m])
   drawbinsig(insigs(k,:),0,1,'LineWidth',2.5);
   line([0 ns],[0 0],'Color',0.7*[1 1 1])
   text(0.015*ns,0.5,variables{k},'FontSize',sz+2,'FontUnits','normalized')
   set(gca,'XLim',[0 ns]); axis off
end
axes('Position',[0 1/m 1 0.5/m])
if length(y)>mc+1
   drawbinsig(y(mc+1:end),mc,1,'LineWidth',2.5,'Color','r');
end
line([0 mc],y(min(end,mc+1))*[1 1],'LineStyle',':','Color','r');
line([0 ns],[0 0],'Color',0.7*[1 1 1])
text(0.015*ns,0.5,'Resultat','FontSize',sz+2,'FontUnits','normalized')
set(gca,'XLim',[0 ns]); axis off
set(fh,'WindowButtonDownFcn','set(gcf,''Pointer'',''fullcrosshair'')')
set(fh,'WindowButtonUpFcn','set(gcf,''Pointer'',''arrow'')')
