function drawearlelatch(mode)
% drawearlelatch(mode)
% Graphik des Earle-Latch
% mode = 0  :  Schaltbild (Default)
% mode = 1  :  Mealy-Modell

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Default
% =======
if nargin<1  mode = 0; end

% Konstanten
% ==========
fsz = 15;               % FontSize
ms  = 18;               % MarkerSize
msn = 30;               % MarkerSize fr Negation
lw  = 1.2;              % LineWidth
col = 'k';              % Color
if mode  yr = 1.5; else yr = 2.5; end

% Fenster
% =======
fh = fullscreen('Earle-Latch','w',0,[1.5 2; 1 1]);
axes('Position',[0.05 0.05 0.9 0.9])
axis([0 11 yr-1.5 12]); axis square; axis off

% Graphik
% =======
if mode  
   drawcsq([6,yr],2,0.7*[1 1 1],'\tau',fsz+5);
else
   line([5 7],yr+[0 0],            'Color',col,'LineWidth',lw)
end
drawgate(8, 7.5,3,'NAND',          'Color',col,'LineWidth',lw);
drawgate(5, 4.5,2,'NAND',          'Color',col,'LineWidth',lw);
drawgate(5, 7.5,2,'NAND',          'Color',col,'LineWidth',lw);
drawgate(5,10.5,2,'NAND',          'Color',col,'LineWidth',lw);
line([0 4],5+[0 0],                'Color',col,'LineWidth',lw)
line([0 3],9+[0 0],                'Color',col,'LineWidth',lw)
line([2 2 4],[5 11 11],            'Color',col,'LineWidth',lw)
line([4 3 3 4],[8 8 10 10],        'Color',col,'LineWidth',lw)
line([5 3 3 4],[yr yr 7 7],        'Color',col,'LineWidth',lw)
line([3 4],[4 4],                  'Color',col,'LineWidth',lw)
line([5 6 6 7],[10.5 10.5 8.5 8.5],'Color',col,'LineWidth',lw)
line([5 7],[7.5 7.5],              'Color',col,'LineWidth',lw)
line([5 6 6 7],[4.5 4.5 6.5 6.5],  'Color',col,'LineWidth',lw)
line([8 11],[7.5 7.5],             'Color',col,'LineWidth',lw)
line([9 9 7],[7.5 yr yr],          'Color',col,'LineWidth',lw)
line([2 3 3 9],[5 9 4 7.5],'Marker','.','LineStyle','none','MarkerSize',ms, 'Color',col)
line([4],[5],              'Marker','.','LineStyle','none','MarkerSize',msn,'Color',col)

% Beschriftung
% ============
if mode
   text(0,9,   'x_2 (D)','FontSize',fsz,'Color',col,'HorizontalAlignment','left',  'VerticalAlignment','bottom','FontUnits','normalized')
   text(0,5,   'x_1 (C)','FontSize',fsz,'Color',col,'HorizontalAlignment','left',  'VerticalAlignment','bottom','FontUnits','normalized')
   text(11,7.5,'y_1 (Q)','FontSize',fsz,'Color',col,'HorizontalAlignment','right', 'VerticalAlignment','bottom','FontUnits','normalized')
   text(4,yr,  'z',      'FontSize',fsz,'Color',col,'HorizontalAlignment','center','VerticalAlignment','bottom','FontUnits','normalized')
   text(8,yr, 'z_{next}','FontSize',fsz,'Color',col,'HorizontalAlignment','center','VerticalAlignment','bottom','FontUnits','normalized')
else
   text(0,9,   'D',      'FontSize',fsz,'Color',col,'HorizontalAlignment','left',  'VerticalAlignment','bottom','FontUnits','normalized')
   text(0,5,   'C',      'FontSize',fsz,'Color',col,'HorizontalAlignment','left',  'VerticalAlignment','bottom','FontUnits','normalized')
   text(11,7.5,'Q',      'FontSize',fsz,'Color',col,'HorizontalAlignment','right', 'VerticalAlignment','bottom','FontUnits','normalized')
end