function drawetdff(mode,z)
% drawetdff(mode,z)
% Schaltbild eines flankengesteuerten-D-Flipflops
% mode = '+' : Vorderflankensteuerung
% mode = '-' : Rckflankensteuerung (Default)
%    z = 0   : nur Schaltbild (Default) 
%    z~= 0   : Zustandsvariable eingezeichnet ins Schaltbild 

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Default
% =======
if nargin<1  mode = '-'; end

% Konstanten
% ==========
fsz = 18;               % FontSize
ms  = 18;               % MarkerSize
lw  = 1.2;              % LineWidth
col = 'k';              % Color
switch mode
   case '+',  typ = 'NAND';
   case '-',  typ = 'NOR';
   otherwise, typ = 'NAND';
end
   
% Fenster
% =======
fh = fullscreen('D-FF','w',0,[2 2.2; 1 1]);
axes('Position',[0.05 0.05 0.9 0.9])
axis([-10 8 -2.5 11]); axis off

% Graphik
% =======
drawgate(-5,0.5,2,typ,'Color',col,'LineWidth',lw);
drawgate(-5,8,  2,typ,'Color',col,'LineWidth',lw);
drawgate(0,1.5,3, typ,'Color',col,'LineWidth',lw);
drawgate(0,7.5,2, typ,'Color',col,'LineWidth',lw);
drawgate(4,2,2,   typ,'Color',col,'LineWidth',lw);
drawgate(4,7,2,   typ,'Color',col,'LineWidth',lw);
line([-10 -6],[1 1],  'Color',col,'LineWidth',lw)
line([-6 -7 -7 1 1],[8.5 8.5 10 10 7.5],'Color',col,'LineWidth',lw)
line([-6 -7 -7 -4 -4],[7.5 7.5 5.5 3.5 0.5],'Color',col,'LineWidth',lw)
line([-6 -7 -7 1 1],[0 0 -1.5 -1.5 1.5],'Color',col,'LineWidth',lw)
line([1 1 -2 -2 -1],[7.5 5.5 3.5 2.5 2.5],'Color',col,'LineWidth',lw)
line([-5 -1],8+[0 0],  'Color',col,'LineWidth',lw)
line([-5 -1],0.5+[0 0],'Color',col,'LineWidth',lw)
line([-10 -3],4.5+[0 0],'Color',col,'LineWidth',lw)
line([-1 -3 -3 -1],[1.5 1.5 7 7],'Color',col,'LineWidth',lw)
line([0 3],1.5+[0 0],'Color',col,'LineWidth',lw)
line([0 3],7.5+[0 0],'Color',col,'LineWidth',lw)
line([4 7],2+[0 0],  'Color',col,'LineWidth',lw)
line([4 7],7+[0 0],  'Color',col,'LineWidth',lw)
line([5 5 2 2 3],[2 3.5 5.5 6.5 6.5],'Color',col,'LineWidth',lw)
line([5 5 2 2 3],[7 5.5 3.5 2.5 2.5],'Color',col,'LineWidth',lw)
line([-4 1 -3 1 5 5],[0.5 1.5 4.5 7.5 2 7],'Marker','.','LineStyle','none','MarkerSize',ms,'Color',col)

% Beschriftung
% ============
text(-10,5,'Takt', 'FontSize',fsz,'Color',col,'HorizontalAlignment','left','VerticalAlignment','middle','FontUnits','normalized')
text(-10,1.5,'D',  'FontSize',fsz,'Color',col,'HorizontalAlignment','left','VerticalAlignment','middle','FontUnits','normalized')
text(6.5,2.5,'Q''','FontSize',fsz,'Color',col,'HorizontalAlignment','center','VerticalAlignment','middle','FontUnits','normalized')
text(6.5,7.5,'Q',  'FontSize',fsz,'Color',col,'HorizontalAlignment','center','VerticalAlignment','middle','FontUnits','normalized')

% Zustandsvariable
% ================
if nargin>1
   if z
      line([-7 -7 2 2],[-1.5 10 3.5 5.5],'Marker','.','LineStyle','none','MarkerSize',25,'Color','r')
      text(-7,-1.5,'z_1','FontSize',fsz-5,'Color','r','HorizontalAlignment','center','VerticalAlignment','top','FontUnits','normalized')
      text(-7,10,  'z_2','FontSize',fsz-5,'Color','r','HorizontalAlignment','center','VerticalAlignment','bottom','FontUnits','normalized')
      text(2,3.5,  'z_3','FontSize',fsz-5,'Color','r','HorizontalAlignment','right', 'VerticalAlignment','top','FontUnits','normalized')
      text(2,5.5,  'z_4','FontSize',fsz-5,'Color','r','HorizontalAlignment','right', 'VerticalAlignment','bottom','FontUnits','normalized')
   end
end