function lh = drawgate(x,y,nin,typ,varargin)
% lh = drawgate(pos,nin,typ,varargin)
% zeichnet ein Gatter mit nin Inputs und Ausgang an der Stelle [x,y].
% Als Typ sind vorgesehen: 
% 'AND','and','ALL','all','&';  'NAND','nand','~ALL','~all','~&'; 
% 'OR','or','ANY,'any','|';     'NOR','nor','~ANY,'~any','~|'; 
% 'XOR';  XNOR;  'NOT','not','~';  'BUF','buf';
% Nach dem vierten Parameter knnen alle line properties gesetzt werden.
% Zurckgegeben wird das line handle.
% Einheiten:  horizontale Breite = 1;  vertikaler Abstand der Inputs = 1 .

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Konstanten und Defaults
% =======================
if nargin<4 typ = 'AND'; end
if nargin<3 nin = 2;     end
ms  = 30;
c30 = cos(30/180*pi);

% Gatterrand
% ==========
if nin==1 | ismember(typ,{'NOT','not','~','BUF','buf'})
   lh = line(x+[0 -1 -1 0],y+[0 +0.5 -0.5 0],varargin{:});
else
   w = (0:5:90)/180*pi;
   c = cos(w);
   s = sin(w);
   lh = line(x+[c-1 c(end:-1:1)-1 0],y+[s+(nin-2)/2 -(s(end:-1:1)+(nin-2)/2) (nin-2)/2],varargin{:});
end

% Gattertyp
% =========
switch typ
case {'NOT','not','~'}
   lh = [lh line(x,y,'Marker','.','LineStyle','none','MarkerSize',ms,varargin{:})];  
case {'NAND','nand','~ALL','~all','~&'}
   lh = [lh line(x,y,'Marker','.','LineStyle','none','MarkerSize',ms,varargin{:})];  
case {'OR','or','ANY','any','|'}
   dx = [c30 ones(1,nin-2) c30];
   for k=1:nin
      lh = [lh; line(x+[0 dx(k)]-1,y+(nin+1)/2-k*[1 1],varargin{:})];
   end
case {'NOR','nor','~ANY','~any','~|'}
   dx = [c30 ones(1,nin-2) c30];
   for k=1:nin
      lh = [lh; line(x+[0 dx(k)]-1,y+(nin+1)/2-k*[1 1],varargin{:})];
   end
   lh = [lh; line(x,y,'Marker','.','LineStyle','none','MarkerSize',ms,varargin{:})];
case {'XOR','xor','+'}
   fs = 1/diff(get(gca,'YLim'));
   text(x-0.51,y,' \oplus ','FontUnits','normalized','FontSize',fs,...
        'HorizontalAlignment','center','VerticalAlignment','Middle')
case {'XNOR','xnor','~+'}
   fs = 1/diff(get(gca,'YLim'));
   text(x-0.55,y,'\oplus','FontUnits','normalized','FontSize',fs,...
        'HorizontalAlignment','center','VerticalAlignment','Middle')
   lh = [lh; line(x,y,'Marker','.','LineStyle','none','MarkerSize',ms,varargin{:})];
end