function fh = drawswsigs2(binexpr,variables,delays,insigs)
% fh = drawswsigs(binexpr,variables,delays,insigs)
% Darstellung der Input- und Zustands-Signale fr das Schaltwerk 
% mit den schaltalgebraischen Ausdrcken binexpr fr die berfhrungsfunktion. 
%
% binexpr   : CellArray von Strings der vollstndig geklammerten schaltalgebraischen 
%             Ausdrcke in MATLAB-Syntax (berzhlige Klammern sind verboten)
%             in der Reihenfolge der Zustandsvariablen.
%             Die Zustandsvariablen heissen fest  z1, z2, z3, ...  Sie knnen so 
%             in der Ausdrcken verwendet werden.
%
% variables : CellArray der Strings der Inputvariablen.
%             Folgende Bezeichner der Inputvariablen sind nicht erlaubt:
%             kkkkk,t,z,binexpr,eval,sprintf,variables sowie alle 
%             Schlsselwrter von MATLAB wie for, end, ...
%
% delays    : CellArray der Vektoren der Verzgerungen der logischen Glieder in 
%             der Reihenfolge ihres Auftretens beim Traversieren des zu binexpr 
%             gehrigen Baumes (ganzzahlig).  Die Verzgerungen der Ausdrcke 
%             mssen auf dem Rckkopplungsweg grsser als 0 sein.
%
% insigs    : Matrix der abgetasteten Inputsignale mit je einer Zeile pro Variable
%             in der Reihenfolge wie in variables.

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

% Konstanten und Defaults
% =======================
nz = length(binexpr);          % Anzahl der Ausdrcke (bzw. Zustandsvariablen)
nv = length(variables);        % Anzahl der Inputvariablen
%fsiz = ceil(10 + 8/nz);        % FontSize
fsiz = ceil(15);               % FontSize
[nn,nt] = size(insigs);        % Lnge der Inputsequenz
if nn~=nv  warning('DRAWSWSIGS: Anzahl der Variablen und der zugehrigen Signale verschieden'); end
if nargin<4  insigs = [num2binmat((nfach(0:2^nv-1,10))',nv)',zeros(nv,10)]; end
for a=1:nz
   if nargin<3  delays{a} = ceil(4*rand(1,2^(nv-1))); end
   dly{a} = round(delays{a});
   if any(dly{a}~=delays{a})  
      warning('drawdelay: Delays werden gerundet!'); 
      delays{a} = dly{a};   
   end
end

% Ausdrcke in Prfixnotation bringen
% ===================================
for a=1:nz
   s{a} = binexpr2prefix(binexpr{a});
end

% Delay-Funktion einfgen
% =======================
for a=1:nz
   s{a} = strrep(s{a},') ','); ');
   s{a} = strrep(s{a},'any','|');
   s{a} = strrep(s{a},'all','&');
   s{a} = strrep(s{a},'gfa','+');
   for k=1:nv  lv(k) = length(variables{k}); end
   [dummy,iv] = sort(lv);
   for k=length(iv):-1:1
      s{a} = strrep(s{a},variables{iv(k)},[variables{iv(k)} '(t- ---);']);
   end
   for k=1:nz
      s{a} = strrep(s{a},['z' num2str(k)],['z' num2str(k) '(t- ---);']);
   end
   s{a} = strrep(s{a},'|','any');
   s{a} = strrep(s{a},'&','all');
   s{a} = strrep(s{a},'+','gfa');
end

% Verzgerungen einsetzen
% =======================
for a=1:nz
   s{a}  = [s{a} '   '];
   dl = [];
   c  = 0;
   mc(a) = 0;
   j  = 0;
   for k=1:length(s{a})-2
      if s{a}(k)=='['  
         j  = j + 1;
         dl = [dl delays{a}(j)];
         c  = sum(dl);
         mc(a) = max(mc(a),c);
      elseif s{a}(k)==']'
         dl = dl(1:end-1);
         c  = sum(dl);
      elseif s{a}(k:k+2)=='---'
         s{a}(k:k+2) = sprintf('%3d',c);
      end
      dt(k) = c;
   end
   s{a} = s{a}(1:end-3);
end
md = max(mc);                  % maximale Verzgerung

% Berechnung der Zustandssignale
% ==============================
for kkkkk=1:nv
   eval([variables{kkkkk} ' = insigs(kkkkk,:);']);
end
for kkkkk=1:nz
   eval(['z' num2str(kkkkk) '(1:md+1) = randb;']);
end
for t=1+md:nt
   for kkkkk=1:nz
      eval(['z' num2str(kkkkk) '(t) = ' s{kkkkk} ';']);
   end
end
for k=1:nz
   eval(['zz(k,:) = z' num2str(k) ';']); 
end

% Graphik
% =======
m  = nv + nz + 4;
fh = fullscreen('Delay');
[dum,ns] = size(insigs);
axes('Position',[0 0.9 1 0.1])
titel = ['Zeitverhalten eines Schaltwerks mit  z1 = ' binexpr{1}];
for a=2:nz
   titel = [titel ' ;  z' num2str(a) ' = ' binexpr{a}]; 
end
text(0.5,0.5,titel,...
     'FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','middle','FontUnits','normalized')
axis off
for k=1:nv
   axes('Position',[0 0.9-k/m 1 0.5/m])
   drawbinsig(insigs(k,:),0,1,'LineWidth',2.5);
   line([0 ns],[0 0],'Color',0.7*[1 1 1])
   text(0.015*ns,0.5,variables{k},'FontSize',fsiz,'FontUnits','normalized')
   axis([0 ns 0 1]); axis off
end
for k=1:nz
   axes('Position',[0 0.9-(nv+k)/m 1 0.5/m])
   if length(zz(k,:))>mc(k)+1
      drawbinsig(zz(k,mc(k)+1:end),mc(k),1,'LineWidth',2.5,'Color','r');
   end
   line([0 mc(k)],zz(k,min(end,mc(k)+1))*[1 1],'Color',0.7*[1 1 1])
   line([0 ns],[0 0],'Color',0.7*[1 1 1])
   text(0.015*ns,0.5,['z' num2str(k)],'FontSize',fsiz,'FontUnits','normalized')
   axis([0 ns 0 1]); axis off
end
set(fh,'WindowButtonDownFcn','set(gcf,''Pointer'',''fullcrosshair'')')
set(fh,'WindowButtonUpFcn',  'set(gcf,''Pointer'',''arrow'')')
