function nft = elimeqs(ft)
% nft = elimeqs(ft)
% Eliminierung quivalenter Zustnde eines Schaltwerks.
% ft  :    m*n-Matrix, die in der k-ten Zeile die Indices der
%          Folgezustnde fr den Zustand mit dem Index k enthlt
%          und zwar in der Spalte j den zur Eingangsbelegung mit 
%          dem Index j gehrigen Folgezustand.
%          ft kann auch ein StructureArray sein von zwei Tabellen:
%          die erste wie oben beschrieben fr die bergangsfunktion,
%          die zweite fr die Ausgabefunktion. Letztere ist im Falle 
%          eines Moore-Schaltwerks ein m-Spaltenvektor und im Falle 
%          eines Mealy-Schaltwerks eine m*n-Matrix von Indices in yname.
%          Do'ntCares werden durch -1 in den Tabellen angegeben. 
%          Do'ntCares werden beim Vergleich zweier Zustnde als 
%          gleich angesehen.

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Datenstruktur von ft
% ====================
if iscell(ft)  
   fyt = [ft{1} ft{2}]; 
else
   fyt = ft;
end
[ns,ni] = size(fyt);

% quivalente Zustnde
% ====================
elim = 1;
while elim
   fyt
   eq = 0;
   for k=1:ns
      for j=k+1:ns
         if all(fyt(k,:)==fyt(j,:))  eq = 1; end
         if eq break; end
      end
      if eq break; end
   end
   if eq
      fyt = [fyt(1:j-1,:);fyt(j+1:end,:)];
      fyt(find(fyt==j)) = k;
      ns = ns - 1;
   else
      elim = 0;
   end
end

% Ausgabe
% =======
if iscell(ft)
   [ns,ni] = size(ft{1});
   nft = {fyt(:,1:ni);fyt(:,ni+1:end)};
else
   nft = fyt;
end