function x0 = fehl_vert(n,p,ysc)
% x0 = fehl_vert(n,p,ysc)
% Hufigkeitsverteilung von Bitfehlern in n-Bit-Wrtern 
% bei der Bitfehlerwahrscheinlichkeit p.
% Ist p eine Matrix, so werden die Diagramme in dieser 
% Matrixform angeordnet. 
% ysc = 1 : logarithmische Teilung (optional)
% Beispiele:  fehl_vert(32,[0.5,0.2; 0.1,0.01]);
%             fehl_vert(32,[0.5,0.2; 0.1,0.01],1)

% Autor: Klaus von der Heide, e-mail:  v.d.heide@on-line.de

fsiz = 12;                           % FontSize
if nargin < 3  ysc = 0; end;
if nargin < 2  p = [0.5,0.2; 0.1,0.01]; end
if nargin < 1  n = 32; end
[sp,zl] = size(p);                   % Matrixformat
pp = [];
for i = 1:sp  pp = [pp,p(i,:)]; end; % p-Matrix flach machen
x0 = [];
fh = fullscreen('Bitfehlerstatistik');
for i = 1:length(pp)                 % alle Diagramme rechnen
   for k = 0:n
      wr(k+1) = prod((k+1):n)/prod(1:(n-k))*(1-pp(i))^(n-k)*pp(i)^k;
   end;
   subplot(sp,zl,i); plot(0:n,wr,'r.');
   xlabel('r','FontSize',fsiz); 
   ylabel('Wahrscheinlichkeit fr genau r Fehler','FontSize',fsiz);
   title(sprintf('n = %g Bit; Bitfehlerwahrscheinlichkeit p = %g',...
                 n,pp(i)),'FontSize',fsiz);
   if ysc == 1  set(gca,'YScale','log'); axis([0 n 1e-12 1]);
      else axis([0 n 0 1]);
   end;
   x0 = [x0;wr];                     % Ausgabe der Funktion
end;
