function fsmtiming
% fsmtiming
% Graphik fr das Zeitverhalten eines Schaltwerks.

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

% Konstanten
% ==========
fsiz  = 16;
fst   = 1.3*fsiz;
pos1  = [0.15 0 1 1];
pos2  = [0 0.05 0.53 0.50];
pos3  = [0 0.65 0.53 0.35];
pcol  = 0.9*[1 1 1];
lcol1 = 0.8*[0.7 1 1];
lcol2 = 0.8*[1 0.7 1];
lcol3 = 0.8*[1 1 0.7];
lcol4 = 0.8*[0.8 0.8 1];
lw    = 2.5;
H     = 3;
S     = 3;
dmx   = 4.5;
dmn   = 2.0;
tff   = 2.0;

% Schaltwerkmodell
% ================
fh = fullscreen('Timing');
axes('Position',pos1)

patch([28 38 38 28],...
      [28 28 38 38],pcol);
text(30,33,'\lambda',  'FontSize',  25,'HorizontalAlignment','center','VerticalAlignment','middle')

patch([28 38 38 28],...
      [14 14 24 24],pcol);
text(30,19,'\delta',   'FontSize',  25,'HorizontalAlignment','center','VerticalAlignment','middle')

patch([30 36 36 30],...
      [ 4  4 10 10],pcol);
   
patch([38 48 48 50 48 48 38],...
      [32 32 31 33 35 34 34],lcol1);
text(44, 35,'Output',  'FontSize',fsiz,'HorizontalAlignment','center')
text(44, 33,'y',       'FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','middle')

patch([38 42 42 38 38 36 38 38 44 44 38],...
      [18 18  8  8  9  7  5  6  6 20 20],lcol2);
text(42.3,7,'z_{next}','FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','middle')
   
patch([30 23 23 26 26 28 26 26 23 23 26 26 28 26 26 21 21 30],...
      [ 8  8 15 15 14 16 18 17 17 32 32 31 33 35 34 34  6  6],lcol3);
text(25,  7,'z',       'FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','middle')

patch([ 4 26 26 28 26 26  4],...
      [21 21 20 22 24 23 23],lcol4);
text(10, 24,'Input',   'FontSize',fsiz,'HorizontalAlignment','center')
text(10, 22,'x',       'FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','middle')

axis([4 50 -6 40]);  axis square; axis off

drawarr([33 -5],[33 4],'k',2,1);
text(33.3,-2,'Takt',   'FontSize',fst,'HorizontalAlignment','left',  'VerticalAlignment','baseline')

drawarr([33 4],[30 7],'r',2.5,1);
text(32,6,'\tau_{ff}', 'FontSize',fst,'HorizontalAlignment','left',  'VerticalAlignment','baseline','Color','r')

drawarr([28 16],[38 19],'r',2.5,1);
text(33,16,'\tau_\delta','FontSize',fst,'HorizontalAlignment','left','VerticalAlignment','baseline','Color','r')

% Internes Zeitverhalten
% ======================
axes('Position',pos2)
axis([-3 +13 -1 +11]); axis off
patch([-S +H +H -S],   [0 0 2 2],'r')
patch([4 6.5 6.5 4],   [0 0 2 2],0.7*[1 1 1])
lax = legend('z_{next}: nderung verboten','Zeitbereich der nderungen');
set(lax,'Position',[0.361 0.102 0.202 0.061])
patch(10+[-S +H +H -S],[0 0 2 2],'r')
line([-3 +13],[0 0],'LineWidth',lw,'Color','k')
text(5,0,'Zeit','FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','top','Color','k')

line([0 10; 0 10],[0 0; 10 10],'LineWidth',lw,'Color','b')
text(-0.2,5,'Takt','FontSize',fsiz,'HorizontalAlignment','right','VerticalAlignment','baseline','Color','b')
text(10.2,5.4,'nchster','FontSize',fsiz,'HorizontalAlignment','left','VerticalAlignment','baseline','Color','b')
text(10.6,4.7,'Takt','FontSize',fsiz,'HorizontalAlignment','left','VerticalAlignment','baseline','Color','b')

drawarr([0 8],[tff 8],'r',lw,0.5);
text(tff/2,8.7,'\tau_{ff}','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

drawarr([tff 10],[tff+dmx 10],'r',lw,0.5);
text(tff+dmx/2,10.7,'\tau_{\deltamax}','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

drawarr([tff 6],[tff+dmn 6],'r',lw,0.5);
text(tff+dmn/2,6.7,'\tau_{\deltamin}','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

drawarr([10-S 4],[10 4],'r',lw,0.5);
text(10-S/2,4.2,'S','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

drawarr([0 4],[H 4],'r',lw,0.5);
text(H/2,4.2,'H','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

line([2 2],[6 10],'LineStyle',':','Color','m')
line(H*[1 1],[0 4],'LineStyle',':','Color','m')
line((10-S)*[1 1],[0 4],'LineStyle',':','Color','m')
line(4*[1 1],[0 6],'LineStyle',':','Color','m')
line(6.5*[1 1],[0 10],'LineStyle',':','Color','m')

% Externes Zeitverhalten
% ======================
axes('Position',pos3)
axis([-3 +13 -1 +7]); axis off
patch([-S-dmx H-dmn H-dmn -S-dmx],[0 0 2 2],'r')
patch([H-dmn 10-S-dmx 10-S-dmx H-dmn],[0 0 2 2],'g')
lax = legend('x: nderung verboten','nderungen erlaubt');
set(lax,'Position',[0.363 0.711 0.167 0.052])
patch(10+[-S-dmx H-dmn H-dmn -S-dmx],[0 0 2 2],'r')
line([-3 +13],[0 0],'LineWidth',lw,'Color','k')
text(5,0,'Zeit','FontSize',fsiz,'HorizontalAlignment','center','VerticalAlignment','top','Color','k')

line([0 10; 0 10],[0 0; 6 6],'LineWidth',lw,'Color','b')
text(-0.2,3,'Takt','FontSize',fsiz,'HorizontalAlignment','right','VerticalAlignment','baseline','Color','b')
text(10.2,3.4,'nchster','FontSize',fsiz,'HorizontalAlignment','left','VerticalAlignment','baseline','Color','b')
text(10.6,2.7,'Takt','FontSize',fsiz,'HorizontalAlignment','left','VerticalAlignment','baseline','Color','b')

drawarr([10-S-dmx 3],[10-S 3],'r',lw,0.5);
text(10-S-dmx/2,3.6,'\tau_{\deltamax}','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

drawarr([H-dmn 4.25],[H 4.25],'r',lw,0.5);
text(H-dmn/2,4.85,'\tau_{\deltamin}','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

drawarr([10-S 5.5],[10 5.5],'r',lw,0.5);
text(10-S/2,5.7,'S','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

drawarr([0 5.5],[H 5.5],'r',lw,0.5);
text(H/2,5.7,'H','FontSize',fst,'HorizontalAlignment','center','VerticalAlignment','baseline','Color','r')

line((10-S)*[1 1],[3 5.5],'LineStyle',':','Color','m')
line(H*[1 1],[4.25 5.5],'LineStyle',':','Color','m')
line((H-dmn)*[1 1],[0 4.25],'LineStyle',':','Color','m')
line((10-S-dmx)*[1 1],[0 3],'LineStyle',':','Color','m')


