function gf = gf_new(p, n, irr_poly)
% Anlegen eines Objektes fr GF(p^n)
% irr_poly muss vom Grad n-1 und irreduzibel mod p sein.

% Autor: Peter Grotrian, grotrian@acm.org

% p, n und irrpoly werden direkt in gf bernommen
% auerdem werden noch q=p^n und die Primfaktorzerlegung
% von q-1 = prod( pr.^ex )in pr und ex gespeichert

% Tabelle irreduzibler Trinome vom Grad n ber Z2. 1 <= n <= 31
% Die Polynome sind von der Form x^n + x^k + 1.
% Eintrge mit k=0 bedeuten, da es fr dieses n kein irr. Trinom gibt.
%
%                            1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3 3
%          1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
irr_k_2 = [0 1 1 1 2 1 1 0 1 3 2 3 0 5 1 0 3 3 0 3 2 1 5 0 3 0 0 1 2 1 3];

if (nargin == 2) & (p == 2) & (n <= length (irr_k_2) & (irr_k_2(n) > 0))
   k = irr_k_2(n);
 	irr_poly = zeros(1, n+1);
   irr_poly([1, n-k+1, n+1]) = [1 1 1];
else
   irr_poly = poly_find_primitive(n, p);
end

if ~isequal (size (irr_poly), [1, n+1])
   sprintf ('Das irreduzible Polynom mu vom Grad n-1=%d sein!', n)
   return;
end

if ~poly_is_irreducible(irr_poly, p)
   'Das Polynom ist nicht irreduzibel!'
   return;
end


gf.p = p;
gf.n = n;
gf.irr_poly = irr_poly;
gf.q = p^n;
gf.q1 = gf.q-1;
oldf = -1;
gf.pr = [];
gf.ex = [];
if isstudent & (gf.q1 > 2^28)
   'q-1 not factored, because of student edition limitation'
   return
end

facs = factor (gf.q-1);
for f = facs
   if f ~= oldf
      gf.pr = [gf.pr, f];
      gf.ex = [gf.ex, 1];
      oldf = f;
   else
      gf.ex(end) = gf.ex(end)+1;
   end
end
