function harddecodeber(n,d,pl)
% harddecodeber(n,d,pl)
% Graphik fr die Wort- und Bitfehlerrate eines Blockcodes
% n :  Lnge des Codewortes             (Default: 7)
% d :  Minimalabstand des Codes         (Default: 3)
% pl:  Die Darstellung erfolgt ber der Bitfehlerwarscheinlichkeit 
%      des Kanals im Bereich  0 ... pl  (Default: 0.5)

% Defaults
% ========
if nargin<3  pl = 0.5; end
if nargin<2   d = 3;   end
if nargin<1   n = 7;   end

% Konstanten
% ==========
fsiz = 15;                         % FontSize
lw = 2;                            % LineWidth
rr = floor(d/2) + 1;               % Mindestanzahl nicht korrigierbarer Bits
m  = 200;                          % Anzahl der Sttzstellen der Graphik
p  = pl*(0:m)/m;                   % Sttzstellen

% Wortfehlerrate
% ==============
for j=1:length(p) 
   x(j) = binomialsum(n,rr,p(j)); 
end

% Bitfehlerrate
% =============
if n==d  y = x; else y = x/2; end  % sehr einfaches Modell fr p => 0.5 falsch

% Bildgrsse
% ==========
[fnam,fwgt,fsz,pres] = getfont;
if pres
   figp = [0 1 0 1];
else
   figp = [0 0.5 0.5 1];
end

% Graphik
% =======
fh = fullscreen('BER','w',0,figp); 
line(p,p,'Color','k','LineWidth',lw)
line(p,x,'Color','b','LineWidth',lw)
line(p,y,'Color','r','LineWidth',lw)
legend('uncodierte bertragung','Wortfehlerrate','Bitfehlerrate (ungenau bei p => 0.5)',2)
xlabel('Kanal-Bitfehlerrate','FontSize',fsiz)
ylabel('Wort- und Bitfehlerrate','FontSize',fsiz)
title(sprintf('Wort- und Bitfehlerrate fr einen binren Blockcode\nder Lnge%4d und des Minimalabstands%3d',n,d),'FontSize',fsiz)
axis([0 pl 0 1])
