function w = ieee754(x,type)
% w = ieee754(x,type)
% Es wird das Bitmuster der real-Reprsentation von x ausgegeben. 
% type = 1 :  short real
% type = 2 :  long real
% type = 3 :  temporary real
%!!! Vorsicht: Diese Funktion soll nur der Demonstration dienen.
%!!! Wegen der Benutzung der Rechner-internen Arithmetik kann 
%!!! das Resultat in Grenzfllen falsch sein.  Insbesondere ist 
%!!! die Darstellung in temporary real immer unprzise.

%  Autor:  Klaus von der Heide,  v.d.heide@on-line.de

if nargin<2  type = 1; end

switch type
   case 1
      n  = 32;                  % Wortlnge
      m  = 24;                  % Index des LSB des Exponenten;
      kx = 2^-149;              % kleinste positive Zahl   
   case 2
      n = 64;                   % Wortlnge
      m  = 53;                  % Index des LSB des Exponenten;
      kx = 2^-1074;             % kleinste positive Zahl   
   case 3
      n = 80;                   % Wortlnge
      m  = 65;                  % Index des LSB des Exponenten;
      kx = 2^-16446;            % kleinste positive Zahl   
end
nm = n - m - 1;

if isnan(x)
   w(1) = '0';
   w(2:nm+2) = num2basstr(2^(nm+1)-1,2,nm+1);
   w(nm+3:n) = num2basstr(1,2,m-1);
else
   switch x
   case +inf
      w(1) = '0';
      w(2:nm+2) = num2basstr(2^(nm+1)-1,2,nm+1);
      w(nm+3:n) = num2basstr(0,2,m-1);
   case -inf
      w(1) = '1';
      w(2:nm+2) = num2basstr(2^(nm+1)-1,2,nm+1);
      w(nm+3:n) = num2basstr(0,2,m-1);
   otherwise
      a = abs(x);
      if a<kx                                 % absolut kleiner als kleinste Zahl > 0
         w(1) = '0';
         w(2:nm+2) = num2basstr(0,2,nm+1);
         w(nm+3:n) = num2basstr(0,2,m-1);
      elseif a<2^-(2^nm-2)                    % denormalisiert
         if x>0  w(1) = '0'; else w(1) = '1'; end
         ss = 0;
         tt = round(a*2^(2^nm + m - 3));
         w(2:nm+2) = num2basstr(ss,2,nm+1);
         w(nm+3:n) = num2basstr(tt,2,m-1);
      elseif a>(2 - 2^-(m-1))*2^(2^nm-1)      % absolut grsser als grsste Zahl
         if x>0  w(1) = '0'; else w(1) = '1'; end
         w(2:nm+2) = num2basstr(2^(nm+1)-1,2,nm+1);
         w(nm+3:n) = num2basstr(0,2,m-1);
      else
         s  = floor(log2(a));
         ss = s + 2^nm - 1;
         tt = round(a*2^(m-s-1));
         if x<0  w(1) = '1'; else w(1) = '0'; end
         w(2:nm+2) = num2basstr(ss,2,nm+1);
         if type<3
            w(nm+3:n) = num2basstr(tt,2,m-1); % mit impliziter 1
         else
            w(nm+3:n) = ['1' num2basstr(tt/2,2,m-2)]; % explizite 1 bei temporary real
         end
      end   
   end
end
