function z = ieee754interpret(w)
% z = ieee754interpret(w)
% Interpretation eines Bitmusters der Lnge 32 bzw. 64 
% als short real bzw. long real oder temporary real nach IEEE-754
% w kann ein Bitvektor sein mit dem LSB w(1)
% oder ein Bitstring mit dem LSB w(end).

%  Autor:  Klaus von der Heide,  v.d.heide@on-line.de

n = length(w);

if n~=32 & n~=64 & n~=80  error('realinterpret: falsche Wortlnge!'); end

if ischar(w)  w = reverse(bitstring2bitvec(w)); end

switch n
   case 32,  m  = 24;           % Index des LSB des Exponenten;
   case 64,  m  = 53;           % Index des LSB des Exponenten;
   case 80,  m  = 65;           % Index des LSB des Exponenten;
end
nm = n - m - 1;

v = 1 - 2*w(n);                 % Vorzeichen

h = sum(w(m:n-1).*2.^(0:nm));   % Exponent

t = sum(w(1:m-1).*2.^(1-m:-1)); % Mantisse 

if h==0   
   s = -2^nm + 2;               % Exponent
else   
   s = h - 2^nm + 1;            % Exponent
   t = t + 1;                   % Mantisse
end

if h==2^(nm+1)-1
   if t==1
      z = v*inf;                % inf
   else
      z = NaN;                  % NaN
   end
else
   z = v*t*2^s;                 % Zahlenwert
end
