function kantorovic(expr)
% kantorovic(expr)
% Darstellung des Kantorovic-Baumes fr den Ausdruck expr.
%
% expr :    String des vollstndig geklammerten schaltalgebraischen Ausdrucks 
%           in MATLAB-Syntax (berzhlige Klammern sind verboten).
%           Folgende Operationssymbole in expr sind vorgesehen: 
%           '&' und '*' (AND), '|' (OR), '+' (XOR), '~' (NOT)
%           Negationen der Operationen werden durch einen vorangestellten /
%           gekennzeichnet.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1   expr = '(u&~v)|~(((a&b)|~c)&(d|e))'; end

% Wandlung in einen Baum
% ======================
x = binexpr2tree(expr);

% Konstanten
% ==========
r   = 0.5;                             % Radius fr Knoten
col = 0.8*[1 1 1];                     % Fllfarbe fr Knoten
lw  = 1.5;                             % Dicke der Kanten
n   = treedepth(x);                    % Tiefe des Baumes = Anzahl der Variablen
fsz = 15;                              % FontSize fr Operation

% Graphik
% =======
dy = (2^(n+0.7)+2)/(n+2)/1.333;        % vertikaler Abstand der Ebenen 
fsiz = round(400*2^-n);                % FontSize
fh = fullscreen('Kantorovic');         % Fenster
axes('Position',[0 0 1 1])             % Achsen
drawktrv(x,[0 0; 2^(n-2) dy],r,col,fsz,lw) % Graphen zeichnen
set(gca,'XLim',[-1.05 +1]*2^(n-0.9))   % Achsenskalierung
if n<3  axis(2*axis+[0 0 -1 -1]); end  % Achsenskalierung
axis square
axis off

%-------------------------------------------------------------------------------

% Lokale Funktion zum rekursiven Zeichnen eines Kantorovic-Baumes
% ===============================================================
function  drawktrv(x,pos,r,col,fsz,lw)
a  = atan(pos(2,1)/pos(2,2));
dx = r*sin(a);
dy = r*cos(a);
if iscell(x)
   drawccir(pos(1,:),r,col,x{1},fsz);
   line(pos(1,1)-[dx pos(2,1)],pos(1,2)+[dy pos(2,2)],'Color','k','LineWidth',lw)
   line(pos(1,1)+[dx pos(2,1)],pos(1,2)+[dy pos(2,2)],'Color','k','LineWidth',lw)
   drawktrv(x{2},[pos(1,1)-pos(2,1),pos(1,2)+pos(2,2);pos(2,1)/2,pos(2,2)],r,col,fsz,lw)
   drawktrv(x{3},[pos(1,1)+pos(2,1),pos(1,2)+pos(2,2);pos(2,1)/2,pos(2,2)],r,col,fsz,lw)
else
   drawsq(pos(1,:)-r,2*r,col,num2str(x),fsz);
end
