function [ftabmin,rm,sing,t0] = minrm(ftab)
% [ftabmin,rm,sing,t0] = minrm(ftab)
% Minimierung des Realisierungsaufwandes fr eine Reed-Muller-Form.
% ftab    : Vektor der Funktionstabelle mit 0 und 1 sowie -1 fr Do'nt Care.
% ftabmin : realisierte Funktionstabelle
% rm      : Reed-Muller-Koeffizienten
% sing    : wenn 1 dann ist das Resultat unbrauchbar
% t0      : Indexvektor der zu 0 gemachten Terme
%
% !!! Zur Auflsung von Widersprchen und linearen Abhngigkeiten wird 
% !!! ein suboptimales Verfahren benutzt.

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Default
% =======
if nargin<1  ftab =[1 -1  1  0  0  1  1 -1 -1  1 -1  0  0  1  0 -1]; end

% Transformationsmatrix aufstellen
% ================================
n = length(ftab);
A = 1;
Z = 0;
while n>1
   A = [A Z; A A];
   Z = [Z Z; Z Z];
   n = n/2;
end

% Transformation
% ==============
r  = rem(A*(ftab'>0),2)';      % Reed-Muller-Form mit Do'ntCares = 0
ix = find(ftab<0);             % Indices der Do'nt Cares in ftab
for k=1:length(ix)
   B(:,k) = A(:,ix(k));        % individuelle Transformation der Do'ntCares
end

% Aufwand der Terme
% =================
a = [0]; 
while length(a)<length(ftab)  a = [a 1+a]; end
[dummy,iy] = sort(a);
iy = reverse(iy);              % Termnummern sortiert nach sinkendem Aufwand
ra = reverse(a);               % Aufwand der Terme

% Minimierung
% ===========
iz = 1:length(ix);
ftabmin = [];
rm   = [];
sing = 1;
try
   t0 = iy(iz);                % Indexvektor der Null-gesetzen Terme
   C  = B(t0,:);               % Matrix des Gleichungssystems
   d  = r(t0)';                % rechte Seite des Gleichungssystems
   x  = bingausselim(C,d);     % Bestimmung der optimalen Do'ntCare-Werte
   ftabmin = ftab;             % feste Werte bernehmen
   ftabmin(ix) = x;            % berechnete Werte anstelle Do'ntCare einsetzen
   rm = rem(A*(ftabmin'>0),2)';% Reed-Muller-Form ohne Do'ntCares
   sing = 0;
catch                          % im Fall der Singularitt minrm rekursiv aufrufen
   for k=1:length(ix)
      ftb = ftab;
      ftb(ix(k)) = 0;          % Singularitt beseitigen durch Setzen einer Variablen
      [ftabmin0,rm0,sing,t0] = minrm2(ftb);
      ftb(ix(k)) = 1;          % 0 und 1 probieren
      [ftabmin1,rm1,sing,t0] = minrm2(ftb);
      if ~sing  
         if sum(rm0)<sum(rm1)  % prfen ob 0 besser als 1
            ftabmin = ftabmin0;
            rm = rm0;
         else
            ftabmin = ftabmin1;
            rm = rm1;
         end
         break;                % beenden wenn erstmalig nicht singulr 
      end
   end
end
