function bitseq = morsecode(string,lengthpar)
% bitseq = morsecode(string,lengthpar)
% transforms the character string to a bit vector of Morse symbols.
% lengthpar is a vector of 5 integers:
% lengthpar(1) :   Number of ones representing a dot.
% lengthpar(2) :   Number of ones representing a dash.
% lengthpar(3) :   Number of zeros representing the space between dots or dashes.
% lengthpar(4) :   Number of zeros between characters.
% lengthpar(5) :   Number of zeros between words.
% Example:  morsecode('CQ DE ',[1 2 1 2 4]) yields 
% [1 1 0 1 0 1 1 0 1 0 0 1 1 0 1 1 0 1 0 1 1 0 0 0 0 1 1 0 1 0 1 0 0 1 0 0 0 0]

% Autor:  K. v. d. Heide,  e-mail:  v.d.heide@on-line.de

if nargin<2 lengthpar = [1 2 1 2 4]; end
dot   = [strrep(blanks(lengthpar(1)),' ','1'),strrep(blanks(lengthpar(3)),' ','0')];
dash  = [strrep(blanks(lengthpar(2)),' ','1'),strrep(blanks(lengthpar(3)),' ','0')];
csep  = strrep(blanks(lengthpar(4)-lengthpar(3)),' ','0');
wsep  = strrep(blanks(lengthpar(5)-lengthpar(3)),' ','0');
morse = str2morse(string);
bitstr= strrep(strrep(strrep(strrep(morse,'-',dash),'.',dot),'  ',wsep),' ',csep);
bitseq= bitstr=='1';