function symb = morsesound2symbol(snd,fs,diag)
% symb = morsesound2symbol(snd,fs,diag)
% Umsetzung tnender Morsetelegraphie in punkte und Striche
% snd  :  Samplevektor
% fs   :  Abtastrate des Samplevektors in Hz
% diag :  falls dieser Parameter mit beliebigem Wert angegeben ist,
%         gibt es eine Reihe von Graphiken

if nargin<1  snd = morsesound; end
if nargin<2   fs = 4000;       end

% Konstanten
% ==========
bpm  = 160;           % Grssenordnung der Buchstaben pro Minute
wt   = 0.46;          % Wahrscheinlichkeit des getasteten Zustands
fsiz = 13;            % FontSize
g    = 30;            % Filtergrad fr Hilbertfilter
bw   = 25;            % Bandbreite des Tiefpassfilters in Hz

% Energie im Zeitbereich
% ======================
x  = snd/sqrt(snd*snd'/length(snd));  % Normierung auf mittlere Energie = 1
xa = real2analyt(x,g,0.1);            % analytisches Eingangssignal
en = abs(xa).^2;                      % Energie

% Rauschenergie
% =============
re = mean(abs(diff(en)));             % 

% Verteilung der ungefilterten Energie
% ====================================
hen = hist(en,50);

% Tiefpassfilter und Dezimation
% =============================
[koef,srate,dly,aw] = mrdesign(bw/fs,bw/fs,2,30,0);
[enf,dlay] = mrfilter(koef,srate,en);

if ~diag  return; end

% Graphische Darstellungen
% ========================
fh1 = fullscreen('Verteilung','w',2,[3 3; 1 1]);
hist(en,50)
title('Energieverteilung','FontSize',fsiz)
xlabel('normierte Energie','FontSize',fsiz)

fh2 = fullscreen('Energie','w',2,[3 3; 1 2]);
plot((1:length(en)),en)
title('Energie im Zeitbereich','FontSize',fsiz)
xlabel('Zeit','FontSize',fsiz)

%mrdesign(bw/fs,bw/fs,2,30,0);

figure; spectrum(enf)

fh3 = fullscreen('Energie gefiltert','w',2,[6 1; 5 1]);
plot(enf)
set(gca,'XLim',[1 length(enf)])
title('Energie gefiltert','FontSize',fsiz)
xlabel('Zeit','FontSize',fsiz)

return

komplexanimat(xa(1:1000))

fh4 = fullscreen('Signalbetrag','w',2,[3 3; 2 2]);
plot(sqrt(zz))

fh5 = fullscreen('Filter 1','w',2,[3 3; 1 3]);
filterplot({{b1;a1}},0,0.1,'log',-60,+10)

fh6 = fullscreen('Gesamtfilter','w',2,[3 3; 2 3]);
filterplot({{b1;a1};{b2;a2}},0,0.1,'log',-60,+10,[1 1],24,1)

fh7 = fullscreen('Datenentscheidung','w',2,[6 1; 5 1]);
plot(d)
set(gca,'YLim',[0 1.5])

set(gca,'XLim',[0 30])
title('Spektrum des Amplitudenverlaufs','FontSize',fsiz)
xlabel('Frequenz [Hz]','FontSize',fsiz)


