function symb = morsesound3symbol(snd,fs,diag)
% symb = morsesound3symbol(snd,fs,diag)
% Umsetzung tnender Morsetelegraphie in Punkte und Striche
% snd  :  Samplevektor
% fs   :  Abtastrate des Samplevektors in Hz
% diag :  falls dieser Parameter mit beliebigem Wert angegeben ist,
%         gibt es eine Reihe von Graphiken

if nargin<1  snd = morsesound; end
if nargin<2   fs = 4000;       end

% Konstanten
% ==========
bpm  = 150;           % Grssenordnung der Buchstaben pro Minute
wt   = 0.46;          % Wahrscheinlichkeit des getasteten Zustands
nfft = 1024;          % FFT-Lnge
nffe = 256;           % FFT-Lnge fr Tastfrequenz
fsiz = 13;            % FontSize
g    = 30;            % Filtergrad fr Hilbertfilter
g2   = 30;            % Filtergrad des Energiefilters
bw1  = 0.4*bpm;       % Bandbreite des ersten Tiefpassfilters in Hz
bw   = 0.056*bpm;     % Bandbreite des Energiefilters in Hz
sw   = 2.0;           % Schwelle fr die Tasterkennung

% Berechnung des Spektrums um den Ton zu finden
% =============================================
[ps,fps] = spectrum(snd,nfft,0,hanning(nfft),fs); % Spektrum
[mx,j] = max(ps(:,1));     % Maximum des Spektrums
f = fps(j(1))              % Tonfrequenz

% Transformation ins komplexe Basisband
% =====================================
xa = snd.*exp(-i*2*pi*f/fs*(0:length(snd)-1));

% Signalfilter
% ============
[koef1,srate1,dly1,aw1] = mrdesign(bw1/fs,bw1/fs,1,60,0);
fs1 = fs*prod(srate1);
xaf = mrfilter(koef1,srate1,xa);

% Energie im Zeitbereich
% ======================
xaf = xaf/sqrt(sum(abs(xaf).^2)/length(xaf)); % Normierung auf mittlere Energie = 1
en = abs(xaf).^2;                       % Energie

% Rauschenergie
% =============
%re = mean(abs(diff(en)));             % 

% Verteilung der ungefilterten Energie
% ====================================
%hen = hist(en,50);

% Tiefpassfilter und Dezimation
% =============================
koef2 = remez(g2,[0 2*bw/fs1 4*bw/fs1 1],[1 1 0 0]);
enf = filter(koef2,1,en);


% Verteilung der gefilterten Energie
% ==================================
henf = hist(en,50);

% Berechnung des Spektrums der Energie um die Tastrate zu finden
% ==============================================================
[pe,fpe] = spectrum(enf,nffe,0,hanning(nffe),fs1);
j  = find(abs(diff(pe(:,1)>sw)));
k  = (j(end-1):j(end));
ft = sum(fpe(k).*pe(k,1))/sum(pe(k,1)); % Tastrate
ft
% hieraus folgt nun der korrekte Wert fr bpm
% deshalb besser nochmals von vorn beginnen!

% Niveaus schtzen
% ================
senf = sort(enf);
tr = round(wt*length(senf));
niv0 = mean(senf(1:tr));
niv1 = mean(senf(tr+1:end));
% diese Niveaus sind noch umzurechnen
% die Niveaus von Punkt und Strich sind ja verschieden

if ~diag  return; end

% Graphische Darstellungen
% ========================
fh1 = fullscreen('Signalspektrum','w',0,[3 3; 1 1]);
plot(fps,ps(:,1))
title('Signalspektrum','FontSize',fsiz)
xlabel('Tonfrequenz [Hz]','FontSize',fsiz)

fh2 = fullscreen('Verteilung','w',0,[3 3; 1 2]);
hist(enf,50)
title('Energieverteilung','FontSize',fsiz)
xlabel('normierte Energie','FontSize',fsiz)

fh3 = fullscreen('Energiespektrum','w',0,[3 3; 1 3]);
plot(fpe,pe(:,1))
title('Energiespektrum im Basisband','FontSize',fsiz)
xlabel('Tastfrequenz [Hz]','FontSize',fsiz)

fh4 = fullscreen('Filter','w',0,[3 3; 2 1]);
filterplot(koef2)
title('Basisbandfilter','FontSize',fsiz)

fh5 = fullscreen('Energie','w',0,[6 1; 5 1]);
axes('Position',[0 0.1 1 0.8])
plot((1:length(en)),en)
axis([1 length(en) min(en)-0.1  1.1*max(en)]) 
axis off
title('Energie im Zeitbereich','FontSize',fsiz)
xlabel('Zeit','FontSize',fsiz)

fh5 = fullscreen('Energie gefiltert','w',0,[6 1; 6 1]);
axes('Position',[0 0.1 1 0.8])
plot(enf)
axis([1 length(enf) min(enf)-0.1  1.1*max(enf)]) 
axis off
title('Energie gefiltert','FontSize',fsiz)
xlabel('Zeit','FontSize',fsiz)

%mrdesign(bw/fs,bw/fs,2,30,0);
%komplexanimat(xa(1:1000))

return

fh4 = fullscreen('Signalbetrag','w',2,[3 3; 2 2]);
plot(sqrt(zz))

fh5 = fullscreen('Filter 1','w',2,[3 3; 1 3]);
filterplot({{b1;a1}},0,0.1,'log',-60,+10)

fh6 = fullscreen('Gesamtfilter','w',2,[3 3; 2 3]);
filterplot({{b1;a1};{b2;a2}},0,0.1,'log',-60,+10,[1 1],24,1)

fh7 = fullscreen('Datenentscheidung','w',2,[6 1; 5 1]);
plot(d)
set(gca,'YLim',[0 1.5])

set(gca,'XLim',[0 30])
title('Spektrum des Amplitudenverlaufs','FontSize',fsiz)
xlabel('Frequenz [Hz]','FontSize',fsiz)


ende=0;