function  p = multbabylon(a,b,fnt)
% p = multbabylon(a,b,fnt)
% Multiplikation fr beliebig grosse Zahlen in linearer 
% babylonischer Zahlendarstellung im Sexagesimalsystem.
% Siehe num2babylon
% Wenn s nicht zugewiesen wird erfolgt die Ausgabe im Font fnt in ein Fenster.

%  Autor:  Klaus von der Heide,  v.d.heide@on-line.de

if nargin<3  fnt = 'FixedWidth'; end

% Konstante
% =========
q = 60;                                   % Basis

% Wandlung der Babylonischen Ziffern von Operand a in Zahlen
% ==========================================================
s = a;
k = 1;
while length(s)>0
   zz = 0;
   s1 = s(1);
   s  = s(2:end);
   while s1~=' '
      if s1=='<' zz = zz + 10; end
      if s1=='1' zz = zz + 1;  end
      if length(s)>0
         s1 = s(1);
         s  = s(2:end);
      else
         s1 = ' ';
      end
   end
   if zz==0  s = s(2:end); end
   z(k) = zz;
   k = k + 1;
end
az = reverse(z);                           % Ziffern von a

% Wandlung der Babylonischen Ziffern von Operand b in Zahlen
% ==========================================================
z = [];
s = b;
k = 1;
while length(s)>0
   zz = 0;
   s1 = s(1);
   s  = s(2:end);
   while s1~=' '
      if s1=='<' zz = zz + 10; end
      if s1=='1' zz = zz + 1;  end
      if length(s)>0
         s1 = s(1);
         s  = s(2:end);
      else
         s1 = ' ';
      end
   end
   if zz==0  s = s(2:end); end
   z(k) = zz;
   k = k + 1;
end
bz = reverse(z);                           % Ziffern von b

% Multiplikation  
% ==============
la = length(az);
lb = length(bz);
c = 0;
for k=1:la+lb                       % alle Stellen des Resultats 
   zk = c;                          % bertrag als Anfangswert der Summation
   for i=1:la                       % alle Stellen des ersten Operanden
      if (k-i>0) & (k-i)<=lb        % nur wenn die Stelle existiert
         zk = zk + az(i)*bz(k-i);
      end
   end
   z(k) = rem(zk,q);                % k-te Ziffer des Resultats
   c = floor(zk/q);                 % bertrag in die (k+1)-te Stelle
end
z(la+lb+1) = c;                     % letzter bertrag

% Fhrende Nullen streichen
% =========================
while (z(1)==0) & (length(z)>1)  z = z(2:end); end

% Wandlung der Zahlenziffern in babylonische Schreibweise
% =======================================================
bv = reverse(z);
s = '';
for k=1:length(bv)
   nz = floor(bv(k)/10);
   ne = rem(bv(k),10);
   for i=1:nz   s = [s '<']; end
   for i=1:ne   s = [s '1']; end
   if bv(k)==0  s = [s ' ']; end
   s = [s ' '];
end


% Fhrende Blanks streichen
% =========================
while s(1)==' '  s = s(2:end); end
p = s(1:end-1);

if ~nargout  result2fig(strrep(p,'1','I'),fnt); end



