function  pixelgraphik(A,col,msize)
% pixelgraphik(A,col,msize)
% A      :  Matrix von Indices in die Struktur col
%           Die Zeilen von A werden von links nach 
%           rechts oben beginnend geschrieben. 
% col    :  Struktur-Array mit den Farben von A
%           Es knnen RGB-Vektoren und die in MATLAB 
%           definierten Farben Short Names und Long Names 
%           'w','k','b',... durcheinander benutzt werden:  
%           col={'k'; 0.8*[1 1 1]; 'r'; 'cyan'}
% col = 0 : A ist eine Matrix von Binrwerten  0 = weiss;  1 = schwarz
% col = 1 : A ist eine Matrix von Grauwerten  0.0 = weiss;  1.0 = schwarz
% col = 2 : A ist eine Matrix von natrlichen Zahlen 0,...,7 , die
%           fr die Farben  {'w'; 'k'; 'b'; 'g'; 'r'; 'c'; 'm'; 'y'} stehen.
% msize: relative Grsse des Markers (Default: 1.0)

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  A = ceil(6*rand(100,133)); end
if nargin<2  col = 0;   end
if nargin<3  msize = 1; end
if ~iscell(col)
   switch col
   case 0
      col = {'w';'k'};
      A = A + 1;
   case 1
      clear col
      for k=1:256  col{k} = (1-k/256)*[1 1 1]; end
      A = min(256,1 + round(256*A));
   case 2
      col = {'w'; 'k'; 'b'; 'g'; 'r'; 'c'; 'm'; 'y'};
   end
end

% Konstanten
% ==========
ss = get(0,'ScreenSize');              % Bildschirmformat in Pixel
mark = '.';                            % Marker ist gefllter Kreis
[m,n] = size(A);                       % Grsse des Bildes in Pixel
vb = n/m;                              % Seitenverhltnis des Bildes
vs = ss(3)/ss(4);                      % Seitenverhltnis des Bildschirms
d  = 1 - min(vb/vs,vs/vb);             % Breite des weissen Feldes
[fname,fwgt,sz,pres] = getfont;
if pres
   fpos = [0 0 ss(3:4)];
   msiz = 2*msize*min(ss(3)/n,ss(4)/m);
else
   fpos = [3 356 537 393];
   msiz = 5;
end

% Fenster und Achsen
% ==================
fh = fullscreen('Graphik');            % Erzeugung des Bildfensters
set(fh,'Units','pixels','Position',fpos)
if vb<vs
   axes('Position',[d/2  0  1-d  1])   % Achsen
else
   axes('Position',[0  d/2  1  1-d])   
end
axis([0 n+1 0 m+1]); axis off          % Skalierung der Achsen

% Zeichnen des Bildes
% ===================
for k=1:length(col)
   c = col{k};
   [i,j] = find(A==k);
   line(j,1+m-i,'Color',c,'Marker',mark,'LineStyle','none','MarkerSize',msiz)
end
