function  nexpr = replacevariablenames(oexpr,oldvar,newvar)
% nexpr = replacevariablenames(oexpr,oldvar,newvar)
% nderung von Variablennamen in einem algebraischen Ausdruck.
% oexpr  :  String mit dem algebraischen Ausdruck
% oldvar :  Cell-Array mit den Strings der zu ndernden Variablennamen
% newvar :  wie oldvar mit den neuen Variablennamen in der entsprechenden
%           Reihenfolge
% nexpr  :  String mit dem neuen Ausdruck
% Die Variablennamen mssen den Einschrnkungen von symvar gengen:
% Erstes Zeichen ist ein Buchstabe, danach beliebige Buchstaben, 
% Ziffern oder Unterstrich.  Vordefinierte Konstanten wie 
% 'i', 'j', 'pi', 'inf', 'nan', 'eps', 'sin', etc. sind nicht erlaubt.
% Namen von Funktionen, die mit Parametern aufgerufen werden, knnen 
% nicht ersetzt werden.  
% Beispiel:  replacevariablenames('a*aaa+aa-aaa*a',{'a';'aaa'},{'b';'a'})
%            liefert  'b*a+aa-a*b'

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Konstanten
% ==========
ltv  = 3;                % Lnge der temporren Namen
nvar = length(oldvar);   % Anzahl der zu ndernden Namen
if length(newvar)~=nvar  
   error('Anzahl der Variablen in oldvar und newvar ungleich');
end

% Ermittlung geeigneter temporrer Variablennamen
% ===============================================
svar = symvar(oexpr);    % smtliche symbolischen Namen im Ausdruck
nsvar = length(svar);    % Anzahl der symbolischen Namen
symbols = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
nsymb = length(symbols);
for k=1:nsvar
   tvar{k} = svar{1};
   no = 1;
   while no | sum(findstr(tvar{k},[strcat(svar{:}), ...
                                   strcat(newvar{:}), ...
                                   strcat(' ',tvar{1:k-1})]))
      tvar{k} = symbols(1+floor(nsymb*rand(1,ltv)));
      no = 0;
      for j=1:nvar
         if findstr(newvar{j},tvar{k})  no = 1; end
      end
   end
end

% Ersetzung aller Variablennamen durch temporre
% ==============================================
for k=1:nsvar  
   lsv(k) = length(svar{k}); 
end
[lsvs,kk] = sort(lsv);
texpr = oexpr;
for k=reverse(kk)
   texpr = strrep(texpr,svar{k},tvar{k});
end

% Ersetzung der temporren Variablennamen durch die neuen
% =======================================================
tnvar = svar;
for k=1:nvar
   for j=1:nsvar
      if strcmp(oldvar{k},svar{j})
         tnvar{j} = newvar{k};
      end
   end
end
for k=1:nsvar
   texpr = strrep(texpr,tvar{k},tnvar{k});
end

% Resultat
% ========
nexpr = texpr;
