function s = rm2expr(r,variable)
% s = rm2expr(r,variable)
% Wandlung des binren Koeffizientenvektors r der Reed-Muller-Form
% in den algebraischen Ausdruck mit den Variablen variable.
% r        :  Binrvektor der Lnge 2^k mit den Koeffizienten
% variable :  Cell array mit den Variablen als Strings
%             Beispiel:  {'a1'; 'a2'; 'b'; 'c'}

% Defaults
% ========
if nargin<1  r = [1 0 0 1 1 1 0 0]; end
if nargin<2  variable = {'a';'b';'c';'d';'e';'f';'g';'h'}; end

% Konstante
% =========
n = round(log2(length(r)));

% Rechnung
% ========
if r(1)  
   s = '1 + ';
else
   s = '';
end
for k=2:length(r)                  % alle Terme der Reed-Muller-Form
   if r(k)                         % wenn Term vorhanden
      bvk = num2binvec(k-1,n);     % bvk zeigt welche Variablem im Term vorkommen
      for j=1:n
         if bvk(n-j+1)             % wenn Variable vorkommt dann einfgen
            s = [s sprintf('%s*',variable{j})];
         end
      end
      s = [s(1:end-1) ' + '];      % Verknpfung mit nchstem Term
   end
end
if length(s)>0 
   s = s(1:end-3);                 % letzte Verknpfung streichen
else
   s = '0';
end
