function basis = rmbasis(variable)
% basis = rmbasis(variable)
% generiert die Basisfunktionen der Reed-Muller-Form 
% mit den Variablen variable als CellArray von Strings.
%
% variable :  Cell array mit den Variablen als Strings
%             z.B.  {'a1'; 'a2'; 'b'; 'c'}
%             Es kann auch die Anzahl der Variablen angegeben werden.
%             Diese heissen dann 'a', 'b', ...

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Default
% =======
if nargin<1  variable = {'a';'b';'c';'d'}; end

if length(variable)==1 & isnumeric(variable)
   n = variable;
   if n>25  error('rmbasis: mehr als 25 Variable sind nicht erlaubt.'); end
   v = {'a';'b';'c';'d';'e';'f';'g';'h';'i';'j';'k';'l';...
         'm';'n';'p';'q';'r';'s';'t';'u';'v';'w';'x';'y';'z'};
   variable = reverse({v{1:n}});
else
   n = length(variable);
end

% Iterationsanfang
% ================
basis{1} = '1';
basis{2} = variable{n};

% Iteration
% =========
for k=2:n
   m = length(basis);
   v = variable{n-k+1};
   basis{m+1} = v;
   for j=2:m
      basis{m+j} = [basis{j}  '*'  v];
   end
end

% Ausgabe
% =======
if ~nargout
   ReedMullerBasis = basis';
   result2fig(ReedMullerBasis)
end