function  stellen2stellen(q,z)
% stellen2stellen(q,z)
% Darstellung des Algorithmus zur Umrechnung zwischen
% Stellensystemen.
% q  :  Basis des Stellensystems
% z  :  darzustellende Zahl
% r  :  String mit dem Resultat
% q und z knnen interaktiv gendert werden.
%
% Bekannter Fehler bei MATLAB 5.3: Klickt man nach der Eingabe 
% von q (z) ins Eingabefeld von z (q), ohne die Eingabe mit der 
% Eingabetaste beendet zu haben, so strzt MATLAB 5.3 ab.
% Neuere Versionen sind unempfindlich.

% Autor:  Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

%#function stellen2stellencb

% Defaults
% ========
if nargin<1  q = 2;   end
if nargin<2  z = 123; end

% Konstanten
% ==========
zeiger = '\rightarrow';             % Character fr Programmzeiger
txt  = {'Algorithmus'; ' '; ' '; 'darzustellende Zahl x'; 'Basis q'; 'Resultat'};
ptxt = {' ';' '; 'a := x';' '; 'while  a > 0';' '; '    y_n:= a mod q';' '; '   a  := a div q';' '; 'end'};
ntxt = {'n = 0';' '; 'a_n= x';' '; 'while  a_n> 0';' '; '    y_n = a_nmod q';' '; '   a_{n+1}= a_ndiv q';' '; 'end'};
utxt = {'mathematisch'; 'rechentechnisch'; 'Takt'};
font = 'Albany';
scrs = get(0,'ScreenSize');
fsiz = round([30  24  40  20  20  24*ones(1,5)  28  28]/1024*scrs(3));
usiz = [20  22  24  24];
fwgt = 'demi';                      % FontWeigt
lwl  = 3;                           % Strichstrke
symb = '0123456789ABCDEF';          % Ziffernsymbole
ns   = 16;                          % Anzahl der Stellen des Resultats
     
aax  = 0.07;
tax  = 0.54;   
tay  = [0.57  0.465  0.36  0.23  0.05  0.65];   

tpos = [0.15   0.9;                 % 1
        0.15   0.4;                 % 2
        aax    tay(1);              % 3
        0.475  0.9;                 % 4
        0.798  0.9;                 % 5
        tax    tay(1);              % 6  a =
        tax    tay(2);              % 7  a > 0
        tax    tay(3);              % 8  a mod q
        tax    tay(4);              % 9  a div q
        tax    tay(6);              % 10 n =
        0.9285 0.1;                 % 11 ... 18 Resultat
        0.67   0.04];               % 19 "Resultat"

uipos = [tpos(1,1)  tpos(1,2)-0.1  0.21  0.05;
         0.27       0.0            0.2   0.1;
         0.45       0.8            0.3   0.06;
         0.8        0.8            0.08  0.06];
      
dx   = -0.05;
dy   = 0;
plt  = 0.02;
pbr  = 0.01;
lp1x = tax - 0.02 + [0  dx  dx; dx  dx  -plt];
lp1y = dy + [tay(4) tay(4) tay(1); tay(4) tay(1) tay(1)];
pp1x = lp1x(2,3) + [0 0 plt];
pp1y = lp1y(2,3) + [-pbr +pbr 0];

lp2x = 0.94 + [-0.18  0  0  0; 0  0  0  0];
lp2y = dy + [tay(3) tay(3)  0.2  0.2; tay(3)  0.2  0.2  0.15];
pp2x = lp2x(2,4) + [-pbr +pbr 0];
pp2y = lp2y(2,4) + [0 0 -plt];

% Bild und Achsen
% ===============
fh = fullscreen('Q2Q');
set(fh,'UserData',[q,ns,z])
ax = axes('Position',[0 0 1 1]);
axis([0 1 0 1]); axis off

% Texte
% =====
th(1) = text(tpos(1,1),tpos(1,2),txt{1},'FontSize',fsiz(1),'FontName','Times','FontWeight',fwgt,'Color','b');      
th(2) = text(tpos(2,1),tpos(2,2),ntxt,'FontSize',fsiz(2),'FontName',font,'FontWeight',fwgt,'Color','k');
th(3) = text(tpos(3,1),tpos(3,2),zeiger,'FontSize',fsiz(3),'Color','w');
th(4) = text(tpos(4,1),tpos(4,2),txt{4},'FontSize',fsiz(4),'FontName','Times','FontWeight',fwgt,'Color','b');
th(5) = text(tpos(5,1),tpos(5,2),txt{5},'FontSize',fsiz(5),'FontName','Times','FontWeight',fwgt,'Color','b');
thr   = text(tpos(12,1),tpos(12,2),txt{6},'FontSize',fsiz(12),'FontName','Times','FontWeight',fwgt,'Color','b');

for k=6:10  
   th(k) = text(tpos(k,1),tpos(k,2),'','FontSize',fsiz(k),'FontName',font,'FontWeight',fwgt,'Color',[0 0.5 0]);
end

anr = 11;    % Anfangsnummer der Resultatshandle
dx  = -0.027;
for k=0:ns-1
   th(k+anr) = text(tpos(anr,1)+k*dx,tpos(anr,2),'0','FontSize',fsiz(anr),'FontName',font,'FontWeight',fwgt,'Color','b');
end

% Pfeile
% ======
ar1 = line(lp1x,lp1y,'LineWidth',lwl,'Color','w');
a1p = patch(pp1x,pp1y,'w');
set(a1p,'FaceColor','w','EdgeColor','w')   
ar1 = [ar1; a1p];
ar2 = line(lp2x,lp2y,'LineWidth',lwl,'Color','w');
a2p = patch(pp2x,pp2y,'w');
set(a2p,'FaceColor','w','EdgeColor','w')   
ar2 = [ar2; a2p];

% Interaktivitt
% ==============
ui1 = uicontrol(fh,'Units','normalized','Position',uipos(1,:),'Style','togglebutton','CallBack','stellen2stellencb',...
                   'String',utxt{1},'BackgroundColor',0.5*[1 1 1],'FontSize',usiz(1),'ForegroundColor','w',...
                   'FontName',font,'UserData',{1; 0; th(2); ntxt; ptxt; utxt{1}; utxt{2}; th(3)});

ui2 = uicontrol(fh,'Units','normalized','Position',uipos(2,:),'Style','pushbutton','CallBack','stellen2stellencb',...
                   'String',utxt{3},'BackgroundColor',[0 0.5 0],'FontSize',usiz(2),'ForegroundColor','w',...
                   'FontName',font,'UserData',{2; th([6:9, 3, 10:end]); [1 z 0 0 1]; aax; tay; ar1; ar2; ui1});
                
uix = uicontrol(fh,'Units','normalized','Position',uipos(3,:),'String',int2str(z),'Style','edit',...
                   'FontSize',usiz(3),'FontName','FixedWidth',...
                   'CallBack','stellen2stellencb','UserData',{3});
                
uiq = uicontrol(fh,'Units','normalized','Position',uipos(4,:),'String',int2str(q),'Style','edit',...
                   'FontSize',usiz(4),'FontName','FixedWidth',...
                   'CallBack','stellen2stellencb','UserData',{4});
