function  z = stradd(x,y,q,n)
% z = stradd(x,y,q,n)
% Addierer fr ganze Zahlen zur Basis q, die als Strings vorliegen.
% Negative Zahlen werden durch ein voranstehendes "-" gekennzeichnet.
% wenn die Stellenzahl n angegeben ist, so werden fhrende Nullen 
% angefgt bzw. fhrende Stellen abgeschnitten.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Konstante Ziffern
% =================
charzif = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

% Prfung der Parameter
% =====================
lz = 2 + max(length(x),length(y));
if (q<2)|(q>length(charzif)) error('stradd: Basis nicht aus 2:64'); end
if (x(1)=='-')|(x(1)=='+')  xx = x(2:end); else xx = x; end
if (y(1)=='-')|(y(1)=='+')  yy = y(2:end); else yy = y; end
if checkchar(xx,charzif(1:q)) error('stradd: x mit falscher Ziffer'); end
if checkchar(yy,charzif(1:q)) error('stradd: y mit falscher Ziffer'); end
xx = ['0'*ones(1,lz-length(xx)) xx];
yy = ['0'*ones(1,lz-length(yy)) yy];

% Komplemente falls Operanden negativ
% ===================================
co = 0;
if x(1)=='-'
   for k=1:lz 
      xx(k) = charzif(1 + q - find(xx(k)==charzif));
   end
   co = 1;
end
if y(1)=='-'
   for k=1:lz 
      yy(k) = charzif(1 + q - find(yy(k)==charzif));
   end
   co = co+1;
end

% Addition
% ========
for k=lz:-1:1
   ci = co;
   ix = find(xx(k)==charzif);
   iy = find(yy(k)==charzif);
   [zk co] = fulladd(ix-1,iy-1,ci,q);
   z(k) = charzif(zk+1);
end

% Komplement falls Resultat negativ
% =================================
if z(1)==charzif(q)
   for k=1:lz 
      z(k) = charzif(1 + q - find(z(k)==charzif));
   end
   z = stradd(z,'1',q);
   while (z(1)=='0') & (length(z)>1)  z = z(2:end); end
   z = ['-' z];
else
   while (z(1)=='0') & (length(z)>1)  z = z(2:end); end
end

if nargin>3
   if length(z)<n  
      for k=1:n-length(z)  z = ['0' z]; end
   else
      z = z(length(z)-n+1:end);
   end   
end

% Evtl. Ausgabe ins Fenster
% =========================
if ~nargout
   b  = num2str(q);
   nb = length(b);
   nx = length(x);
   ny = length(y);
   nz = length(z);
   m  = max([nb nx ny nz]);
   s  = {['Basis:     '  blanks(m-nb)  b];
         ['Operand1 : '  blanks(m-nx)  x];
         ['Operand2 : '  blanks(m-ny)  y];
         ['Summe    : '  blanks(m-nz)  z]
        };
   result2fig(s)
end