%
%
%11.2  Entwurf getakteter Schaltwerke %<'FontWeight',MSVTITLWEIGHT,'FontSize',MSVTITLSIZE-6,'Color',MSVTITLCOLOR>
%
%
%
% Beim Entwurf von Schaltnetzen geht man aus von einer eindeutigen Verhaltensbeschreibung 
% durch binralgebraische Ausdrcke, Funktionstabellen oder BDDs etc..  Entwurfsfreiheiten 
% bestehen in der Wahl der realisierenden Struktur.  Dieses gilt also fr den Entwurf der 
% \delta- und \lambda-Schaltnetze von Schaltwerken auch.  Der grundstzliche Unterschied beim 
% Entwurf eines Schaltwerkes besteht jedoch darin, dass die Zustandsvariablen  z_i  
% Eingangsgrssen der beiden Schaltnetze sind.  Sowohl die Anzahl dieser binren 
% Zustandsvariablen wie auch die Codierung der Zustnde in diesen Variablen  z_i  gehren 
% zu den Freiheiten des Entwurfs eines Schaltwerkes.  Selbst die Zahl der bentigten 
% Zustnde liegt nicht fest. 
%
% Ziel beim Entwurf eines Schaltwerkes ist wie beim Schaltnetz eine Realisierung mit 
% geringem Aufwand.  Neben den beiden Schaltnetzen sind auch die Zeitglieder zu realisieren.  
% Wir nehmen in diesem Abschnitt deren Realisierung als rckflankengesteuerte Flipflops an. 
% Die Zahl der Zustandsvariablen und vor allem die Codierung der Zustnde hat erheblichen 
% Einfluss auf die beiden Schaltnetze.  Die Zusammenhnge sind so komplex, dass es kein 
% Verfahren zur Minimierung des Realisierungsaufwandes gibt, welches alle Entwurfsfreiheiten 
% benutzt.  Die Verfahren bieten also alle nicht die Garantie, dass eine gefundene Lsung 
% nicht zu verbessern ist.
%
% Der Entwurf getakteter Schaltwerke erfolgt meistens in sechs Phasen:
%
% \bf1.  Spezifikation textuell oder grafisch z.B. mit dem Zustandsdiagramm
%
% \bf2.  Aufstellen der formalen bergangstabelle
%
% \bf3.  Reduktion der Zahl der Zustnde
%
% \bf4.  Wahl der Zustandscodierung und Aufstellen der bergangstabelle
%
% \bf5.  Minimierung der Schaltnetze
%
% \bf6.  berprfung des realisierten Schaltwerks
%
% Bei Unzufriedenheit mit dem Ergebnis geht man zurck nach 4., evtl. auch nach 2.
% letzteres evt. ohne 3.
%
% Die Entwurfsphasen sollen nachfolgend an einem Beispiel erlutert werden:
%
% (1) Ausgangspunkt sei folgendes Zustandsdiagramm:
%
drawfsm( [1 2; 3 4; 1 4; 5 6; 1 6; 7 6; 1 6] )
%
% (2) Aus dem Zustandsdiagramm folgt die bergangstabelle, die hier wieder in Form der 
% Flusstafel dargestellt wird.  Wir fgen noch eine Ausgabetafel fr das Mealy-Modell an:
%
fsmtab(...
   { [1 2; 3 4; 1 4; 5 6; 1 6; 7 6; 1 6];...      % bergangstabelle
      1 + [0 0; 0 0; 0 0; 0 1; 0 1; 0 1; 0 1]},...
   {'A';'B';'C';'D';'E';'F';'G'},...
   1,...
   0,...
   {'00';'01';'11';'10'},...
   '',...
   {'0';'1'}...
      )
%
% (3) Meistens weist ein Zustandsdiagramm bei der Spezifikation mehr Zustnde auf
% als ntig.  Wenn nmlich zwei Zustnde fr alle mglichen Eingaben jeweils die 
% selben Folgezustnde und auch die selben Ausgaben haben, so kann man diese 
% quivalenten Zustnde als den selben betrachten.
%
% {\bfDefinition}:  Zwei Zustnde  {\fontname{Courier}z_i}  und  {\fontname{Courier}z_j}  heissen {\bfquivalent}, wenn  
%
% {\fontname{Courier}  \lambda(z_i) = \lambda(z_j)         }beim Moore-Schaltwerk bzw.
% {\fontname{Courier}\lambda(x,z_i) = \lambda(x,z_j)  \forallx  } beim Mealy-Schaltwerk
%
% und entweder
%
% {\fontname{Courier}\delta(x,z_i) = \delta(x,z_j)}
%
% oder
%
% {\fontname{Courier}\delta(x,z_i)}  quivalent  {\fontname{Courier}\delta(x,z_j)  \forall x} .
%
% Die quivalenz ist offensichtlich einfach automatisch zu prfen.  Bei der Elimination 
% eines der beiden quivalenten Zustnde mssen Referenzen auf ihn durch Referenzen 
% auf den quivalenten ersetzt werden.  Dies tut die Funktion  {\fontname{Courier}elimeqs} . 
% Damit ergeben sich das folgende reduzierte Zustandsdiagramm und die reduzierte 
% Flusstafel:
%
result2fig(...
   select(...
      elimeqs(...
         {[1 2; 3 4; 1 4; 5 6; 1 6; 7 6; 1 6];...
           1 + [0 0; 0 0; 0 0; 0 1; 0 1; 0 1; 0 1]}...
              ),...
          1)...
           );
%
result2fig(...
   select(...
      elimeqs(...
         {[1 2; 3 4; 1 4; 5 6; 1 6; 7 6; 1 6];...
           1 + [0 0; 0 0; 0 0; 0 1; 0 1; 0 1; 0 1]}...
              ),...
          2)...
           );
%
% Das Schaltwerk ist in der Funktion gleich geblieben:
%:
demofsm( ...
   select( ...
      elimeqs( ...
         {[1 2; 3 4; 1 4; 5 6; 1 6; 7 6; 1 6]; ...
          1 + [0 0; 0 0; 0 0; 0 1; 0 1; 0 1; 0 1] }...
             )...
      ,1)...
       )
%
% Die quivalenz der beiden Zustnde  {\fontname{Courier}E}  und  {\fontname{Courier}G}  war unmittelbar in der ursprnglichen 
% Flusstafel zu erkennen.  Dass auch  {\fontname{Courier}D}  und  {\fontname{Courier}F}  quivalent sind, ergibt sich erst als 
% Folge der quivalenz von  {\fontname{Courier}E}  und  {\fontname{Courier}G} .
%
% (4) Die Wahl einer binren Codierung der Zustnde ist bei getakteten Schaltwerken 
% in keiner Weise eingeschrnkt.  Bisweilen ist der Realisierungsaufwand sogar am 
% kleinsten, wenn der Block-Code mehr Bits enthlt als erforderlich.  Im vorliegenden 
% Fall bentigen wir wenigstens 3 Zustandsbits, um die 5 Zustnde zu codieren.  
% Das sind nicht weniger Bits als zur Codierung der 7 Zustnde des ursprnglichen 
% Zustandsdiagramms erforderlich gewesen wren.  Dennoch lohnt sich die Reduktion 
% des Zustandsdiagramms, da die nicht belegten Zustnde als nicht spezifizierte 
% Punkte (Do'ntCares) in den KV-Diagrammen der \delta- und \lambda-Schaltnetze auftauchen 
% und so zur Vereinfachung ntzlich sind.  In unserem einfachen Fall wollen wir 
% die Zustnde mit 3 Bits codieren.  Die Anzahl der mglichen Zustandscodierungen 
% von  {\fontname{Courier}n}  Zustnden mit  {\fontname{Courier}k}  Bits ist gleich der Zahl der Variationen der  {\fontname{Courier}2^k}  Binrwrter 
% zu je  {\fontname{Courier}n}  Zustnden, nmlich  
%
fsmcod
%
% In dieser Tabelle wurde jeweils die minimale Anzahl  {\fontname{Courier}k}  von Zustandsbits fr die gegebene 
% Anzahl  {\fontname{Courier}n}  von Zustnden gewhlt.  In unserem Fall sind es  6720 Mglichkeiten.  
% Die wollen wir nicht alle untersuchen sondern nur zwei:
%
% (a)  Die Zustnde  {\fontname{Courier}A,B,C,D,E}  werden binr codiert der Reihe nach durch
%       {\fontname{Courier}000  001  010  011  100} .
%
% (b)  Die Codierung wird generiert nach folgendem Verfahren:
%
% Man frbe alle Kanten des Zustandsdiagramms jeweils mit dem Hammingabstand 
% der Codierungen der Knoten, die die Kante verbindet, und versuche die Summe aller 
% Farben durch geeignete Codierung zu minimieren.
%
% Diese Heuristik ist plausibel, denn sie bewirkt, dass sich bei nderung von  {\fontname{Courier}x}  
% oder eines der Zustandsbits meist nur ein Ausgang ndert, whrend die anderen 
% konstant bleiben.  Die Einsen im KV-Diagramm des \delta-Schaltnetzes  liegen also 
% meist zusammen, so dass grosse Schleifen mglich sind.
%
% Die Zustandsdiagramme fr beide Codierungen sind:
%
drawfsmcod
%
% (5) Aus diesen Diagrammen entnimmt man die Funktionstabellen der drei binren 
% Funktionen des \delta-Schaltnetzes.  Hinzu kommt in diesem Fall nur eine Schaltfunktion 
% fr die Ausgabe.  Die Funktionswerte kann man in KV-Diagramme eintragen und 
% anschliessend eine Bndelminimierung vornehmen.
%
minfsmcoda
%
% Die Minimierung ergibt:
%
%     h = z_1\capx                              %<'FontName',MSVFORMFONT>
% z_{1next} = z_2\capz_3\capx'                  %<'FontName',MSVFORMFONT>
% z_{2next} = z_2\capx \cup z_2'\capz_3 \cup h  %<'FontName',MSVFORMFONT>
% z_{3next} = x                                 %<'FontName',MSVFORMFONT>
%     y = h \cup z_2\capz_3\capx                %<'FontName',MSVFORMFONT>
%
% Die Aufwandsmasse sind hierfr  {\fontname{Courier}M1 = 7}  und  {\fontname{Courier}M2 = 17}
%
% Fr die Zustandscodierung (b) haben wir:
%
minfsmcodb
%
% Die Minimierung ergibt:
%
%     y = z_1\capx                              %<'FontName',MSVFORMFONT>
% z_{1next} = z_1\capz_3 \cup z_3\capx \cup y   %<'FontName',MSVFORMFONT>
% z_{2next} = z_1'\capz_2'\capz_3\capx          %<'FontName',MSVFORMFONT>
% z_{3next} = z_{2next} \cup x                  %<'FontName',MSVFORMFONT>
%
% Die Aufwandsmasse sind hierfr  {\fontname{Courier}M1 = 6}  und  {\fontname{Courier}M2 = 15} .  Der Aufwand ist bei der 
% Codierung (b) bezglich beider Aufwandsmasse etwas geringer als bei Codierung (a).  
% Man wird also die Zustandscodierung (b) benutzen.
%
% (6) Das entworfene Schaltwerk entspricht der Spezifikation.  Eine abschliessende 
% berprfung ist dennoch sinnvoll, weil die Spezifikation hufig Randbedingungen 
% nicht ausdrcklich nennt, die aber eingehalten werden mssen.  Z.B. knnte es sein,
% dass das Schaltwerk beim Einschalten in einen in der Spezifikation nicht vorkommenden
% Zustand gert, der nur Schlingen aufweist und damit keinen Weg zum vorgesehenen 
% Schaltwerk hat.  Wenn die Spezifikation nicht eine Eingabe vorsieht, die das 
% Schaltwerk in einen definierten Grundzustand bringt (sog. Reset) und wenn eine 
% Zustandscodierung gewhlt wurde, die mehr Zustnde ermglicht als bentigt, dann 
% ist dieser sechste Schritt wichtig.
%
% Wir mssen also das Zustandsdiagramm mit allen  2^k  Zustnden untersuchen, das 
% durch die ermittelten schaltalgebraischen Ausdrcke realisiert wird. Wir notieren 
% also zunchst die schaltalgebraischen Ausdrcke fr die drei Zustandsbits.  Daraus 
% wird fr smtliche 8 mglichen Zustnde mal 2 Inputs der Folgezustand berechnet, 
% um die bergangstafel zu ermitteln.  Nach dieser wird das Zustandsdiagramm 
% gezeichnet.  Wir automatisieren dies fr beide Zustandscodierungen.
%
% Zustandscodierung (a)
%
demofsmcoda
%
% Zustandscodierung (b)
%
demofsmcodb
%
% Offenbar fhren in beiden Fllen zuflligerweise alle Kanten von den zustzlichen 
% Knoten direkt zum gewnschten Automaten.  Wenn man nach dem Einschalten einen Takt 
% gibt, so wird sich dieses Schaltwerk mit Sicherheit in einem der vorgesehenen Zustnde 
% befinden.  Dass zustzliche Zustnde vllig isoliert sind, ist zwar selten, aber es kommt 
% vor.  Abhilfe ist dann relativ einfach.  Unter Beibehaltung der Zustandscodierung kann 
% man im KV-Diagramm einen oder mehrere Do'ntCares gezielt je nach Erfordernis auf  
% 0  oder  1  setzen und das Schaltnetz neu minimieren.
%
% Damit ist der Entwurf des Schaltwerkes abgeschlossen.
%
