%
%
%
%
%12.6  Addierer  %<'FontWeight',MSVTITLWEIGHT,'FontSize',MSVTITLSIZE,'Color',MSVTITLCOLOR>
%
%
%
% Binraddierer sind Schaltnetze zur Realisierung der Addition von Binrzahlen in 
% Festkommadarstellung.  Eingnge sind die beiden {\fontname{Courier}k}-Bit-Operanden  {\fontname{Courier}a}  und  {\fontname{Courier}b}  sowie 
% der bertrag fr die  {\fontname{Courier}0}-te Stelle.  Ausgegeben werden das {\fontname{Courier}k}-Bit-Resultat und der 
% bertrag von der hchsten Stelle.  Das Verhalten auf der R-T-Ebene ist damit 
% festgeschrieben.  Der Addierer fr  {\fontname{Courier}k = 1}  Bit heisst Volladdierer.  Das Schaltnetz 
% ist ziemlich einfach:
%
binexp2bintab3({'a+b+c';'(a&b)|(a&c)|(b&c)'},{'a';'b';'c'},2)
%
% Die Simulation des Volladdierers ist entsprechend berschaubar:
%
%       function  [s,c] = fulladd(a,b,ci,q)            %<'FontName',MSVFORMFONT>
%       % [s,c] = fulladd(a,b,ci,q)                    %<'FontName',MSVFORMFONT>
%       % Volladdierer fr Stellensysteme zur Basis q  %<'FontName',MSVFORMFONT>
%       if nargin<4  q = 2; end;                       %<'FontName',MSVFORMFONT>
%       z = a + b + ci;                                %<'FontName',MSVFORMFONT>
%       s = rem(z,q);                                  %<'FontName',MSVFORMFONT>
%       c = floor(z/q);                                %<'FontName',MSVFORMFONT>
%
% Wir wollen im folgenden beispielhaft mgliche Realisierungen auf der Gatterebene 
% angeben sowie den Aufbau schneller Addierer fr groe  {\fontname{Courier}k} .
%
%
% \bfSerieller-Addierer
%
% Der serielle Addierer arbeitet nach dem in der Grundschule gelernten Prinzip der 
% stellenweisen Addition beginnend bei der niederwertigsten Stelle.  Die Hardware 
% ist ein einfaches Schaltwerk bestehend aus einem Volladdierer und einem 
% Zustandsflipflop, dem pro Takt je ein Bit der beiden Operanden beginnend mit 
% dem niederwertigsten am Eingang angeboten wird:
%
%       q = 2;                        % Basis des Stellensystems       %<'FontName',MSVFORMFONT>
%       a = reverse([0 1 1 0 1]);     % Stellenvektor des Summanden a  %<'FontName',MSVFORMFONT>
%       b = reverse([0 0 1 1 1]);     % Stellenvektor des Summanden b  %<'FontName',MSVFORMFONT>
%       c = 0;                        % bertrag 0                     %<'FontName',MSVFORMFONT>
%       clear s                                                        %<'FontName',MSVFORMFONT>
%       for k=1:length(a)                                              %<'FontName',MSVFORMFONT>
%          [s(k),c] = fulladd(a(k),b(k),c,q);                          %<'FontName',MSVFORMFONT>
%       end                                                            %<'FontName',MSVFORMFONT>
%       reverse(s),c                  % Ausgabe ins Kommandofenster    %<'FontName',MSVFORMFONT>
%
% Dieser Code soll nur das zeitliche Nacheinander aufzeigen.  Die Addition 
% selbst wird durch die Funtion  {\fontname{Courier}demofixpointadd}  dargestellt.
%
%
% \bfRipple-Carry-Addierer
%
% Der Ripple-Carry-Addierer arbeitet nach dem selben Schema wie der serielle Addierer. 
% Jedoch werden fr n-stellige Operanden n Volladdierer in Hardware realisiert.  Der 
% k-te Volladdierer empfngt den bertrag des (k-1)-ten Volladdierers und gibt den 
% selbst berechneten bertrag an den nchsten Volladdierer.  Obwohl alle Volladdierer 
% im Schaltnetz parallel arbeiten, kann die hchstwertige Stelle erst endgltig berechnet 
% werden, wenn bei diesem Volladdierer der bertrag eintrifft.  Im Gegensatz zum 
% seriellen Addierer ist der Ripple-Carry-Addierer ein Schaltnetz.
%
% Eine rekursive Simulation des Ripple-Carry-Addierers zeigt folgende Matlab-Funktion:
%
%       function [s,c] = ripplecarryadd(a,b,c0,q)                  %<'FontName',MSVFORMFONT>
%       % [s,c] = ripplecarryadd(a,b,ci,q)                         %<'FontName',MSVFORMFONT>
%       % Ripple-Carry-Addierer                                    %<'FontName',MSVFORMFONT>
%       % a, b :  Vektoren der Ziffern der Summanden,              %<'FontName',MSVFORMFONT>
%       %         Die Lnge der Vektoren mssen gleich sein.       %<'FontName',MSVFORMFONT>
%       % c0   :  bertrag in die niederwertigste Stelle           %<'FontName',MSVFORMFONT>
%       % q    :  Basis des Systems, Default: 2                    %<'FontName',MSVFORMFONT>
%       % s    :  Vektor der Ziffern der Summe.                    %<'FontName',MSVFORMFONT>
%       %         s hat die selbe Anzahl von Stellen wie a und b.  %<'FontName',MSVFORMFONT>
%       % c    :  bertrag von der hchstwertigen Stelle.          %<'FontName',MSVFORMFONT>
%       %         c kann als hchstwertige Stelle dem Vektor s     %<'FontName',MSVFORMFONT>
%       %         nachgestellt werden.                             %<'FontName',MSVFORMFONT>
%       if length(a)==1                                            %<'FontName',MSVFORMFONT>
%          [s,c]   = fulladd(a,b,c0,q);                            %<'FontName',MSVFORMFONT>
%       else                                                       %<'FontName',MSVFORMFONT>
%          [sx,cx] = ripplecarryadd(a(1:end-1),b(1:end-1),c0,q);   %<'FontName',MSVFORMFONT>
%          [sy,c]  = fulladd(a(end),b(end),cx,q);                  %<'FontName',MSVFORMFONT>
%          s = [sx,sy];                                            %<'FontName',MSVFORMFONT>
%       end                                                        %<'FontName',MSVFORMFONT>
%
%
% \bfCarry-Lookahed-Addierer
%
% Anstatt die bertrge stellenweise zu berechnen kann man smtliche bertrge direkt 
% aus den Operanden ermitteln.  Das Schaltnetz dafr ist ziemlich aufwndig - besonders 
% fr grosse Stellenzahlen.  Diese Addierer heissen Carry-Lookahead-Addierer.
%
%
% \bfCarry-Select-Addierer
%
% Fr grosse Stellenzahlen ist der Carry-Lookahead-Addierer nicht mehr praktikabel. 
% Mit drei n-Bit-Addierern (z.B. Carry-Lookahead- oder auch Carry-select-) kann man 
% einen 2n-Bit-Addierer wie folgt realisieren:
%
% Ein n-Bit-Addierer berechnet die niederwertigen n Bit der Summe und den bertrag 
% der n-ten Stelle.  Zugleich berechnen zwei vllig gleiche n-Bit-Addierer die 
% hherwertigen n Bit der Summe fr die beiden mglichen bertrge der n-ten Stelle. 
% Die Summen werden alle zeitgleich fertig.  Mit einem n-fachen 2-zu-1-Multiplexer 
% wird dann das Ergebnis aufgrund des tatschlichen bertrags der n-ten Stelle 
% ausgewhlt, das andere wird verworfen.
%
% Eine Simulation zeigt folgende Matlab-Funktion:
%
%       function [s,c] = carryselectadd(a,b,ci,q)                                   %<'FontName',MSVFORMFONT>
%       % [s,c] = carryselectadd(a,b,ci,q)                                          %<'FontName',MSVFORMFONT>
%       % Carry-Select-Addierer                                                     %<'FontName',MSVFORMFONT>
%       % a, b :  Vektoren der Ziffern der Summanden,                               %<'FontName',MSVFORMFONT>
%       % c0   :  bertrag in die niederwertigste Stelle                            %<'FontName',MSVFORMFONT>
%       % q    :  Basis des Systems, Default: 2                                     %<'FontName',MSVFORMFONT>
%       % s    :  Vektor der Ziffern der Summe.                                     %<'FontName',MSVFORMFONT>
%       %         s hat die selbe Anzahl von Stellen wie a und b.                   %<'FontName',MSVFORMFONT>
%       % c    :  bertrag von der hchstwertigen Stelle.                           %<'FontName',MSVFORMFONT>
%       %         c kann als hchstwertige Stelle dem Vektor s nachgestellt werden. %<'FontName',MSVFORMFONT>
% 
%       n = max(1,floor(length(a)/2));                                              %<'FontName',MSVFORMFONT>
% 
%       % Simulation der parallelen Berechnung mit drei Addierern                   %<'FontName',MSVFORMFONT>
%       % =======================================================                   %<'FontName',MSVFORMFONT>
%       [sn,cn] = add(a(1:n),b(1:n),ci,q);         % niedere n Bit                  %<'FontName',MSVFORMFONT>
%       [s0,c0] = add(a(n+1:end),b(n+1:end),0,q);  % hhere Bit fr Carryin = 0     %<'FontName',MSVFORMFONT>
%       [s1,c1] = add(a(n+1:end),b(n+1:end),1,q);  % hhere Bit fr Carryin = 1     %<'FontName',MSVFORMFONT>
% 
%       % Selektion des richtigen Ergebnisses                                       %<'FontName',MSVFORMFONT>
%       % ===================================                                       %<'FontName',MSVFORMFONT>
%       if cn==0                                                                    %<'FontName',MSVFORMFONT>
%          s = [sn s0];                                                             %<'FontName',MSVFORMFONT>
%          c = c0;                                                                  %<'FontName',MSVFORMFONT>
%       else                                                                        %<'FontName',MSVFORMFONT>
%          s = [sn s1];                                                             %<'FontName',MSVFORMFONT>
%          c = c1;                                                                  %<'FontName',MSVFORMFONT>
%       end                                                                         %<'FontName',MSVFORMFONT>
%
