%
%
%5.2.2  Lineare Block-Codes%<'FontWeight',MSVTITLWEIGHT,'FontSize',MSVTITLSIZE-8,'Color',MSVTITLCOLOR>
%
%
%
% Ein linearer Block-Code mit  {\fontname{Courier}k}  Informationsbits und  {\fontname{Courier}n} Codewortbits wird als binrer 
% {\fontname{Courier}(n,k)}-Code bezeichnet.  Der lineare Coderaum wird aufgespannt durch  {\fontname{Courier}k}  linear unabhngige 
% Basisvektoren von je  {\fontname{Courier}n}  Bits.  Man fasst sie zusammen in der sog. Generatormatrix  {\fontname{Courier}G} . 
% Aus einem Informationsvektor  {\fontname{Courier}u}  ergibt sich das Codewort durch 
%
%      x = u * G   %<'FontName',MSVFORMFONT>
%
% wo die Matrixmultiplikation im endlichen Krper mit den Elementen  {\fontname{Courier}0}  und  {\fontname{Courier}1}  ({\fontname{Courier}GF(2)})  
% durchzufhren ist.  Die  {\fontname{Courier}GF(2)}-Arithmetik ergibt sich durch Ganzzahlarithmetik plus 
% modulo-2-Rechnung.  
%
%% Die Arithmetik endlicher Krper wird im Kapitel 6 ausfhrlich behandelt.  Wer dieses nicht 
%% beherrscht, sollte zunchst diesen Abschnitt berspringen.
%
% Die berprfung eines empfangenen Wortes  {\fontname{Courier}y}  auf Fehler erfolgt durch Multiplikation mit 
% einer Prfmatrix  {\fontname{Courier}H} :
%
%      s = y * H   %<'FontName',MSVFORMFONT>
%
% Im Fehlerfreien Fall ist  {\fontname{Courier}s}  der Nullvektor.  Andernfalls gibt  {\fontname{Courier}s}  Hinweise auf die Position 
% von Fehlern.
%
% Die Prfmatrix ergibt sich wie folgt:
% Der Coderaum ist ein  {\fontname{Courier}k}-dimensionaler Unterraum des  {\fontname{Courier}n}-dimensionalen  {\fontname{Courier}GF(2)^n} , der von 
% den  {\fontname{Courier}k} Zeilenvektoren der Generatormatrix aufgespannt wird.  Zum Aufspannen des  {\fontname{Courier}GF(2)^n}  
% kann man diese Basisvektoren und weitere  {\fontname{Courier}n-k}  zu diesen orthogonale Vektoren.  Letztere sind 
% die Zeilen von  {\fontname{Courier}H'} .  Wegen der Orthogonalitt gilt dann
%
%      G * H = 0   %<'FontName',MSVFORMFONT>
%
% D.H. jede Linearkombination der Zeilen von  {\fontname{Courier}G} , also jedes Codewort, mit  {\fontname{Courier}H}  multipliziert ergibt {\fontname{Courier}0} .
%
%
% \bfBeispiele
%
% (1)  \bf(5,4)-Parittscode
%
% Eine Generatormatrix fr den Parittscode fr  {\fontname{Courier}k = 4, n = 5}  ist
%
%     G = (   1     0     0     0     1  )   %<'FontName',MSVFORMFONT>
%         (   0     1     0     0     1  )   %<'FontName',MSVFORMFONT>
%         (   0     0     1     0     1  )   %<'FontName',MSVFORMFONT>
%         (   0     0     0     1     1  )   %<'FontName',MSVFORMFONT>
%
% In Matlab:
%
k = 4;
G = [eye(k),ones(k,1)];    result2fig(G)
%
% Mit dem Informationsvektor  {\fontname{Courier}u = [ 1 1 0 1 ]}  ergibt sich sofort das Codewort:
%
%     x = u * G = ( 1 1 0 1 ) * ( 1  0  0  0  1 ) = ( 1  1  0  1  1 )   %<'FontName',MSVFORMFONT>
%                               ( 0  1  0  0  1 )                       %<'FontName',MSVFORMFONT>
%                               ( 0  0  1  0  1 )                       %<'FontName',MSVFORMFONT>
%                               ( 0  0  0  1  1 )                       %<'FontName',MSVFORMFONT>
%
% In Matlab:
% ( Die Funktion  {\fontname{Courier}mod2(x)}  berechnet  {\fontname{Courier}mod( round(x) , 2 )} 
% und realisiert damit die Arithmetik im GF(2) )
%
u = [ 1 1 0 1 ];
x = mod2( u*G );    result2fig(x)
%
% Der Parittscode ist systematisch, denn die ersten vier Bits des Codewortes sind die Information.
%
% Die Prfmatrix ist
%
%      H = ( 1 )   %<'FontName',MSVFORMFONT>
%          ( 1 )   %<'FontName',MSVFORMFONT>
%          ( 1 )   %<'FontName',MSVFORMFONT>
%          ( 1 )   %<'FontName',MSVFORMFONT>
%
% In Matlab:
%
H = ones(k+1,1);     result2fig(H)
%
% Die Prfung unseres Codewortes  {\fontname{Courier}x}  muss   {\fontname{Courier}s = 0}  ergeben:
%
%      s = x * H = ( 1  1  0  1  1 ) * ( 1 ) = 0    %<'FontName',MSVFORMFONT>
%                                      ( 1 )        %<'FontName',MSVFORMFONT>
%                                      ( 1 )        %<'FontName',MSVFORMFONT>
%                                      ( 1 )        %<'FontName',MSVFORMFONT>
%
% In Matlab:
%
s = mod( x*H ,2);    result2fig(s)
%
% Der Parittscode wird blicherweise im letzten Bit modifiziert.  Man addiert 
% einen Vektor dessen Komponenten der Reihe nach angeben, ob das jeweilige Bit zu
% modifizieren ist.   In diesem Fall ist also im GF(2) folgender Vektor zu addieren:
%
%      d = ( 0  0  0  0  1 )     %<'FontName',MSVFORMFONT>
%
% Die Codierungsvorschrift lautet damit:
%
%     x = u * G + d = ( 1 1 0 1 ) * ( 1 0 0 0 1 ) + ( 0 0 0 0 1 ) = ( 1 1 0 1 0 ) %<'FontName',MSVFORMFONT>
%                                   ( 0 1 0 0 1 )                       %<'FontName',MSVFORMFONT>
%                                   ( 0 0 1 0 1 )                       %<'FontName',MSVFORMFONT>
%                                   ( 0 0 0 1 1 )                       %<'FontName',MSVFORMFONT>
%
% In Matlab:
%
d = [ 0 0 0 0 1 ];
x = mod( u*G + d , 2);   result2fig(x)
%
% Der Minimalabstand des Parittscodes ist   {\fontname{Courier}D = 2} .   Es ist also keine Fehlerkorrektur, 
% aber eine Erkennung von Einbitfehlern mglich.
%
%
% (2)  \bf(3,1)-Wiederholungscode
%
% Eine Generatormatrix fr den  {\fontname{Courier}(3,1)}-Wiederholungscode ist
%
%     G = ( 1  1  1 )
%
% Mit dem Informationsvektor  {\fontname{Courier}u} , der in diesem Fall nur ein Bit enthlt, ergibt sich sofort 
% das Codewort:
%
% a.  u = 0 :    x = ( 0 ) * ( 1  1  1 ) = ( 0  0  0 )   %<'FontName',MSVFORMFONT>
% b.  u = 1 :    x = ( 1 ) * ( 1  1  1 ) = ( 1  1  1 )   %<'FontName',MSVFORMFONT>
%
% Der  {\fontname{Courier}(n,k)}-Wiederholungscode ist systematisch.  Sein Minimalabstand   {\fontname{Courier}D = n} . 
% Es ist also eine Korrektur von bis zu  {\fontname{Courier}floor((n-1)/2)}  Bits (hier also  {\fontname{Courier}1} Bit) 
% oder die Erkennung von  {\fontname{Courier}(n-1)}-Bitfehlern mglich.  Die Korrektur erfolgt einfach 
% durch Mehrheitsentscheid.
%
% In Matlab:
%
n = 3;
G = ones(1,n);    result2fig(G)    % Generatormatrix
%
u = [0];
x = mod2( u*G );  result2fig(x)    % Codewort 0
%
% bzw.
%
u = [1];
x = mod2( u*G );  result2fig(x)    % Codewort 1
%
% Mehrheitsentscheid:
%
u = randb(1);                       % zuflliges Bit
x = u*G;              result2fig(x) % Codewort
f = [0 1 0];          result2fig(f) % zweites Bit wird falsch
y = mod2( x + f );    result2fig(y) % Fehler anbringen
z = sum(y)>sum(y==0); result2fig(z) % Korrektur durch Mehrheitsentscheid
%
%
% (3)  \bf(7,4)-Hammingcode
%
% Eine Generatormatrix fr den (7,4)-Hammingcode ist
%
%     G = ( 1 0 0 0 1 0 1 )   %<'FontName',MSVFORMFONT>
%         ( 0 1 0 0 1 1 1 )   %<'FontName',MSVFORMFONT>
%         ( 0 0 1 0 1 1 0 )   %<'FontName',MSVFORMFONT>
%         ( 0 0 0 1 0 1 1 )   %<'FontName',MSVFORMFONT>
%
% In Matlab:
%
G = [1 0 0 0 1 0 1; 0 1 0 0 1 1 1; 0 0 1 0 1 1 0; 0 0 0 1 0 1 1];
result2fig(G)
%
% Mit dem Informationsvektor  {\fontname{Courier}u}  ergibt sich sofort das Codewort. 
%
%%  Geben Sie den zu kodierenden Vektor von 4 Informationsbits an:
%
u = inputbinmat('Zeilenvektor u (4 Binrstellen)')
x = mod( u*G ,2);    result2fig(x)
%
% Dieser Hamming-Code ist systematisch, denn die ersten vier Bits des Codewortes sind 
% die Information.  Dies liegt daran, dass die k*n-Generatormatrix links mit einer 
% k*k-Einheitsmatrix beginnt.
%
% Der Hamming-Code wird meistens durch ein Parittsbit ergnzt und hufig modifiziert. 
%
% Die Prfung auf Fehler erfolgt durch die Berechnung des Syndroms  {\fontname{Courier}s}  durch Multiplikation 
% mit der Matrix 
%
%     H = ( 1  0  1 )   %<'FontName',MSVFORMFONT>
%         ( 1  1  1 )   %<'FontName',MSVFORMFONT>
%         ( 1  1  0 )   %<'FontName',MSVFORMFONT>
%         ( 0  1  1 )   %<'FontName',MSVFORMFONT>
%         ( 1  0  0 )   %<'FontName',MSVFORMFONT>
%         ( 0  1  0 )   %<'FontName',MSVFORMFONT>
%         ( 0  0  1 )   %<'FontName',MSVFORMFONT>
%
% In Matlab (hier gleich allgemein fr alle systematischen Hammingcodes):
%
[k,n] = size(G);
H = [G(:,k+1:n); eye(n-k)];    result2fig(H)
%
% Im Falle eines fehlerfreien Codewortes ist  {\fontname{Courier}s = 0} :
%
s = mod2( x*H );    result2fig(s)
%
% Bei Einbitfehlern ist das Syndrom  {\fontname{Courier}s}  gleich der Spalte, deren Nummer die Stelle des 
% Fehlers im empfangenen Wort angibt. 
%
%% Geben Sie den 4-stelligen Informationsvektor und einen 7-stelligen Fehlervektor an, 
%% der die Fehlerstellen durch Einsen markiert, fr Fehler an der 3. Stelle also 0010000:
%
G = [1 0 0 0 1 0 1; 0 1 0 0 1 1 1; 0 0 1 0 1 1 0; 0 0 0 1 0 1 1];
H = [G(:,5:7); eye(3)];
u = inputbinmat('Informationsvektor u (4 Binrstellen)') % Information
x = mod( u*G ,2)                   % Codewort
f = inputbinmat('Fehlervektor f (7 Binrstellen)')       % Fehlervektor
y = mod2( x+f )                    % Bitfehler anbringen
s = mod2( y*H )                    % Syndrom
b = 1;                             % Schleifenanfangswert
z = y;                              
if any(s) while any(s~=H(b,:)) b = b+1; end; end  % Suche nach Identitt der Zeile
if any(s) z(b) = mod2( y(b)+1 ); end              % Korrektur
v = z(1:4);   result2fig(v)                       % die dekodierte Information
%
% Dieses Skript ist Kern der Funktion  {\fontname{Courier}hammingcode(u,f)}.  Als Default generiert 
% diese Funktion zufllige Bitmuster fr die Information  {\fontname{Courier}u}  (4 Bits) und fr 
% den Fehlervektor (7 Bits mit Fehlerwahrscheinlichkeit  {\fontname{Courier}p = 0.1} ):
%
demohammingcode
%
demohammingcode( [0 0 1 1], 3 )
%
demohammingcode( [0 0 1 1], [3 5] )
%
%% Prfen Sie den Hammingcode mit folgenden Anzahlen von Bitfehlern:  0, 1, 2, 3.
%
%
% (4)  \bfWeitere Hammingcodes
%
% Die Generatormatrizen und Prfmatrizen fr binre systematische (n,k)-Hamming-Codes 
% werden durch die Funktion  {\fontname{Courier}hamminggenerator(r,m)}  erzeugt (  {\fontname{Courier}n = 2^r - 1} ; {\fontname{Courier}k = n - r} ).  
% Das Prinzip ist sehr einfach:  
% Man bildet eine binre Matrix  {\fontname{Courier}A} , deren Zeilen smtliche Binrmuster mit wenigstens 
% zwei Einsen enthlt.  Dieser Matrix wird eine Einheitsmatrix passender Gre links 
% vorangestellt.  {\fontname{Courier}H}  wird entsprechend generiert, so dass   {\fontname{Courier}G*H = 0} .
%
hamminggenerator(4,0);
%
hamminggenerator(4,1);
%
% In Matlab kann man auch gleich prfen, ob  {\fontname{Courier}G*H = 0}  erfllt ist:
%
r = 4;
G = hamminggenerator(r,0);  result2fig(G)
H = hamminggenerator(r,1);  result2fig(H)
GxH = mod2( G*H );          result2fig(GxH)
%
% Mit  {\fontname{Courier}r = 2}  ergibt sich der  {\fontname{Courier}(3,1)}-Wiederholungscode, der also auch zu den 
% Hamming-Codes gezhlt werden kann.
%
%
% (5)  \bfCyclic Redundancy Check (CRC)
%
% Eine beliebte Methode zur Erkennung von Fehlern in Blcken von  {\fontname{Courier}m}  Binrwrtern der 
% Breite  {\fontname{Courier}n}  Bit sind systematische zyklische Codes.  Diese haben den entscheidenden 
% Vorteil der Generierung und Prfung durch sehr einfache lineare rckgekoppelte 
% Schieberegister deren formale Behandlung auf Polynomarithmetik beruht.  Eine 
% MATLAB-Beschreibung, die die Schieberegister-Realisierung simuliert, liegt in  {\fontname{Courier}crc} 
% vor (siehe deren Rumpf fr die einfache Realisierung).  Diese Funktion wird 
% benutzt in der Funktion  {\fontname{Courier}democrc(f)} , die zufllige Informationsvektoren 
% der Lnge 512 generiert, diese mit dem CRC codiert und nach der Invertierung 
% der im Parameter  f  angegebenen Stellen des Codewortes wieder decodiert.  
% Zurckgegeben wird der boolesche Wert, ob ein Fehler erkannt wurde:
%
democrc([]);      % keine Bitfehler: leerer Fehlerindexvektor
%
democrc(1+ceil(527*rand(1,10)));   % 10 Bitfehler
%
democrc(1+ceil(527*rand(1,200)));  % 200 Fehlerindices
%
% Die Fehlerindices werden hier als Zufallszahlen generiert und knnen deshalb 
% mehrfach auftreten.  Daher ist die Gesamtzahl der Fehlerstellen u.U. kleiner 
% als die Anzahl der Fehlerindices.
%
% Falls ein Bndelfehler der Lnge  {\fontname{Courier}L}  auftritt (und sonst kein Fehler),  und  {\fontname{Courier}g}  der 
% Grad des Polynoms ist (gleich der Anzahl der Bits des CRC), so gilt fr die 
% Wahrscheinlichkeit  {\fontname{Courier}P}  der Entdeckung durch den CRC:
%
%     L \leq g     :     P = 1.0         %<'FontName',MSVFORMFONT>
%     L = g + 1 :     P = 2^{-(g+1)}  %<'FontName',MSVFORMFONT>
%     L > g + 1 :     P = 2^{-g}      %<'FontName',MSVFORMFONT>
%
% Der zuunterst genannte Fall ist der allgemeine Fall von ber die gesamte Sequenz 
% verteilten Fehlern.
%
%
% (6)  \bfZyklische-Codes
%
% Zyklische Codes eignen sich besonders zur Erkennung von Bndelfehlern.  Es gibt aber auch 
% viele zyklische Codes zur Fehler-Korrektur und zur Bndelfehler-Korrektur.  In der Literatur 
% findet man dann anstelle der Generatormatrizen die Generatorpolynome.  Eine Umrechnung 
% kann man vornehmen mit den Funktionen  {\fontname{Courier}genpoly2genmat}  bzw.  {\fontname{Courier}checkpoly2checkmat} .
% Ein perfekter Dreibit-Fehlerkorrigierender zyklischer (23,12)-Code ist z.B. der Golay-Code 
% mit dem Generatorpolynom
%
%     g(x) = x^{11} + x^{10} + x^6 + x^5 + x^4 + x^2 + 1   %<'FontName',MSVFORMFONT>
%
% Eine Generatormatrix ergibt sich nach:
%
genpoly2genmat(23,12,[1 1 0 0 0 1 1 1 0 1 0 1]);  % Golay-Generator
%
%
% (7)  \bfRandom-Block-Codes
%
% Ein Code ist umso besser je mehr Bits des Codewortes von jeweils einem Bit der 
% Information abhngen.  Die Generatormatrix sollte im Idealfall also in jeder Zeile
% etwa gleich viele Einsen und Nullen haben.  Das legt nahe, eine zufllige Matrix zu 
% verwenden.  Allerdings steht man dann vor dem Problem der Decodierung.  Wir 
% beschrnken uns auf die Codierung von  {\fontname{Courier}k\leq12}  Informationsbits.  Dann kann man 
% noch gut eine vollstndige Suche nach dem kleinsten euklidischen Abstand des 
% empfangenen Wortes von smtlichen  {\fontname{Courier}2^k}  mglichen Codewrtern durchfhren. 
% Die Decodierung durch Suche des kleinsten Abstandes wird erlutert durch das sehr 
% einfache Beispiel eines binren 3-Bit-Codewortes, welches man als Ecke eines 
% Wrfels ansehen kann:
%
demosoftdecode
%
% Die Bits des Codewortes werden durch Addition normalverteilter Werte gestrt. 
% Als wahrscheinlichstes Codewort whlt man dasjenige, welches im 3-D-Raum den 
% kleinsten Abstand vom empfangenen Wort hat.  Bei lngeren Codewrtern ist der 
% Abstand im entsprechend hherdimensionalen Raum zu berechnen.
%
% Wir ermitteln die Verteilung der Distanzen durch Simulation mit der Funktion  
% {\fontname{Courier}decoddemo} :
%
help decoddemo
%
% Zunchst ein Versuch mit einem zuflligen {\fontname{Courier}(32,8)}-Code ohne Rauschen:
%
decoddemo( randb([8,32]), randb(8), 0)
%
% Und nun mit Rauschen:
%
decoddemo( randb([8,32]), randb(8), 0.5)
%
% Hinweise zur erwarteten Verteilung:
% (1)  Die kumulative Verteilung weicht aufgrund des zuflligen Informationsmusters und der 
% zuflligen Wahl von der erwarteten Verteilung ab.  Mehrfaches Aufrufen wird die erwartete 
% Kurve besttigen.
% (2)  Der Mittelwert des Abstandsquadrates fr einen zuflligen binren {\fontname{Courier}(n,k)}-Code ist ohne 
% Rauschen  {\fontname{Courier}(n/2) * (Signalabstand)^2} .  Bei Rauschen mit der Standardabweichung  {\fontname{Courier}\sigma}
% kommt der Erwartungswert  {\fontname{Courier}n*\sigma^2}  der  {\fontname{Courier}\chi^2}-Verteilung hinzu.
% (3)  Die Varianz des Abstandsquadrates ist  {\fontname{Courier}(n/2)*(Signalabstand)^2 + 2*n*\sigma^2} .
%
% Hier ein Versuch mit einem {\fontname{Courier}(2048,8)}-Code:
%
decoddemo( randb([8,2048]), randb(8), 1.0)
%
% Auch bei starkem Rauschen ist die Decodierung noch sehr sicher:
%
decoddemo( randb([8,2048]), randb(8), 3.0)
%
% Die ertrgliche Grenze der Decodierzeit ist etwa erreicht mit dem  {\fontname{Courier}(12,1024)}-Code:
%
decoddemo( randb([12,1024]), randb(12), 4.0)
%
% Die Codierung ist bei diesen Codes nicht sehr aufwndig.  Das Problem jedoch ist 
% die Decodierung.  Nur mit einer vollstndigen Suche findet man das am besten 
% passende Codewort.  Fr groe  {\fontname{Courier}k}  kommt daher eine vollstndige Suche nicht 
% in Betracht.  Unter einem guten Code versteht man im allgemeinen einen Code, der 
% mit ertrglichem Decodieraufwand auch bei relativ groem Rauschen eine kleine 
% Fehlerrate ergibt.
%
% Fr kleine Codewortlngen sind zufllig generierte Codes meistens nicht optimal.  
% Wir vergleichen als Bespiel den oben besprochenen  {\fontname{Courier}(23,12)}-Golay-Code, dessen 
% Minimalabstand  {\fontname{Courier}7}  betrgt,  mit zufllig generierten  {\fontname{Courier}(23,12)}-Codes, deren 
% Minimalabstand so gut wie immer deutlich kleiner als  {\fontname{Courier}7}  ist:
%
% Golay-Code:
decoddemo( genpoly2genmat(23,12,[1 1 0 0 0 1 1 1 0 1 0 1]), randb(12), 0)
%
% Random-Code:
decoddemo( randb([12,23]), randb(12), 0)
%
% Wiederholt man diese Aufrufe mehrfach mit  {\fontname{Courier}\sigma = 0.7} , so wird man feststellen, 
% dass zwar der Golay-Code wegen der optimalen Anordnung der Codewrter mit 
% Abstand  {\fontname{Courier}7}  besser ist, aber nicht so wesentlich, wie man vielleicht vermutet hat.
%
%
%% Aufgabe 5.2   Der  {\fontname{Courier}(15,11)}-Hamming-Code soll mit dem Parittscode erweitert 
%% werden zu einem systematischen  {\fontname{Courier}(16,11)}-Code.  Der Code lsst sich mit der 
%% Funktion  {\fontname{Courier}hamminggenerator}  erzeugen.
%% Geben Sie  {\fontname{Courier}G}  und  {\fontname{Courier}H}  an.
%