%
%
%7.4.1  Bewertung des Realisierungsaufwandes%<'FontWeight',MSVTITLWEIGHT,'FontSize',MSVTITLSIZE-12,'Color',MSVTITLCOLOR>
%
%
% Beim Realisierungsaufwand unterscheiden wir
%
% (a)  {\bfmaterielle Kosten}
%
% (b)  {\bfzeitliche Kosten}.
%
% Materielle Kosten fallen an als Entwurfskosten, und Herstellungskosten sowie 
% als Betriebskosten durch Energieverbrauch einschlielich Khlung.  Die realen 
% Zusammenhnge sind ziemlich komplex.  Wir benutzen deshalb zwei einfache 
% Modelle.  Danach werden die materiellen Kosten bewertet durch
%
% {\bfM1}:  die Anzahl der benutzten Verknpfungsglieder
%
% {\bfM2}:  die Gesamtzahl aller Verknpfungseingnge
%
% Dabei zhlen Verknpfungen der Art
%   2^n        2^n        %<'FontName',MSVFORMFONT>
%  {\fontname{Courier} _{\fontsize{26}\cup}}    und    {\fontname{Courier}_{\fontsize{26}\cap}}
%  i=1       i=1        %<'FontName',MSVFORMFONT>
% jeweils als  {\bfein}  Verknpfungsglied mit  {\fontname{Courier}n}  Eingngen.  Invertierungen zhlen nicht.
%
% Zeitliche Kosten fallen zur Betriebszeit als Verarbeitungszeit der Verknpfungsglieder 
% an.  Die Gesamtverarbeitungszeit einer Schaltfunktion ergibt sich als Summe der 
% Verarbeitungszeiten der Verknpfungen in der Verschachtelung des schaltalgebraischen 
% Ausdrucks der Funktion.
%
% Die Verarbeitungszeit von Verknpfungsgliedern ist stark von der Technologie abhngig.
% Wir benutzen zwei einfache Modelle:
%
% {\bfZ1}:  Alle Verknpfungen sind gleichschnell
%
% {\bfZ2}:  Die Verarbeitungszeit einer Verknpfung ist proportional zur 
%        Anzahl der Eingnge
%
% {\bfM1}  und  {\bfZ1}  gelten ungefhr bei Bipolar-Technologien (TTL,ECL).  
% {\bfM2}  und  {\bfZ2}  gelten ungefhr bei MOS-Technologien.
%
% Im Gegensatz zu den materiellen Kosten, die immer anfallen, spielen die zeitlichen 
% Kosten einer Schaltfunktion innerhalb eines Gesamtsystems nur dann eine Rolle,
% wenn die Funktion im sog. (Zeit-) {\bfkritischen Pfad}  liegt.
%
% Neben den genannten Kosten knnen weitere - hufig gegenlufige - Gesichtspunkte 
% bei der Realisierung von Bedeutung sein wie Testbarkeit und Energieverbrauch.  Sie
% werden an dieser Stelle noch nicht betrachtet.
%
% \bfBeispiel
%
% Die selbe Schaltfunktion in zwei verschiedenen Ausdrcken:
%
% u\capv \cup u\capw\capx  \rightarrow  M1 = 3;  M2 = 7;  Z1 = 2;  Z2 = 5  %<'FontName',MSVFORMFONT>
%
% u\cap(v \cup w\capx)   \rightarrow  M1 = 3;  M2 = 6;  Z1 = 3;  Z2 = 6  %<'FontName',MSVFORMFONT>
% 
%
% Der Realisierungsaufwand nach diesen Modellen kann direkt aus dem schaltalgebraischen 
% Ausdruck berechnet werden.  Die Funktion  {\fontname{Courier}aufwand(binexpr)}  tut dies fr den Fall  M2:
% 
hilfe( 'aufwand' )
%
% Eine Zhlung der logischen Verknpfungen entsprechend dem Modell {\bfM1/Z1} wird durchgefhrt mit
%
aufwand({'h=~a|(b&c)'; 'x=h|a'; 'y=h&d'}, 'M1' );  % Gatterzahl und Schaltzeit
%
% Fr das Modell {\bfM2/Z2} ergibt sich bei dem selben Schaltnetz
%
aufwand({'h=~a|(b&c)'; 'x=h|a'; 'y=h&d'}, 'M2' );  % Inputzahl und Schaltzeit 
%
% Der tatschliche Realisierungsaufwand ist stark von der Technologie und vom Entwurfsstil 
% abhngig.  Eine brauchbare Aufwandsabschtzung fr Gate-Arrays in CMOS liefert z.B.
%
aufwand( {'h=~a|(b&c)'; 'x=h|a'; 'y=h&d'} ,...
         [2 0 2.5 4 2; 2 0 2.5 4 2; 0 0 4 4 4]);    % #Transistoren und Schaltzeit 
%
% Die folgenden Abschnitte behandeln nur eine graphische Methode zur Ermittlung einer 
% zweistufigen disjunktiven bzw. konjunktiven Form.  Von Quine und McCluskey wurde die 
% Methode als Algorithmus zur Rechner-gesttzten Minimierung weiterentwickelt.  Hufig 
% findet man bei relativ einfachen Ausdrcken aber noch bessere Lsungen durch intuitive 
% direkte Umwandlung.
%
% \bfBeispiel
%
aufwand( '(a&~b&~c) | (a&b&~c) | (a&~b&c)' , 'M1' )
%
% Hieraus folgt intuitiv mit dem Distributivgesetz:
%  = a&((~b | b)&c | ~b&c)  =  a&(c | ~b&c)  %<'FontName',MSVFORMFONT>
% und weiter durch Anwendung der deMorgan-Gesetze der Ausdruck  {\fontname{Courier}a&~(c&~(~b&c))}  , also:
%
aufwand( 'a&~(c&~(~b&c))' , 'M1' )
%
% Wir prfen noch die quivalenz der beiden Ausdrcke:
%
compareexpression('(a&~b&~c)|(a&b&~c)|(a&~b&c)','a&~(c&~(~b&c))',{'a';'b';'c'})
%