function taktdemo(mode,sig)
% taktdemo(mode,sig)
% Interaktive Graphik zu taktgesteuerten Zeitgliedern.
% mode='+' :  Vorderflankensteuerung
% mode='-' :  Rckflankensteuerung
% mode='p' :  Pegelsteuerung
% mode='2' :  Zweiflankensteuerung
% sig      :  Vektor des Datensignals (Default: Zufallsmuster)

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

% Konstanten und Defaults
% =======================
if nargin<2  sig = nfach(randb(32),100); end
if nargin<1 mode = '-'; end
lw = 2.5;
cl = [0 0 1; 1 0 0; 0 1 0];
nam = {'Daten', 'Takt', 'Ausgabe'};
n = length(sig);
switch mode
   case '+', fignam = 'Vorderflankensteuerung';
   case '-', fignam = 'Rckflankensteuerung';
   case 'p', fignam = 'Pegelsteuerung';
   case '2', fignam = 'Zweiflankensteuerung';
end
   
% Graphik
% ======= 
fh = fullscreen(fignam,'w',0,[0 1 0.7 1]);

xd = axes('Position',[0 0.7 1 0.2]);                % Daten
drawbinsig(sig,0,1,'LineWidth',lw,'Color',cl(1,:));
line([0 n],[0 0],'Color',0.7*[1 1 1])
text(0.015*n,0.5,nam{1},'FontSize',18)
set(xd,'XLim',[0 n]); axis off

xc = axes('Position',[0 0.4 1 0.2]);                % Takt
ch = line(0,0,'LineWidth',lw,'Color',cl(2,:));
line([0 n],[0 0],'Color',0.7*[1 1 1])
text(0.015*n,0.5,nam{2},'FontSize',18)
set(xc,'XLim',[0 n]); axis off

xq = axes('Position',[0 0.1 1 0.2]);                % Ausgang
qh = line(0,0,'LineWidth',lw,'Color',cl(3,:));
line([0 n],[0 0],'Color',0.7*[1 1 1])
text(0.015*n,0.5,nam{3},'FontSize',18)
set(xq,'XLim',[0 n]); axis off

xt = axes('Position',[0 0 1 1]);                    % Zeit
zh = line([0 0],[0 1],'Color',0.7*[1 1 1],'LineWidth',lw)
axis([0 n 0 1]); axis off

set(fh,'UserData',0)
set(fh,'WindowButtonDownFcn','set(gcf,''UserData'',1)')
set(fh,'WindowButtonUpFcn',  'set(gcf,''UserData'',0)')

% Animation und Interaktion
% =========================
pause(0.5)
q  = rand>0.5;
qq = q;
cx = 0;
cy = 0;
cc = 0;
qx = 0;
qy = q;
for k=1:n
   c = get(fh,'UserData');
   switch mode
   case {'+','-'}
      if c~=cc
         cx = [cx k  k];
         cy = [cy cc c];
         cc = c;
         if cc==(mode=='+')
            q  = sig(k);
            qx = [qx k k];
            qy = [qy qy(end) q];
         end
      end
   case 'p'
      if c~=cc
         cx = [cx k  k];
         cy = [cy cc c];
         cc = c;
      end
      if c
         q = sig(k);
         if q~=qq
            qx = [qx k k];
            qy = [qy qy(end) q];
            qq = q;
         end
      end
   case '2'
      if c~=cc
         cx = [cx k  k];
         cy = [cy cc c];
         cc = c;
         if cc  qi = sig(k); end
         if ~cc
            q  = qi;
            qx = [qx k k];
            qy = [qy qy(end) q];
         end
      end
   end
   set(zh,'XData',k*[1 1])
   set(ch,'XData',[cx k],'YData',[cy cc])
   set(qh,'XData',[qx k],'YData',[qy  q])
   drawnow
end

% Interaktion nach der Animation
% ==============================
set(fh,'WindowButtonDownFcn','set(gcf,''Pointer'',''fullcrosshair'')')
set(fh,'WindowButtonUpFcn',  'set(gcf,''Pointer'',''arrow'')')
